=== modified file 'NEWS'
--- NEWS	2010-05-06 07:48:22 +0000
+++ NEWS	2010-05-19 02:20:50 +0000
@@ -13,6 +13,22 @@
 Compatibility Breaks
 ********************
 
+* An API break has been made to the lock_write method of ``Branch`` and
+  ``Repository`` objects; they now return ``branch.BranchWriteLockResult``
+  and ``repository.RepositoryWriteLockResult`` objects. This makes
+  changing the API in future easier and permits some cleaner calling code.
+  The lock_read method has also changed from having no defined return
+  value to returning ``LogicalLockResult`` objects.
+  (Robert Collins)
+
+* ``bzr`` does not try to guess the username as ``username@hostname``
+  and requires it to be explictly set. This can be set using ``bzr
+  whoami``.
+  (Parth Malwankar, #549310)
+
+* ``bzrlib.commands.Command`` will now raise ValueError during
+  construction if there is no __doc__ set. (Robert Collins)
+
 New Features
 ************
 
@@ -30,6 +46,21 @@
   pack operation.
   (Parth Malwankar, #304320)
 
+* New command line option ``--authors`` to ``bzr log`` allows users to
+  select which of the apparent authors and committer should be
+  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
+
+* Support ``--directory`` option for a number of additional commands:
+  added, annotate, bind, cat, cat-revision, clean-tree, deleted,
+  export, ignore, ignored, lookup-revision, ls, modified, nick,
+  re-sign, unbind, unknowns.
+  (Martin von Gagern, #527878)
+
+* The bash_completion plugin from the bzr-bash-completion project has
+  been merged into the tree. It provides a bash-completion command and
+  replaces the outdated ``contrib/bash/bzr`` script with a version
+  using the plugin. (Martin von Gagern, #560030)
+
 Bug Fixes
 *********
 
@@ -38,17 +69,31 @@
   better with sudo.
   (Martin <gzlist@googlemail.com>, Parth Malwankar, #376388)
 
+* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
+  support of bzr-externals and scmproj plugins.
+  (Alexander Belchenko, bug #572098)
+
+* ``bzr ignore`` will no longer add duplicate patterns to .bzrignore.
+  (Gordon Tyler, #572092)
+
 * ``bzr log --exclude-common-ancestry -r X..Y`` displays the revisions that
   are part of Y ancestry but not part of X ancestry (aka the graph
   difference).
   (Vincent Ladeuil, #320119)
 
+* ``bzr rm`` should not refuse to delete directories which contained a file
+  which has been moved elsewhere in the tree after the previous commit.
+  (Marius Kruger, Daniel Watkins, #129880)
+
 * ``bzr selftest --parallel=fork`` wait for its children avoiding zombies.
   (Vincent Ladeuil, #566670)
 
 * ``bzr selftest`` should not use ui.note() since it's not unicode safe.
   (Vincent Ladeuil, #563997)
 
+* CommitBuilder refuses to create revisions whose trees have no root.
+  (Aaron Bentley)
+
 * Don't mention --no-strict when we just issue the warning about unclean trees.
   (Vincent Ladeuil, #401599)
 
@@ -56,13 +101,38 @@
   versions before 1.6.
   (Andrew Bennetts, #528041)
 
+* Heavyweight checkout operation now shows a message to the user indicating
+  history is being copied.
+  (Parth Malwankar, #538868)
+
 * Reduce peak memory by one copy of compressed text.
   (John Arbash Meinel, #566940)
 
+* Selftest was failing with testtools 0.9.3, which caused an
+  AssertionError raised from a cleanUp to be reported as a Failure, not an
+  Error, breaking on of our test hygiene tests.
+  (Robert Collins, Vincent Ladeuil).
+
+* ``set_user_option`` with a dict on remote branches no longer fails with
+  an AttributeError.  There is a new ``Branch.set_config_option_dict`` RPC
+  to support this efficiently.
+  (Andrew Bennetts, #430382)
+  
 * Show the filenames when a file rename fails so that the error will be
   more comprehensible.
   (Martin Pool, #491763)
 
+* Unicode characters in aliases are now handled correctly and do not cause
+  UnicodeEncodeError exception. (Parth Malwankar, #529930)
+
+* Unicode commit messages that are the same as a file name no longer cause
+  UnicodeEncodeError. ``ui.text.show_warning`` now handles unicode
+  messages.
+  (Parth Malwankar, #563646)
+
+* Using bzr with `lp:` urls behind an http proxy should work.
+  (Robert Collins, #558343)
+
 Improvements
 ************
 
@@ -74,6 +144,10 @@
   (case-sensitive) as false.
   (Brian de Alwis, Vincent Ladeuil)
 
+* ``bzr ls`` now supports short options for existing long options.
+  ``-k/--kind``, ``-i/--ignored``, ``-u/--unknown`` and ``-0/--null``.
+  (Parth Malwankar, #181124)
+
 * ``Config.get_user_option_as_bool`` will now warn if a value cannot
   be interpreted as a boolean.
   (Vincent Ladeuil)
@@ -96,9 +170,32 @@
   implementations.
   (Martin Pool)
 
+* Lock methods on ``Tree``, ``Branch`` and ``Repository`` are now
+  expected to return an object which can be used to unlock them. This reduces
+  duplicate code when using cleanups. The previous 'tokens's returned by
+  ``Branch.lock_write`` and ``Repository.lock_write`` are now attributes
+  on the result of the lock_write. ``repository.RepositoryWriteLockResult``
+  and ``branch.BranchWriteLockResult`` document this. (Robert Collins)
+
+* ``Repository.refresh_data`` may now be called in a write group on
+  pack-based repositories.  Older repositories will still raise an error
+  in this case.  Subclasses of ``Repository`` can still override
+  ``Repository._refresh_data``, but are now responsible for raising
+  ``bzrlib.repository.IsInWriteGroupError`` if they do not support
+  ``refresh_data`` during a write group.
+  (Andrew Bennetts, #574236)
+
 Internals
 *********
 
+* ``chk_map._bytes_to_text_key`` is now an optimized function to extract
+  the (file-id, revision-id) key from a CHKInventory entry. This can
+  potentially shave 5-10% time off during a large fetch. Related to bug
+  #562666. (John Arbash Meinel)
+
+* ``log._get_info_for_log_files`` now takes an add_cleanup callable.
+  (Robert Collins)
+
 * ``_remember_remote_is_before`` no longer raises AssertionError when
   suboptimal network behaviour is noticed; instead it just mutters to the
   log file (and warns the user if they have set the ``hpss`` debug flag).
@@ -111,6 +208,10 @@
   stripped, the prefix ``__doc__ =`` should now be used.
   (Martin <gzlist@googlemail.com>)
 
+* No longer require zlib headers to build extensions, and remove the need
+  for seperate copy of zlib library on windows.
+  (John Arbash Meinel, Martin <gzlist@googlemail.com>, #66923)
+
 Testing
 *******
 
@@ -232,6 +333,9 @@
 * ``bzrlib.diff.get_trees_and_branches_to_diff`` is deprecated.  Use
   ``get_trees_and_branches_to_diff_locked`` instead.
   (Andrew Bennetts)
+
+* ``TreeTransform.commit`` supports the full set of commit parameters, and
+  auto-determines branch nick if not supplied.  (Aaron Bentley)
   
 Internals
 *********

=== modified file 'bzrlib/_chk_map_py.py'
--- bzrlib/_chk_map_py.py	2010-02-17 17:11:16 +0000
+++ bzrlib/_chk_map_py.py	2010-05-11 10:45:26 +0000
@@ -157,3 +157,11 @@
     result._node_width = len(prefix)
     result._search_prefix = common_prefix
     return result
+
+
+def _bytes_to_text_key(bytes):
+    """Take a CHKInventory value string and return a (file_id, rev_id) tuple"""
+    sections = bytes.split('\n')
+    kind, file_id = sections[0].split(': ')
+    return (intern(file_id), intern(sections[3]))
+

=== modified file 'bzrlib/_chk_map_pyx.pyx'
--- bzrlib/_chk_map_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_chk_map_pyx.pyx	2010-05-16 15:18:43 +0000
@@ -35,40 +35,26 @@
     Py_ssize_t PyTuple_GET_SIZE(object t)
     int PyString_CheckExact(object)
     char *PyString_AS_STRING(object s)
+    PyObject *PyString_FromStringAndSize_ptr "PyString_FromStringAndSize" (char *, Py_ssize_t)
     Py_ssize_t PyString_GET_SIZE(object)
+    void PyString_InternInPlace(PyObject **)
+    long PyInt_AS_LONG(object)
 
     int PyDict_SetItem(object d, object k, object v) except -1
 
-    object PyTuple_New(Py_ssize_t count)
-    void PyTuple_SET_ITEM(object t, Py_ssize_t offset, object)
-
     void Py_INCREF(object)
+    void Py_DECREF_ptr "Py_DECREF" (PyObject *)
 
-    PyObject * PyTuple_GET_ITEM_ptr "PyTuple_GET_ITEM" (object t,
-                                                        Py_ssize_t offset)
-    int PyString_CheckExact_ptr "PyString_CheckExact" (PyObject *p)
-    Py_ssize_t PyString_GET_SIZE_ptr "PyString_GET_SIZE" (PyObject *s)
-    char *PyString_AS_STRING_ptr "PyString_AS_STRING" (PyObject *s)
     object PyString_FromStringAndSize(char*, Py_ssize_t)
 
 # cimport all of the definitions we will need to access
 from _static_tuple_c cimport StaticTuple,\
     import_static_tuple_c, StaticTuple_New, \
-    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact
-
-cdef extern from "_static_tuple_c.h":
-    # Defined explicitly rather than cimport-ing. Trying to use cimport, the
-    # type for PyObject is a different class that happens to have the same
-    # name...
-    PyObject * StaticTuple_GET_ITEM_ptr "StaticTuple_GET_ITEM" (StaticTuple,
-                                                                Py_ssize_t)
-
-cdef extern from "zlib.h":
-    ctypedef unsigned long uLong
-    ctypedef unsigned int uInt
-    ctypedef unsigned char Bytef
-
-    uLong crc32(uLong crc, Bytef *buf, uInt len)
+    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact, \
+    StaticTuple_GET_SIZE
+
+cdef object crc32
+from zlib import crc32
 
 
 # Set up the StaticTuple C_API functionality
@@ -96,20 +82,30 @@
     return NULL
 
 
+cdef object safe_interned_string_from_size(char *s, Py_ssize_t size):
+    cdef PyObject *py_str
+    if size < 0:
+        raise AssertionError(
+            'tried to create a string with an invalid size: %d @0x%x'
+            % (size, <int>s))
+    py_str = PyString_FromStringAndSize_ptr(s, size)
+    PyString_InternInPlace(&py_str)
+    result = <object>py_str
+    # Casting a PyObject* to an <object> triggers an INCREF from Pyrex, so we
+    # DECREF it to avoid geting immortal strings
+    Py_DECREF_ptr(py_str)
+    return result
+
+
 def _search_key_16(key):
     """See chk_map._search_key_16."""
     cdef Py_ssize_t num_bits
     cdef Py_ssize_t i, j
     cdef Py_ssize_t num_out_bytes
-    cdef Bytef *c_bit
-    cdef uLong c_len
-    cdef uInt crc_val
+    cdef unsigned long crc_val
     cdef Py_ssize_t out_off
     cdef char *c_out
-    cdef PyObject *bit
 
-    if not StaticTuple_CheckExact(key):
-        raise TypeError('key %r is not a StaticTuple' % (key,))
     num_bits = len(key)
     # 4 bytes per crc32, and another 1 byte between bits
     num_out_bytes = (9 * num_bits) - 1
@@ -119,15 +115,7 @@
         if i > 0:
             c_out[0] = c'\x00'
             c_out = c_out + 1
-        # We use the _ptr variant, because GET_ITEM returns a borrowed
-        # reference, and Pyrex assumes that returned 'object' are a new
-        # reference
-        bit = StaticTuple_GET_ITEM_ptr(key, i)
-        if not PyString_CheckExact_ptr(bit):
-            raise TypeError('Bit %d of %r is not a string' % (i, key))
-        c_bit = <Bytef *>PyString_AS_STRING_ptr(bit)
-        c_len = PyString_GET_SIZE_ptr(bit)
-        crc_val = crc32(0, c_bit, c_len)
+        crc_val = PyInt_AS_LONG(crc32(key[i]))
         # Hex(val) order
         sprintf(c_out, '%08X', crc_val)
         c_out = c_out + 8
@@ -139,15 +127,10 @@
     cdef Py_ssize_t num_bits
     cdef Py_ssize_t i, j
     cdef Py_ssize_t num_out_bytes
-    cdef Bytef *c_bit
-    cdef uLong c_len
-    cdef uInt crc_val
+    cdef unsigned long crc_val
     cdef Py_ssize_t out_off
     cdef char *c_out
-    cdef PyObject *bit
 
-    if not StaticTuple_CheckExact(key):
-        raise TypeError('key %r is not a StaticTuple' % (key,))
     num_bits = len(key)
     # 4 bytes per crc32, and another 1 byte between bits
     num_out_bytes = (5 * num_bits) - 1
@@ -157,13 +140,7 @@
         if i > 0:
             c_out[0] = c'\x00'
             c_out = c_out + 1
-        bit = StaticTuple_GET_ITEM_ptr(key, i)
-        if not PyString_CheckExact_ptr(bit):
-            raise TypeError('Bit %d of %r is not a string: %r'
-                            % (i, key, <object>bit))
-        c_bit = <Bytef *>PyString_AS_STRING_ptr(bit)
-        c_len = PyString_GET_SIZE_ptr(bit)
-        crc_val = crc32(0, c_bit, c_len)
+        crc_val = PyInt_AS_LONG(crc32(key[i]))
         # MSB order
         c_out[0] = (crc_val >> 24) & 0xFF
         c_out[1] = (crc_val >> 16) & 0xFF
@@ -198,6 +175,16 @@
     return value
 
 
+cdef _import_globals():
+    """Set the global attributes. Done lazy to avoid recursive import loops."""
+    global _LeafNode, _InternalNode, _unknown
+
+    from bzrlib import chk_map
+    _LeafNode = chk_map.LeafNode
+    _InternalNode = chk_map.InternalNode
+    _unknown = chk_map._unknown
+
+
 def _deserialise_leaf_node(bytes, key, search_key_func=None):
     """Deserialise bytes, with key key, into a LeafNode.
 
@@ -215,10 +202,7 @@
     cdef StaticTuple entry_bits
 
     if _LeafNode is None:
-        from bzrlib import chk_map
-        _LeafNode = chk_map.LeafNode
-        _InternalNode = chk_map.InternalNode
-        _unknown = chk_map._unknown
+        _import_globals()
 
     result = _LeafNode(search_key_func=search_key_func)
     # Splitlines can split on '\r' so don't use it, split('\n') adds an
@@ -322,7 +306,7 @@
                                                next_null - entry_start)
             Py_INCREF(entry)
             StaticTuple_SET_ITEM(entry_bits, i, entry)
-        if len(entry_bits) != width:
+        if StaticTuple_GET_SIZE(entry_bits) != width:
             raise AssertionError(
                 'Incorrect number of elements (%d vs %d)'
                 % (len(entry_bits)+1, width + 1))
@@ -358,10 +342,7 @@
     cdef char *prefix, *line_prefix, *next_null, *c_item_prefix
 
     if _InternalNode is None:
-        from bzrlib import chk_map
-        _LeafNode = chk_map.LeafNode
-        _InternalNode = chk_map.InternalNode
-        _unknown = chk_map._unknown
+        _import_globals()
     result = _InternalNode(search_key_func=search_key_func)
 
     if not StaticTuple_CheckExact(key):
@@ -422,3 +403,51 @@
     result._node_width = len(item_prefix)
     result._search_prefix = PyString_FromStringAndSize(prefix, prefix_length)
     return result
+
+
+def _bytes_to_text_key(bytes):
+    """Take a CHKInventory value string and return a (file_id, rev_id) tuple"""
+    cdef StaticTuple key
+    cdef char *byte_str, *cur_end, *file_id_str, *byte_end
+    cdef char *revision_str
+    cdef Py_ssize_t byte_size, pos, file_id_len
+
+    if not PyString_CheckExact(bytes):
+        raise TypeError('bytes must be a string')
+    byte_str = PyString_AS_STRING(bytes)
+    byte_size = PyString_GET_SIZE(bytes)
+    byte_end = byte_str + byte_size
+    cur_end = <char*>memchr(byte_str, c':', byte_size)
+    if cur_end == NULL:
+        raise ValueError('No kind section found.')
+    if cur_end[1] != c' ':
+        raise ValueError('Kind section should end with ": "')
+    file_id_str = cur_end + 2
+    # file_id is now the data up until the next newline
+    cur_end = <char*>memchr(file_id_str, c'\n', byte_end - file_id_str)
+    if cur_end == NULL:
+        raise ValueError('no newline after file-id')
+    file_id = safe_interned_string_from_size(file_id_str,
+                                             cur_end - file_id_str)
+    # this is the end of the parent_str
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        raise ValueError('no newline after parent_str')
+    # end of the name str
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        raise ValueError('no newline after name str')
+    # the next section is the revision info
+    revision_str = cur_end + 1
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        # This is probably a dir: entry, which has revision as the last item
+        cur_end = byte_end
+    revision = safe_interned_string_from_size(revision_str,
+        cur_end - revision_str)
+    key = StaticTuple_New(2)
+    Py_INCREF(file_id)
+    StaticTuple_SET_ITEM(key, 0, file_id) 
+    Py_INCREF(revision)
+    StaticTuple_SET_ITEM(key, 1, revision) 
+    return StaticTuple_Intern(key)

=== modified file 'bzrlib/_static_tuple_c.h'
--- bzrlib/_static_tuple_c.h	2009-10-13 18:00:16 +0000
+++ bzrlib/_static_tuple_c.h	2010-05-11 14:13:31 +0000
@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Canonical Ltd
+/* Copyright (C) 2009, 2010 Canonical Ltd
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,7 @@
 #define StaticTuple_SET_ITEM(key, offset, val) \
     ((((StaticTuple*)(key))->items[(offset)]) = ((PyObject *)(val)))
 #define StaticTuple_GET_ITEM(key, offset) (((StaticTuple*)key)->items[offset])
+#define StaticTuple_GET_SIZE(key) (((StaticTuple*)key)->size)
 
 
 #ifdef STATIC_TUPLE_MODULE

=== modified file 'bzrlib/_static_tuple_c.pxd'
--- bzrlib/_static_tuple_c.pxd	2010-02-17 17:11:16 +0000
+++ bzrlib/_static_tuple_c.pxd	2010-05-11 14:13:31 +0000
@@ -42,3 +42,4 @@
     # not an 'object' return value.
     void *StaticTuple_GET_ITEM(StaticTuple key, Py_ssize_t offset)
     int StaticTuple_CheckExact(object)
+    Py_ssize_t StaticTuple_GET_SIZE(StaticTuple key)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-04-28 09:40:23 +0000
+++ bzrlib/branch.py	2010-05-11 08:44:59 +0000
@@ -49,7 +49,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.hooks import HookPoint, Hooks
 from bzrlib.inter import InterObject
-from bzrlib.lock import _RelockDebugMixin
+from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib import registry
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -283,10 +283,20 @@
         new_history.reverse()
         return new_history
 
-    def lock_write(self):
+    def lock_write(self, token=None):
+        """Lock the branch for write operations.
+
+        :param token: A token to permit reacquiring a previously held and
+            preserved lock.
+        :return: A BranchWriteLockResult.
+        """
         raise NotImplementedError(self.lock_write)
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         raise NotImplementedError(self.lock_read)
 
     def unlock(self):
@@ -1346,6 +1356,8 @@
         """
         # XXX: Fix the bzrdir API to allow getting the branch back from the
         # clone call. Or something. 20090224 RBC/spiv.
+        # XXX: Should this perhaps clone colocated branches as well, 
+        # rather than just the default branch? 20100319 JRV
         if revision_id is None:
             revision_id = self.last_revision()
         dir_to = self.bzrdir.clone_on_transport(to_transport,
@@ -1521,7 +1533,7 @@
         """Return the current default format."""
         return klass._default_format
 
-    def get_reference(self, a_bzrdir):
+    def get_reference(self, a_bzrdir, name=None):
         """Get the target reference of the branch in a_bzrdir.
 
         format probing must have been completed before calling
@@ -1529,12 +1541,13 @@
         in a_bzrdir is correct.
 
         :param a_bzrdir: The bzrdir to get the branch data from.
+        :param name: Name of the colocated branch to fetch
         :return: None if the branch is not a reference branch.
         """
         return None
 
     @classmethod
-    def set_reference(self, a_bzrdir, to_branch):
+    def set_reference(self, a_bzrdir, name, to_branch):
         """Set the target reference of the branch in a_bzrdir.
 
         format probing must have been completed before calling
@@ -1542,6 +1555,7 @@
         in a_bzrdir is correct.
 
         :param a_bzrdir: The bzrdir to set the branch reference for.
+        :param name: Name of colocated branch to set, None for default
         :param to_branch: branch that the checkout is to reference
         """
         raise NotImplementedError(self.set_reference)
@@ -2157,14 +2171,14 @@
         """See BranchFormat.get_format_description()."""
         return "Checkout reference format 1"
 
-    def get_reference(self, a_bzrdir):
+    def get_reference(self, a_bzrdir, name=None):
         """See BranchFormat.get_reference()."""
-        transport = a_bzrdir.get_branch_transport(None)
+        transport = a_bzrdir.get_branch_transport(None, name=name)
         return transport.get_bytes('location')
 
-    def set_reference(self, a_bzrdir, to_branch):
+    def set_reference(self, a_bzrdir, name, to_branch):
         """See BranchFormat.set_reference()."""
-        transport = a_bzrdir.get_branch_transport(None)
+        transport = a_bzrdir.get_branch_transport(None, name=name)
         location = transport.put_bytes('location', to_branch.base)
 
     def initialize(self, a_bzrdir, name=None, target_branch=None):
@@ -2221,7 +2235,7 @@
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))
         if location is None:
-            location = self.get_reference(a_bzrdir)
+            location = self.get_reference(a_bzrdir, name)
         real_bzrdir = bzrdir.BzrDir.open(
             location, possible_transports=possible_transports)
         result = real_bzrdir.open_branch(name=name, 
@@ -2265,6 +2279,23 @@
     _legacy_formats[0].network_name(), _legacy_formats[0].__class__)
 
 
+class BranchWriteLockResult(LogicalLockResult):
+    """The result of write locking a branch.
+
+    :ivar branch_token: The token obtained from the underlying branch lock, or
+        None.
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock, branch_token):
+        LogicalLockResult.__init__(self, unlock)
+        self.branch_token = branch_token
+
+    def __repr__(self):
+        return "BranchWriteLockResult(%s, %s)" % (self.branch_token,
+            self.unlock)
+
+
 class BzrBranch(Branch, _RelockDebugMixin):
     """A branch stored in the actual filesystem.
 
@@ -2324,6 +2355,12 @@
         return self.control_files.is_locked()
 
     def lock_write(self, token=None):
+        """Lock the branch for write operations.
+
+        :param token: A token to permit reacquiring a previously held and
+            preserved lock.
+        :return: A BranchWriteLockResult.
+        """
         if not self.is_locked():
             self._note_lock('w')
         # All-in-one needs to always unlock/lock.
@@ -2335,13 +2372,18 @@
         else:
             took_lock = False
         try:
-            return self.control_files.lock_write(token=token)
+            return BranchWriteLockResult(self.unlock,
+                self.control_files.lock_write(token=token))
         except:
             if took_lock:
                 self.repository.unlock()
             raise
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._note_lock('r')
         # All-in-one needs to always unlock/lock.
@@ -2354,6 +2396,7 @@
             took_lock = False
         try:
             self.control_files.lock_read()
+            return LogicalLockResult(self.unlock)
         except:
             if took_lock:
                 self.repository.unlock()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-05-03 09:07:50 +0000
+++ bzrlib/builtins.py	2010-05-14 13:39:47 +0000
@@ -232,6 +232,14 @@
     return view_info
 
 
+def _open_directory_or_containing_tree_or_branch(filename, directory):
+    """Open the tree or branch containing the specified file, unless
+    the --directory option is used to specify a different branch."""
+    if directory is not None:
+        return (None, Branch.open(directory), filename)
+    return bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+
+
 # TODO: Make sure no commands unconditionally use the working directory as a
 # branch.  If a filename argument is used, the first of them should be used to
 # specify the branch.  (Perhaps this can be factored out into some kind of
@@ -340,7 +348,7 @@
 
     hidden = True
     takes_args = ['revision_id?']
-    takes_options = ['revision']
+    takes_options = ['directory', 'revision']
     # cat-revision is more for frontends so should be exact
     encoding = 'strict'
 
@@ -353,14 +361,14 @@
         self.outf.write(revtext.decode('utf-8'))
 
     @display_command
-    def run(self, revision_id=None, revision=None):
+    def run(self, revision_id=None, revision=None, directory=u'.'):
         if revision_id is not None and revision is not None:
             raise errors.BzrCommandError('You can only supply one of'
                                          ' revision_id or --revision')
         if revision_id is None and revision is None:
             raise errors.BzrCommandError('You must supply either'
                                          ' --revision or a revision_id')
-        b = WorkingTree.open_containing(u'.')[0].branch
+        b = WorkingTree.open_containing(directory)[0].branch
 
         revisions = b.repository.revisions
         if revisions is None:
@@ -528,10 +536,9 @@
         if tree:
             try:
                 wt = WorkingTree.open_containing(location)[0]
-                wt.lock_read()
+                self.add_cleanup(wt.lock_read().unlock)
             except (errors.NoWorkingTree, errors.NotLocalUrl):
                 raise errors.NoWorkingTree(location)
-            self.add_cleanup(wt.unlock)
             revid = wt.last_revision()
             try:
                 revno_t = wt.branch.revision_id_to_dotted_revno(revid)
@@ -540,8 +547,7 @@
             revno = ".".join(str(n) for n in revno_t)
         else:
             b = Branch.open_containing(location)[0]
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
             revno = b.revno()
         self.cleanup_now()
         self.outf.write(str(revno) + '\n')
@@ -554,12 +560,9 @@
     takes_args = ['revision_info*']
     takes_options = [
         'revision',
-        Option('directory',
+        custom_help('directory',
             help='Branch to examine, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('tree', help='Show revno of working tree'),
         ]
 
@@ -570,13 +573,11 @@
         try:
             wt = WorkingTree.open_containing(directory)[0]
             b = wt.branch
-            wt.lock_read()
-            self.add_cleanup(wt.unlock)
+            self.add_cleanup(wt.lock_read().unlock)
         except (errors.NoWorkingTree, errors.NotLocalUrl):
             wt = None
             b = Branch.open_containing(directory)[0]
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
         revision_ids = []
         if revision is not None:
             revision_ids.extend(rev.as_revision_id(b) for rev in revision)
@@ -681,8 +682,7 @@
                 should_print=(not is_quiet()))
 
         if base_tree:
-            base_tree.lock_read()
-            self.add_cleanup(base_tree.unlock)
+            self.add_cleanup(base_tree.lock_read().unlock)
         tree, file_list = tree_files_for_add(file_list)
         added, ignored = tree.smart_add(file_list, not
             no_recurse, action=action, save=not dry_run)
@@ -760,14 +760,12 @@
 
         revision = _get_one_revision('inventory', revision)
         work_tree, file_list = tree_files(file_list)
-        work_tree.lock_read()
-        self.add_cleanup(work_tree.unlock)
+        self.add_cleanup(work_tree.lock_read().unlock)
         if revision is not None:
             tree = revision.as_tree(work_tree.branch)
 
             extra_trees = [work_tree]
-            tree.lock_read()
-            self.add_cleanup(tree.unlock)
+            self.add_cleanup(tree.lock_read().unlock)
         else:
             tree = work_tree
             extra_trees = []
@@ -833,8 +831,7 @@
         if len(names_list) < 2:
             raise errors.BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list, canonicalize=False)
-        tree.lock_tree_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_tree_write().unlock)
         self._run(tree, names_list, rel_names, after)
 
     def run_auto(self, names_list, after, dry_run):
@@ -845,8 +842,7 @@
             raise errors.BzrCommandError('--after cannot be specified with'
                                          ' --auto.')
         work_tree, file_list = tree_files(names_list, default_branch='.')
-        work_tree.lock_tree_write()
-        self.add_cleanup(work_tree.unlock)
+        self.add_cleanup(work_tree.lock_tree_write().unlock)
         rename_map.RenameMap.guess_renames(work_tree, dry_run)
 
     def _run(self, tree, names_list, rel_names, after):
@@ -960,12 +956,9 @@
     takes_options = ['remember', 'overwrite', 'revision',
         custom_help('verbose',
             help='Show logs of pulled revisions.'),
-        Option('directory',
+        custom_help('directory',
             help='Branch to pull into, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('local',
             help="Perform a local pull in a bound "
                  "branch.  Local pulls are not applied to "
@@ -986,13 +979,11 @@
         try:
             tree_to = WorkingTree.open_containing(directory)[0]
             branch_to = tree_to.branch
-            tree_to.lock_write()
-            self.add_cleanup(tree_to.unlock)
+            self.add_cleanup(tree_to.lock_write().unlock)
         except errors.NoWorkingTree:
             tree_to = None
             branch_to = Branch.open_containing(directory)[0]
-            branch_to.lock_write()
-            self.add_cleanup(branch_to.unlock)
+            self.add_cleanup(branch_to.lock_write().unlock)
 
         if local and not branch_to.get_bound_location():
             raise errors.LocalRequiresBoundBranch()
@@ -1029,8 +1020,7 @@
         else:
             branch_from = Branch.open(location,
                 possible_transports=possible_transports)
-            branch_from.lock_read()
-            self.add_cleanup(branch_from.unlock)
+            self.add_cleanup(branch_from.lock_read().unlock)
 
             if branch_to.get_parent() is None or remember:
                 branch_to.set_parent(branch_from.base)
@@ -1088,12 +1078,9 @@
         Option('create-prefix',
                help='Create the path leading up to the branch '
                     'if it does not already exist.'),
-        Option('directory',
+        custom_help('directory',
             help='Branch to push from, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('use-existing-dir',
                help='By default push will fail if the target'
                     ' directory exists, but does not already'
@@ -1219,8 +1206,7 @@
         accelerator_tree, br_from = bzrdir.BzrDir.open_tree_or_branch(
             from_location)
         revision = _get_one_revision('branch', revision)
-        br_from.lock_read()
-        self.add_cleanup(br_from.unlock)
+        self.add_cleanup(br_from.lock_read().unlock)
         if revision is not None:
             revision_id = revision.as_revision_id(br_from)
         else:
@@ -1350,6 +1336,11 @@
             except errors.NoWorkingTree:
                 source.bzrdir.create_workingtree(revision_id)
                 return
+
+        if not lightweight:
+            message = ('Copying history to "%s". '
+                'To checkout without local history use --lightweight.' % to_location)
+            ui.ui_factory.show_message(message)
         source.create_checkout(to_location, revision_id, lightweight,
                                accelerator_tree, hardlink)
 
@@ -1366,12 +1357,10 @@
     @display_command
     def run(self, dir=u'.'):
         tree = WorkingTree.open_containing(dir)[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         new_inv = tree.inventory
         old_tree = tree.basis_tree()
-        old_tree.lock_read()
-        self.add_cleanup(old_tree.unlock)
+        self.add_cleanup(old_tree.lock_read().unlock)
         old_inv = old_tree.inventory
         renames = []
         iterator = tree.iter_changes(old_tree, include_unchanged=True)
@@ -1415,11 +1404,11 @@
         master = branch.get_master_branch(
             possible_transports=possible_transports)
         if master is not None:
-            tree.lock_write()
             branch_location = master.base
+            tree.lock_write()
         else:
+            branch_location = tree.branch.base
             tree.lock_tree_write()
-            branch_location = tree.branch.base
         self.add_cleanup(tree.unlock)
         # get rid of the final '/' and be ready for display
         branch_location = urlutils.unescape_for_display(
@@ -1545,8 +1534,7 @@
         if file_list is not None:
             file_list = [f for f in file_list]
 
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_write().unlock)
         # Heuristics should probably all move into tree.remove_smart or
         # some such?
         if new:
@@ -2003,16 +1991,14 @@
     # level of effort but possibly much less IO.  (Or possibly not,
     # if the directories are very large...)
     _see_also = ['status', 'ls']
-    takes_options = ['show-ids']
+    takes_options = ['directory', 'show-ids']
 
     @display_command
-    def run(self, show_ids=False):
-        tree = WorkingTree.open_containing(u'.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+    def run(self, show_ids=False, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         old = tree.basis_tree()
-        old.lock_read()
-        self.add_cleanup(old.unlock)
+        self.add_cleanup(old.lock_read().unlock)
         for path, ie in old.inventory.iter_entries():
             if not tree.has_id(ie.file_id):
                 self.outf.write(path)
@@ -2028,15 +2014,11 @@
 
     hidden = True
     _see_also = ['status', 'ls']
-    takes_options = [
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.')
-            ]
+    takes_options = ['directory', 'null']
 
     @display_command
-    def run(self, null=False):
-        tree = WorkingTree.open_containing(u'.')[0]
+    def run(self, null=False, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
         td = tree.changes_from(tree.basis_tree())
         for path, id, kind, text_modified, meta_modified in td.modified:
             if null:
@@ -2051,20 +2033,14 @@
 
     hidden = True
     _see_also = ['status', 'ls']
-    takes_options = [
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.')
-            ]
+    takes_options = ['directory', 'null']
 
     @display_command
-    def run(self, null=False):
-        wt = WorkingTree.open_containing(u'.')[0]
-        wt.lock_read()
-        self.add_cleanup(wt.unlock)
+    def run(self, null=False, directory=u'.'):
+        wt = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(wt.lock_read().unlock)
         basis = wt.basis_tree()
-        basis.lock_read()
-        self.add_cleanup(basis.unlock)
+        self.add_cleanup(basis.lock_read().unlock)
         basis_inv = basis.inventory
         inv = wt.inventory
         for file_id in inv:
@@ -2073,7 +2049,7 @@
             if inv.is_root(file_id) and len(basis_inv) == 0:
                 continue
             path = inv.id2path(file_id)
-            if not os.access(osutils.abspath(path), os.F_OK):
+            if not os.access(osutils.pathjoin(wt.basedir, path), os.F_OK):
                 continue
             if null:
                 self.outf.write(path + '\0')
@@ -2279,6 +2255,11 @@
                    help='Show just the specified revision.'
                    ' See also "help revisionspec".'),
             'log-format',
+            RegistryOption('authors',
+                'What names to list as authors - first, all or committer.',
+                title='Authors',
+                lazy_registry=('bzrlib.log', 'author_list_registry'),
+            ),
             Option('levels',
                    short_name='n',
                    help='Number of levels to display - 0 for all, 1 for flat.',
@@ -2319,6 +2300,7 @@
             limit=None,
             show_diff=False,
             include_merges=False,
+            authors=None,
             exclude_common_ancestry=False,
             ):
         from bzrlib.log import (
@@ -2352,8 +2334,7 @@
         if file_list:
             # find the file ids to log and check for directory filtering
             b, file_info_list, rev1, rev2 = _get_info_for_log_files(
-                revision, file_list)
-            self.add_cleanup(b.unlock)
+                revision, file_list, self.add_cleanup)
             for relpath, file_id, kind in file_info_list:
                 if file_id is None:
                     raise errors.BzrCommandError(
@@ -2377,8 +2358,7 @@
                 location = '.'
             dir, relpath = bzrdir.BzrDir.open_containing(location)
             b = dir.open_branch()
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
             rev1, rev2 = _get_revision_range(revision, b, self.name())
 
         # Decide on the type of delta & diff filtering to use
@@ -2404,7 +2384,8 @@
                         show_timezone=timezone,
                         delta_format=get_verbosity_level(),
                         levels=levels,
-                        show_advice=levels is None)
+                        show_advice=levels is None,
+                        author_list_handler=authors)
 
         # Choose the algorithm for doing the logging. It's annoying
         # having multiple code paths like this but necessary until
@@ -2508,8 +2489,7 @@
         tree, relpath = WorkingTree.open_containing(filename)
         file_id = tree.path2id(relpath)
         b = tree.branch
-        b.lock_read()
-        self.add_cleanup(b.unlock)
+        self.add_cleanup(b.lock_read().unlock)
         touching_revs = log.find_touching_revisions(b, file_id)
         for revno, revision_id, what in touching_revs:
             self.outf.write("%6d %s\n" % (revno, what))
@@ -2528,23 +2508,24 @@
                    help='Recurse into subdirectories.'),
             Option('from-root',
                    help='Print paths relative to the root of the branch.'),
-            Option('unknown', help='Print unknown files.'),
+            Option('unknown', short_name='u',
+                help='Print unknown files.'),
             Option('versioned', help='Print versioned files.',
                    short_name='V'),
-            Option('ignored', help='Print ignored files.'),
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.'),
-            Option('kind',
+            Option('ignored', short_name='i',
+                help='Print ignored files.'),
+            Option('kind', short_name='k',
                    help='List entries of a particular kind: file, directory, symlink.',
                    type=unicode),
+            'null',
             'show-ids',
+            'directory',
             ]
     @display_command
     def run(self, revision=None, verbose=False,
             recursive=False, from_root=False,
             unknown=False, versioned=False, ignored=False,
-            null=False, kind=None, show_ids=False, path=None):
+            null=False, kind=None, show_ids=False, path=None, directory=None):
 
         if kind and kind not in ('file', 'directory', 'symlink'):
             raise errors.BzrCommandError('invalid kind specified')
@@ -2562,8 +2543,8 @@
                 raise errors.BzrCommandError('cannot specify both --from-root'
                                              ' and PATH')
             fs_path = path
-        tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
-            fs_path)
+        tree, branch, relpath = \
+            _open_directory_or_containing_tree_or_branch(fs_path, directory)
 
         # Calculate the prefix to use
         prefix = None
@@ -2584,8 +2565,7 @@
                 view_str = views.view_display_str(view_files)
                 note("Ignoring files outside view. View is %s" % view_str)
 
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         for fp, fc, fkind, fid, entry in tree.list_files(include_root=False,
             from_dir=relpath, recursive=recursive):
             # Apply additional masking
@@ -2638,10 +2618,11 @@
 
     hidden = True
     _see_also = ['ls']
+    takes_options = ['directory']
 
     @display_command
-    def run(self):
-        for f in WorkingTree.open_containing(u'.')[0].unknowns():
+    def run(self, directory=u'.'):
+        for f in WorkingTree.open_containing(directory)[0].unknowns():
             self.outf.write(osutils.quotefn(f) + '\n')
 
 
@@ -2712,12 +2693,13 @@
 
     _see_also = ['status', 'ignored', 'patterns']
     takes_args = ['name_pattern*']
-    takes_options = [
+    takes_options = ['directory',
         Option('default-rules',
                help='Display the default ignore rules that bzr uses.')
         ]
 
-    def run(self, name_pattern_list=None, default_rules=None):
+    def run(self, name_pattern_list=None, default_rules=None,
+            directory=u'.'):
         from bzrlib import ignores
         if default_rules is not None:
             # dump the default rules and exit
@@ -2734,7 +2716,7 @@
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):
                 raise errors.BzrCommandError(
                     "NAME_PATTERN should not be an absolute path")
-        tree, relpath = WorkingTree.open_containing(u'.')
+        tree, relpath = WorkingTree.open_containing(directory)
         ignores.tree_ignores_add_patterns(tree, name_pattern_list)
         ignored = globbing.Globster(name_pattern_list)
         matches = []
@@ -2766,12 +2748,12 @@
 
     encoding_type = 'replace'
     _see_also = ['ignore', 'ls']
+    takes_options = ['directory']
 
     @display_command
-    def run(self):
-        tree = WorkingTree.open_containing(u'.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+    def run(self, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
@@ -2788,15 +2770,16 @@
     """
     hidden = True
     takes_args = ['revno']
+    takes_options = ['directory']
 
     @display_command
-    def run(self, revno):
+    def run(self, revno, directory=u'.'):
         try:
             revno = int(revno)
         except ValueError:
             raise errors.BzrCommandError("not a valid revision-number: %r"
                                          % revno)
-        revid = WorkingTree.open_containing(u'.')[0].branch.get_rev_id(revno)
+        revid = WorkingTree.open_containing(directory)[0].branch.get_rev_id(revno)
         self.outf.write("%s\n" % revid)
 
 
@@ -2829,7 +2812,7 @@
       =================       =========================
     """
     takes_args = ['dest', 'branch_or_subdir?']
-    takes_options = [
+    takes_options = ['directory',
         Option('format',
                help="Type of file to export to.",
                type=unicode),
@@ -2844,11 +2827,11 @@
                     'revision in which it was changed.'),
         ]
     def run(self, dest, branch_or_subdir=None, revision=None, format=None,
-        root=None, filters=False, per_file_timestamps=False):
+        root=None, filters=False, per_file_timestamps=False, directory=u'.'):
         from bzrlib.export import export
 
         if branch_or_subdir is None:
-            tree = WorkingTree.open_containing(u'.')[0]
+            tree = WorkingTree.open_containing(directory)[0]
             b = tree.branch
             subdir = None
         else:
@@ -2873,7 +2856,7 @@
     """
 
     _see_also = ['ls']
-    takes_options = [
+    takes_options = ['directory',
         Option('name-from-revision', help='The path name in the old tree.'),
         Option('filters', help='Apply content filters to display the '
                 'convenience form.'),
@@ -2884,14 +2867,13 @@
 
     @display_command
     def run(self, filename, revision=None, name_from_revision=False,
-            filters=False):
+            filters=False, directory=None):
         if revision is not None and len(revision) != 1:
             raise errors.BzrCommandError("bzr cat --revision takes exactly"
                                          " one revision specifier")
         tree, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(filename)
-        branch.lock_read()
-        self.add_cleanup(branch.unlock)
+            _open_directory_or_containing_tree_or_branch(filename, directory)
+        self.add_cleanup(branch.lock_read().unlock)
         return self._run(tree, branch, relpath, filename, revision,
                          name_from_revision, filters)
 
@@ -2900,8 +2882,7 @@
         if tree is None:
             tree = b.basis_tree()
         rev_tree = _get_one_revision_tree('cat', revision, branch=b)
-        rev_tree.lock_read()
-        self.add_cleanup(rev_tree.unlock)
+        self.add_cleanup(rev_tree.lock_read().unlock)
 
         old_file_id = rev_tree.path2id(relpath)
 
@@ -3380,8 +3361,9 @@
 
     _see_also = ['info']
     takes_args = ['nickname?']
-    def run(self, nickname=None):
-        branch = Branch.open_containing(u'.')[0]
+    takes_options = ['directory']
+    def run(self, nickname=None, directory=u'.'):
+        branch = Branch.open_containing(directory)[0]
         if nickname is None:
             self.printme(branch)
         else:
@@ -3612,7 +3594,11 @@
             self.additional_selftest_args['runner_class'] = SubUnitBzrRunner
             # On Windows, disable automatic conversion of '\n' to '\r\n' in
             # stdout, which would corrupt the subunit stream. 
-            if sys.platform == "win32" and sys.stdout.fileno() >= 0:
+            # FIXME: This has been fixed in subunit trunk (>0.0.5) so the
+            # following code can be deleted when it's sufficiently deployed
+            # -- vila/mgz 20100514
+            if (sys.platform == "win32"
+                and getattr(sys.stdout, 'fileno', None) is not None):
                 import msvcrt
                 msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
         if parallel:
@@ -3690,10 +3676,8 @@
 
         branch1 = Branch.open_containing(branch)[0]
         branch2 = Branch.open_containing(other)[0]
-        branch1.lock_read()
-        self.add_cleanup(branch1.unlock)
-        branch2.lock_read()
-        self.add_cleanup(branch2.unlock)
+        self.add_cleanup(branch1.lock_read().unlock)
+        self.add_cleanup(branch2.lock_read().unlock)
         last1 = ensure_null(branch1.last_revision())
         last2 = ensure_null(branch2.last_revision())
 
@@ -3793,12 +3777,9 @@
                 ' completely merged into the source, pull from the'
                 ' source rather than merging.  When this happens,'
                 ' you do not need to commit the result.'),
-        Option('directory',
+        custom_help('directory',
                help='Branch to merge into, '
-                    'rather than the one containing the working directory.',
-               short_name='d',
-               type=unicode,
-               ),
+                    'rather than the one containing the working directory.'),
         Option('preview', help='Instead of merging, show a diff of the'
                ' merge.'),
         Option('interactive', help='Select changes interactively.',
@@ -3837,8 +3818,7 @@
             unversioned_filter=tree.is_ignored, view_info=view_info)
         pb = ui.ui_factory.nested_progress_bar()
         self.add_cleanup(pb.finished)
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_write().unlock)
         if location is not None:
             try:
                 mergeable = bundle.read_mergeable_from_url(location,
@@ -4100,8 +4080,7 @@
         if merge_type is None:
             merge_type = _mod_merge.Merge3Merger
         tree, file_list = tree_files(file_list)
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_write().unlock)
         parents = tree.get_parent_ids()
         if len(parents) != 2:
             raise errors.BzrCommandError("Sorry, remerge only works after normal"
@@ -4217,8 +4196,7 @@
     def run(self, revision=None, no_backup=False, file_list=None,
             forget_merges=None):
         tree, file_list = tree_files(file_list)
-        tree.lock_tree_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_tree_write().unlock)
         if forget_merges:
             tree.set_parent_ids(tree.get_parent_ids()[:1])
         else:
@@ -4360,8 +4338,7 @@
             restrict = 'remote'
 
         local_branch = Branch.open_containing(u".")[0]
-        local_branch.lock_read()
-        self.add_cleanup(local_branch.unlock)
+        self.add_cleanup(local_branch.lock_read().unlock)
 
         parent = local_branch.get_parent()
         if other_branch is None:
@@ -4378,8 +4355,7 @@
         if remote_branch.base == local_branch.base:
             remote_branch = local_branch
         else:
-            remote_branch.lock_read()
-            self.add_cleanup(remote_branch.unlock)
+            self.add_cleanup(remote_branch.lock_read().unlock)
 
         local_revid_range = _revision_range_to_revid_range(
             _get_revision_range(my_revision, local_branch,
@@ -4440,8 +4416,7 @@
             message("Branches are up to date.\n")
         self.cleanup_now()
         if not status_code and parent is None and other_branch is not None:
-            local_branch.lock_write()
-            self.add_cleanup(local_branch.unlock)
+            self.add_cleanup(local_branch.lock_write().unlock)
             # handle race conditions - a parent might be set while we run.
             if local_branch.get_parent() is None:
                 local_branch.set_parent(remote_branch.base)
@@ -4547,8 +4522,7 @@
             b = Branch.open_containing(branch)[0]
         else:
             b = Branch.open(branch)
-        b.lock_read()
-        self.add_cleanup(b.unlock)
+        self.add_cleanup(b.lock_read().unlock)
         if revision is None:
             rev_id = b.last_revision()
         else:
@@ -4578,24 +4552,22 @@
                      Option('long', help='Show commit date in annotations.'),
                      'revision',
                      'show-ids',
+                     'directory',
                      ]
     encoding_type = 'exact'
 
     @display_command
     def run(self, filename, all=False, long=False, revision=None,
-            show_ids=False):
+            show_ids=False, directory=None):
         from bzrlib.annotate import annotate_file, annotate_file_tree
         wt, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+            _open_directory_or_containing_tree_or_branch(filename, directory)
         if wt is not None:
-            wt.lock_read()
-            self.add_cleanup(wt.unlock)
+            self.add_cleanup(wt.lock_read().unlock)
         else:
-            branch.lock_read()
-            self.add_cleanup(branch.unlock)
+            self.add_cleanup(branch.lock_read().unlock)
         tree = _get_one_revision_tree('annotate', revision, branch=branch)
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         if wt is not None:
             file_id = wt.path2id(relpath)
         else:
@@ -4619,16 +4591,15 @@
 
     hidden = True # is this right ?
     takes_args = ['revision_id*']
-    takes_options = ['revision']
+    takes_options = ['directory', 'revision']
 
-    def run(self, revision_id_list=None, revision=None):
+    def run(self, revision_id_list=None, revision=None, directory=u'.'):
         if revision_id_list is not None and revision is not None:
             raise errors.BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id_list is None and revision is None:
             raise errors.BzrCommandError('You must supply either --revision or a revision_id')
-        b = WorkingTree.open_containing(u'.')[0].branch
-        b.lock_write()
-        self.add_cleanup(b.unlock)
+        b = WorkingTree.open_containing(directory)[0].branch
+        self.add_cleanup(b.lock_write().unlock)
         return self._run(b, revision_id_list, revision)
 
     def _run(self, b, revision_id_list, revision):
@@ -4693,10 +4664,10 @@
 
     _see_also = ['checkouts', 'unbind']
     takes_args = ['location?']
-    takes_options = []
+    takes_options = ['directory']
 
-    def run(self, location=None):
-        b, relpath = Branch.open_containing(u'.')
+    def run(self, location=None, directory=u'.'):
+        b, relpath = Branch.open_containing(directory)
         if location is None:
             try:
                 location = b.get_old_bound_location()
@@ -4729,10 +4700,10 @@
 
     _see_also = ['checkouts', 'bind']
     takes_args = []
-    takes_options = []
+    takes_options = ['directory']
 
-    def run(self):
-        b, relpath = Branch.open_containing(u'.')
+    def run(self, directory=u'.'):
+        b, relpath = Branch.open_containing(directory)
         if not b.unbind():
             raise errors.BzrCommandError('Local branch is not bound')
 
@@ -4784,11 +4755,9 @@
             b = control.open_branch()
 
         if tree is not None:
-            tree.lock_write()
-            self.add_cleanup(tree.unlock)
+            self.add_cleanup(tree.lock_write().unlock)
         else:
-            b.lock_write()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_write().unlock)
         return self._run(b, tree, dry_run, verbose, revision, force, local=local)
 
     def _run(self, b, tree, dry_run, verbose, revision, force, local=False):
@@ -4902,9 +4871,8 @@
                     'result in a dynamically allocated port.  The default port '
                     'depends on the protocol.',
                type=str),
-        Option('directory',
-               help='Serve contents of this directory.',
-               type=unicode),
+        custom_help('directory',
+               help='Serve contents of this directory.'),
         Option('allow-writes',
                help='By default the server is a readonly server.  Supplying '
                     '--allow-writes enables write access to the contents of '
@@ -5339,11 +5307,8 @@
         Option('delete',
             help='Delete this tag rather than placing it.',
             ),
-        Option('directory',
-            help='Branch in which to place the tag.',
-            short_name='d',
-            type=unicode,
-            ),
+        custom_help('directory',
+            help='Branch in which to place the tag.'),
         Option('force',
             help='Replace existing tags.',
             ),
@@ -5357,8 +5322,7 @@
             revision=None,
             ):
         branch, relpath = Branch.open_containing(directory)
-        branch.lock_write()
-        self.add_cleanup(branch.unlock)
+        self.add_cleanup(branch.lock_write().unlock)
         if delete:
             if tag_name is None:
                 raise errors.BzrCommandError("No tag specified to delete.")
@@ -5392,11 +5356,8 @@
 
     _see_also = ['tag']
     takes_options = [
-        Option('directory',
-            help='Branch whose tags should be displayed.',
-            short_name='d',
-            type=unicode,
-            ),
+        custom_help('directory',
+            help='Branch whose tags should be displayed.'),
         RegistryOption.from_kwargs('sort',
             'Sort tags by different criteria.', title='Sorting',
             alpha='Sort tags lexicographically (default).',
@@ -5419,8 +5380,7 @@
         if not tags:
             return
 
-        branch.lock_read()
-        self.add_cleanup(branch.unlock)
+        self.add_cleanup(branch.lock_read().unlock)
         if revision:
             graph = branch.repository.get_graph()
             rev1, rev2 = _get_revision_range(revision, branch, self.name())
@@ -5904,8 +5864,7 @@
 
     def run_for_list(self):
         tree = WorkingTree.open_containing('.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         manager = tree.get_shelf_manager()
         shelves = manager.active_shelves()
         if len(shelves) == 0:
@@ -5966,7 +5925,8 @@
 
     To check what clean-tree will do, use --dry-run.
     """
-    takes_options = [Option('ignored', help='Delete all ignored files.'),
+    takes_options = ['directory',
+                     Option('ignored', help='Delete all ignored files.'),
                      Option('detritus', help='Delete conflict files, merge'
                             ' backups, and failed selftest dirs.'),
                      Option('unknown',
@@ -5975,14 +5935,14 @@
                             ' deleting them.'),
                      Option('force', help='Do not prompt before deleting.')]
     def run(self, unknown=False, ignored=False, detritus=False, dry_run=False,
-            force=False):
+            force=False, directory=u'.'):
         from bzrlib.clean_tree import clean_tree
         if not (unknown or ignored or detritus):
             unknown = True
         if dry_run:
             force = True
-        clean_tree('.', unknown=unknown, ignored=ignored, detritus=detritus,
-                   dry_run=dry_run, no_prompt=force)
+        clean_tree(directory, unknown=unknown, ignored=ignored,
+                   detritus=detritus, dry_run=dry_run, no_prompt=force)
 
 
 class cmd_reference(Command):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-04-28 07:03:38 +0000
+++ bzrlib/bzrdir.py	2010-04-29 08:59:13 +0000
@@ -737,13 +737,18 @@
             raise errors.NoRepositoryPresent(self)
         return found_repo
 
-    def get_branch_reference(self):
+    def get_branch_reference(self, name=None):
         """Return the referenced URL for the branch in this bzrdir.
 
+        :param name: Optional colocated branch name
         :raises NotBranchError: If there is no Branch.
+        :raises NoColocatedBranchSupport: If a branch name was specified
+            but colocated branches are not supported.
         :return: The URL the branch in this bzrdir references if it is a
             reference branch, or None for regular branches.
         """
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
         return None
 
     def get_branch_transport(self, branch_format, name=None):
@@ -994,9 +999,11 @@
                 raise errors.NotBranchError(path=url)
             a_transport = new_t
 
-    def _get_tree_branch(self):
+    def _get_tree_branch(self, name=None):
         """Return the branch and tree, if any, for this bzrdir.
 
+        :param name: Name of colocated branch to open.
+
         Return None for tree if not present or inaccessible.
         Raise NotBranchError if no branch is present.
         :return: (tree, branch)
@@ -1005,9 +1012,12 @@
             tree = self.open_workingtree()
         except (errors.NoWorkingTree, errors.NotLocalUrl):
             tree = None
-            branch = self.open_branch()
+            branch = self.open_branch(name=name)
         else:
-            branch = tree.branch
+            if name is not None:
+                branch = self.open_branch(name=name)
+            else:
+                branch = tree.branch
         return tree, branch
 
     @classmethod
@@ -1736,13 +1746,13 @@
     def destroy_workingtree_metadata(self):
         self.transport.delete_tree('checkout')
 
-    def find_branch_format(self):
+    def find_branch_format(self, name=None):
         """Find the branch 'format' for this bzrdir.
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
         from bzrlib.branch import BranchFormat
-        return BranchFormat.find_format(self)
+        return BranchFormat.find_format(self, name=name)
 
     def _get_mkdir_mode(self):
         """Figure out the mode to use when creating a bzrdir subdir."""
@@ -1750,11 +1760,11 @@
                                      lockable_files.TransportLock)
         return temp_control._dir_mode
 
-    def get_branch_reference(self):
+    def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
         from bzrlib.branch import BranchFormat
-        format = BranchFormat.find_format(self)
-        return format.get_reference(self)
+        format = BranchFormat.find_format(self, name=name)
+        return format.get_reference(self, name=name)
 
     def get_branch_transport(self, branch_format, name=None):
         """See BzrDir.get_branch_transport()."""
@@ -1854,7 +1864,7 @@
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False):
         """See BzrDir.open_branch."""
-        format = self.find_branch_format()
+        format = self.find_branch_format(name=name)
         self._check_supported(format, unsupported)
         return format.open(self, name=name,
             _found=True, ignore_fallbacks=ignore_fallbacks)

=== modified file 'bzrlib/chk_map.py'
--- bzrlib/chk_map.py	2010-04-21 20:33:10 +0000
+++ bzrlib/chk_map.py	2010-05-11 10:45:26 +0000
@@ -1727,6 +1727,7 @@
 
 try:
     from bzrlib._chk_map_pyx import (
+        _bytes_to_text_key,
         _search_key_16,
         _search_key_255,
         _deserialise_leaf_node,
@@ -1735,6 +1736,7 @@
 except ImportError, e:
     osutils.failed_to_load_extension(e)
     from bzrlib._chk_map_py import (
+        _bytes_to_text_key,
         _search_key_16,
         _search_key_255,
         _deserialise_leaf_node,

=== modified file 'bzrlib/clean_tree.py'
--- bzrlib/clean_tree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/clean_tree.py	2010-05-04 09:20:17 +0000
@@ -18,6 +18,7 @@
 import os
 import shutil
 
+from bzrlib import bzrdir, errors
 from bzrlib.osutils import isdir
 from bzrlib.trace import note
 from bzrlib.workingtree import WorkingTree
@@ -51,11 +52,14 @@
     try:
         deletables = list(iter_deletables(tree, unknown=unknown,
             ignored=ignored, detritus=detritus))
+        deletables = _filter_out_nested_bzrdirs(deletables)
         if len(deletables) == 0:
             note('Nothing to delete.')
             return 0
         if not no_prompt:
             for path, subp in deletables:
+                # FIXME using print is very bad idea
+                # clean_tree should accept to_file argument to write the output
                 print subp
             val = raw_input('Are you sure you wish to delete these [y/N]?')
             if val.lower() not in ('y', 'yes'):
@@ -66,6 +70,27 @@
         tree.unlock()
 
 
+def _filter_out_nested_bzrdirs(deletables):
+    result = []
+    for path, subp in deletables:
+        # bzr won't recurse into unknowns/ignored directories by default
+        # so we don't pay a penalty for checking subdirs of path for nested
+        # bzrdir.
+        # That said we won't detect the branch in the subdir of non-branch
+        # directory and therefore delete it. (worth to FIXME?)
+        if isdir(path):
+            try:
+                bzrdir.BzrDir.open(path)
+            except errors.NotBranchError:
+                result.append((path,subp))
+            else:
+                # TODO may be we need to notify user about skipped directories?
+                pass
+        else:
+            result.append((path,subp))
+    return result
+
+
 def delete_items(deletables, dry_run=False):
     """Delete files in the deletables iterable"""
     has_deleted = False

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2010-04-16 07:56:51 +0000
+++ bzrlib/commands.py	2010-05-11 11:05:49 +0000
@@ -407,8 +407,8 @@
 
     def __init__(self):
         """Construct an instance of this command."""
-        if self.__doc__ == Command.__doc__:
-            warn("No help message set for %r" % self)
+        if self.__doc__ == Command.__doc__ or not self.__doc__:
+            raise ValueError("No help message set for %r" % self)
         # List of standard options directly supported
         self.supported_std_options = []
         self._setup_run()
@@ -483,7 +483,9 @@
         """
         doc = self.help()
         if doc is None:
-            raise NotImplementedError("sorry, no detailed help yet for %r" % self.name())
+            raise NotImplementedError(
+                "self.help() returned None -  no detailed help yet for %r" %
+                self.name())
 
         # Extract the summary (purpose) and sections out from the text
         purpose,sections,order = self._get_help_parts(doc)
@@ -1077,15 +1079,9 @@
     if not opt_no_aliases:
         alias_argv = get_alias(argv[0])
         if alias_argv:
-            user_encoding = osutils.get_user_encoding()
-            alias_argv = [a.decode(user_encoding) for a in alias_argv]
             argv[0] = alias_argv.pop(0)
 
     cmd = argv.pop(0)
-    # We want only 'ascii' command names, but the user may have typed
-    # in a Unicode name. In that case, they should just get a
-    # 'command not found' error later.
-
     cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
     run = cmd_obj.run_argv_aliases
     run_argv = [argv, alias_argv]

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2010-04-30 11:35:43 +0000
+++ bzrlib/commit.py	2010-05-10 11:34:20 +0000
@@ -174,6 +174,43 @@
         self.reporter = reporter
         self.config = config
 
+    @staticmethod
+    def update_revprops(revprops, branch, authors=None, author=None,
+                        local=False, possible_master_transports=None):
+        if revprops is None:
+            revprops = {}
+        if possible_master_transports is None:
+            possible_master_transports = []
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = branch._get_nick(
+                local,
+                possible_master_transports)
+        if authors is not None:
+            if author is not None:
+                raise AssertionError('Specifying both author and authors '
+                        'is not allowed. Specify just authors instead')
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if authors:
+                for individual in authors:
+                    if '\n' in individual:
+                        raise AssertionError('\\n is not a valid character '
+                                'in an author identity')
+                revprops['authors'] = '\n'.join(authors)
+        if author is not None:
+            symbol_versioning.warn('The parameter author was deprecated'
+                   ' in version 1.13. Use authors instead',
+                   DeprecationWarning)
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if '\n' in author:
+                raise AssertionError('\\n is not a valid character '
+                        'in an author identity')
+            revprops['authors'] = author
+        return revprops
+
     def commit(self,
                message=None,
                timestamp=None,

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-04-24 09:08:27 +0000
+++ bzrlib/config.py	2010-05-07 06:35:34 +0000
@@ -257,12 +257,10 @@
 
         Something similar to 'Martin Pool <mbp@sourcefrog.net>'
 
-        $BZR_EMAIL can be set to override this (as well as the
-        deprecated $BZREMAIL), then
+        $BZR_EMAIL can be set to override this, then
         the concrete policy type is checked, and finally
         $EMAIL is examined.
-        If none is found, a reasonable default is (hopefully)
-        created.
+        If no username can be found, errors.NoWhoami exception is raised.
 
         TODO: Check it's reasonably well-formed.
         """
@@ -278,11 +276,14 @@
         if v:
             return v.decode(osutils.get_user_encoding())
 
-        name, email = _auto_user_id()
-        if name:
-            return '%s <%s>' % (name, email)
-        else:
-            return email
+        raise errors.NoWhoami()
+
+    def ensure_username(self):
+        """Raise errors.NoWhoami if username is not set.
+
+        This method relies on the username() function raising the error.
+        """
+        self.username()
 
     def signature_checking(self):
         """What is the current policy for signature checking?."""
@@ -899,79 +900,6 @@
         return os.path.expanduser('~/.cache')
 
 
-def _auto_user_id():
-    """Calculate automatic user identification.
-
-    Returns (realname, email).
-
-    Only used when none is set in the environment or the id file.
-
-    This previously used the FQDN as the default domain, but that can
-    be very slow on machines where DNS is broken.  So now we simply
-    use the hostname.
-    """
-    import socket
-
-    if sys.platform == 'win32':
-        name = win32utils.get_user_name_unicode()
-        if name is None:
-            raise errors.BzrError("Cannot autodetect user name.\n"
-                                  "Please, set your name with command like:\n"
-                                  'bzr whoami "Your Name <name@domain.com>"')
-        host = win32utils.get_host_name_unicode()
-        if host is None:
-            host = socket.gethostname()
-        return name, (name + '@' + host)
-
-    try:
-        import pwd
-        uid = os.getuid()
-        try:
-            w = pwd.getpwuid(uid)
-        except KeyError:
-            raise errors.BzrCommandError('Unable to determine your name.  '
-                'Please use "bzr whoami" to set it.')
-
-        # we try utf-8 first, because on many variants (like Linux),
-        # /etc/passwd "should" be in utf-8, and because it's unlikely to give
-        # false positives.  (many users will have their user encoding set to
-        # latin-1, which cannot raise UnicodeError.)
-        try:
-            gecos = w.pw_gecos.decode('utf-8')
-            encoding = 'utf-8'
-        except UnicodeError:
-            try:
-                encoding = osutils.get_user_encoding()
-                gecos = w.pw_gecos.decode(encoding)
-            except UnicodeError:
-                raise errors.BzrCommandError('Unable to determine your name.  '
-                   'Use "bzr whoami" to set it.')
-        try:
-            username = w.pw_name.decode(encoding)
-        except UnicodeError:
-            raise errors.BzrCommandError('Unable to determine your name.  '
-                'Use "bzr whoami" to set it.')
-
-        comma = gecos.find(',')
-        if comma == -1:
-            realname = gecos
-        else:
-            realname = gecos[:comma]
-        if not realname:
-            realname = username
-
-    except ImportError:
-        import getpass
-        try:
-            user_encoding = osutils.get_user_encoding()
-            realname = username = getpass.getuser().decode(user_encoding)
-        except UnicodeDecodeError:
-            raise errors.BzrError("Can't decode username as %s." % \
-                    user_encoding)
-
-    return realname, (username + '@' + socket.gethostname())
-
-
 def parse_username(username):
     """Parse e-mail username and return a (name, address) tuple."""
     match = re.match(r'(.*?)\s*<?([\w+.-]+@[\w+.-]+)>?', username)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-04-21 11:27:04 +0000
+++ bzrlib/errors.py	2010-05-13 17:32:55 +0000
@@ -1923,6 +1923,17 @@
 class CantMoveRoot(BzrError):
 
     _fmt = "Moving the root directory is not supported at this time"
+    
+    
+class TransformRenameFailed(BzrError):
+
+    _fmt = "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
+    
+    def __init__(self, from_path, to_path, why, errno):
+        self.from_path = from_path
+        self.to_path = to_path
+        self.why = why
+        self.errno = errno
 
 
 class BzrMoveFailedError(BzrError):
@@ -3134,3 +3145,10 @@
     def __init__(self, bzrdir):
         self.bzrdir = bzrdir
 
+class NoWhoami(BzrError):
+
+    _fmt = ('Unable to determine your name.\n'
+        "Please, set your name with the 'whoami' command.\n"
+        'E.g. bzr whoami "Your Name <name@example.com>"')
+
+

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-03-02 10:21:39 +0000
+++ bzrlib/help_topics/__init__.py	2010-04-30 01:39:46 +0000
@@ -591,22 +591,27 @@
 _env_variables = \
 """Environment Variables
 
-================ =================================================================
-BZRPATH          Path where bzr is to look for shell plugin external commands.
-BZR_EMAIL        E-Mail address of the user. Overrides EMAIL.
-EMAIL            E-Mail address of the user.
-BZR_EDITOR       Editor for editing commit messages. Overrides EDITOR.
-EDITOR           Editor for editing commit messages.
-BZR_PLUGIN_PATH  Paths where bzr should look for plugins.
-BZR_HOME         Directory holding .bazaar config dir. Overrides HOME.
-BZR_HOME (Win32) Directory holding bazaar config dir. Overrides APPDATA and HOME.
-BZR_REMOTE_PATH  Full name of remote 'bzr' command (for bzr+ssh:// URLs).
-BZR_SSH          Path to SSH client, or one of paramiko, openssh, sshcorp, plink.
-BZR_LOG          Location of .bzr.log (use '/dev/null' to suppress log).
-BZR_LOG (Win32)  Location of .bzr.log (use 'NUL' to suppress log).
-BZR_COLUMNS      Override implicit terminal width.
-BZR_CONCURRENCY  Number of processes that can be run concurrently (selftest).
-================ =================================================================
+=================== =======================================================================
+BZRPATH             Path where bzr is to look for shell plugin external commands.
+BZR_EMAIL           E-Mail address of the user. Overrides EMAIL.
+EMAIL               E-Mail address of the user.
+BZR_EDITOR          Editor for editing commit messages. Overrides EDITOR.
+EDITOR              Editor for editing commit messages.
+BZR_PLUGIN_PATH     Paths where bzr should look for plugins.
+BZR_DISABLE_PLUGINS Plugins that bzr should not load.
+BZR_PLUGINS_AT      Plugins to load from a directory not in BZR_PLUGIN_PATH.
+BZR_HOME            Directory holding .bazaar config dir. Overrides HOME.
+BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA and HOME.
+BZR_REMOTE_PATH     Full name of remote 'bzr' command (for bzr+ssh:// URLs).
+BZR_SSH             Path to SSH client, or one of paramiko, openssh, sshcorp, plink.
+BZR_LOG             Location of .bzr.log (use '/dev/null' to suppress log).
+BZR_LOG (Win32)     Location of .bzr.log (use 'NUL' to suppress log).
+BZR_COLUMNS         Override implicit terminal width.
+BZR_CONCURRENCY     Number of processes that can be run concurrently (selftest).
+BZR_PROGRESS_BAR    Override the progress display. Values are 'none', 'dots', or 'tty'.
+BZR_PDB             Control whether to launch a debugger on error.
+BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a breakin debugger.
+=================== =======================================================================
 """
 
 

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2010-04-23 07:15:23 +0000
+++ bzrlib/help_topics/en/configuration.txt	2010-05-04 14:52:33 +0000
@@ -59,6 +59,19 @@
 
 Path to the editor Bazaar should use for commit messages, etc.
 
+BZR_LOG
+~~~~~~~
+
+Location of the Bazaar log file. You can check the current location by
+running ``bzr version``.
+
+The log file contains debug information that is useful for diagnosing or
+reporting problems with Bazaar.
+
+Setting this to ``NUL`` on Windows or ``/dev/null`` on other platforms
+will disable logging.
+
+
 BZR_PLUGIN_PATH
 ~~~~~~~~~~~~~~~
 

=== modified file 'bzrlib/ignores.py'
--- bzrlib/ignores.py	2010-04-30 07:54:33 +0000
+++ bzrlib/ignores.py	2010-05-03 04:51:58 +0000
@@ -17,6 +17,8 @@
 """Lists of ignore files, etc."""
 
 import errno
+import os
+from cStringIO import StringIO
 
 import bzrlib
 from bzrlib import (
@@ -181,28 +183,44 @@
     The ignore file will be automatically added under version control.
 
     :param tree: Working tree to update the ignore list.
+    :param name_pattern_list: List of ignore patterns.
+    :return: None
     """
+    # read in the existing ignores set
     ifn = tree.abspath(bzrlib.IGNORE_FILENAME)
     if tree.has_filename(ifn):
-        f = open(ifn, 'rt')
+        f = open(ifn, 'rU')
         try:
-            igns = f.read().decode('utf-8')
+            file_contents = f.read()
+            # figure out what kind of line endings are used
+            newline = getattr(f, 'newlines', None)
+            if type(newline) is tuple:
+                newline = newline[0]
+            elif newline is None:
+                newline = os.linesep
         finally:
             f.close()
     else:
-        igns = ""
-
-    # TODO: If the file already uses crlf-style termination, maybe
-    # we should use that for the newly added lines?
-
-    if igns and igns[-1] != '\n':
-        igns += '\n'
-    for name_pattern in name_pattern_list:
-        igns += name_pattern + '\n'
-
+        file_contents = ""
+        newline = os.linesep
+    
+    sio = StringIO(file_contents)
+    try:
+        ignores = parse_ignore_file(sio)
+    finally:
+        sio.close()
+    
+    # write out the updated ignores set
     f = atomicfile.AtomicFile(ifn, 'wb')
     try:
-        f.write(igns.encode('utf-8'))
+        # write the original contents, preserving original line endings
+        f.write(newline.join(file_contents.split('\n')))
+        if len(file_contents) > 0 and not file_contents.endswith('\n'):
+            f.write(newline)
+        for pattern in name_pattern_list:
+            if not pattern in ignores:
+                f.write(pattern.encode('utf-8'))
+                f.write(newline)
         f.commit()
     finally:
         f.close()

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2010-02-17 17:11:16 +0000
+++ bzrlib/inventory.py	2010-05-11 07:52:16 +0000
@@ -1652,11 +1652,7 @@
             # parent_to_children with at least the tree root.)
             return other
         cache = self._fileid_to_entry_cache
-        try:
-            remaining_children = collections.deque(parent_to_children[self.root_id])
-        except:
-            import pdb; pdb.set_trace()
-            raise
+        remaining_children = collections.deque(parent_to_children[self.root_id])
         while remaining_children:
             file_id = remaining_children.popleft()
             ie = cache[file_id]

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2010-04-16 07:56:51 +0000
+++ bzrlib/lock.py	2010-05-11 08:36:16 +0000
@@ -88,6 +88,20 @@
                              self.lock_url, self.details)
 
 
+class LogicalLockResult(object):
+    """The result of a lock_read/lock_write/lock_tree_write call on lockables.
+
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock):
+        self.unlock = unlock
+
+    def __repr__(self):
+        return "LogicalLockResult(%s)" % (self.unlock)
+
+
+
 def cant_unlock_not_held(locked_object):
     """An attempt to unlock failed because the object was not locked.
 

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2010-03-03 22:59:21 +0000
+++ bzrlib/lockdir.py	2010-04-28 06:09:23 +0000
@@ -447,9 +447,9 @@
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
         try:
-            user = config.user_email()
-        except errors.NoEmailInUsername:
             user = config.username()
+        except errors.NoWhoami:
+            user = osutils.getuser_unicode()
         s = rio.Stanza(hostname=get_host_name(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2010-04-14 12:30:17 +0000
+++ bzrlib/log.py	2010-05-11 08:44:59 +0000
@@ -1341,7 +1341,7 @@
 
     def __init__(self, to_file, show_ids=False, show_timezone='original',
                  delta_format=None, levels=None, show_advice=False,
-                 to_exact_file=None):
+                 to_exact_file=None, author_list_handler=None):
         """Create a LogFormatter.
 
         :param to_file: the file to output to
@@ -1355,6 +1355,8 @@
           let the log formatter decide.
         :param show_advice: whether to show advice at the end of the
           log or not
+        :param author_list_handler: callable generating a list of
+          authors to display for a given revision
         """
         self.to_file = to_file
         # 'exact' stream used to show diff, it should print content 'as is'
@@ -1375,6 +1377,7 @@
         self.levels = levels
         self._show_advice = show_advice
         self._merge_count = 0
+        self._author_list_handler = author_list_handler
 
     def get_levels(self):
         """Get the number of levels to display or 0 for all."""
@@ -1412,10 +1415,41 @@
         return address
 
     def short_author(self, rev):
-        name, address = config.parse_username(rev.get_apparent_authors()[0])
-        if name:
-            return name
-        return address
+        return self.authors(rev, 'first', short=True, sep=', ')
+
+    def authors(self, rev, who, short=False, sep=None):
+        """Generate list of authors, taking --authors option into account.
+
+        The caller has to specify the name of a author list handler,
+        as provided by the author list registry, using the ``who``
+        argument.  That name only sets a default, though: when the
+        user selected a different author list generation using the
+        ``--authors`` command line switch, as represented by the
+        ``author_list_handler`` constructor argument, that value takes
+        precedence.
+
+        :param rev: The revision for which to generate the list of authors.
+        :param who: Name of the default handler.
+        :param short: Whether to shorten names to either name or address.
+        :param sep: What separator to use for automatic concatenation.
+        """
+        if self._author_list_handler is not None:
+            # The user did specify --authors, which overrides the default
+            author_list_handler = self._author_list_handler
+        else:
+            # The user didn't specify --authors, so we use the caller's default
+            author_list_handler = author_list_registry.get(who)
+        names = author_list_handler(rev)
+        if short:
+            for i in range(len(names)):
+                name, address = config.parse_username(names[i])
+                if name:
+                    names[i] = name
+                else:
+                    names[i] = address
+        if sep is not None:
+            names = sep.join(names)
+        return names
 
     def merge_marker(self, revision):
         """Get the merge marker to include in the output or '' if none."""
@@ -1523,7 +1557,7 @@
         lines.extend(self.custom_properties(revision.rev))
 
         committer = revision.rev.committer
-        authors = revision.rev.get_apparent_authors()
+        authors = self.authors(revision.rev, 'all')
         if authors != [committer]:
             lines.append('author: %s' % (", ".join(authors),))
         lines.append('committer: %s' % (committer,))
@@ -1703,7 +1737,8 @@
                                self.show_timezone,
                                date_fmt='%Y-%m-%d',
                                show_offset=False)
-        committer_str = revision.rev.get_apparent_authors()[0].replace (' <', '  <')
+        committer_str = self.authors(revision.rev, 'first', sep=', ')
+        committer_str = committer_str.replace(' <', '  <')
         to_file.write('%s  %s\n\n' % (date_str,committer_str))
 
         if revision.delta is not None and revision.delta.has_changed():
@@ -1774,6 +1809,34 @@
         raise errors.BzrCommandError("unknown log formatter: %r" % name)
 
 
+def author_list_all(rev):
+    return rev.get_apparent_authors()[:]
+
+
+def author_list_first(rev):
+    lst = rev.get_apparent_authors()
+    try:
+        return [lst[0]]
+    except IndexError:
+        return []
+
+
+def author_list_committer(rev):
+    return [rev.committer]
+
+
+author_list_registry = registry.Registry()
+
+author_list_registry.register('all', author_list_all,
+                              'All authors')
+
+author_list_registry.register('first', author_list_first,
+                              'The first author')
+
+author_list_registry.register('committer', author_list_committer,
+                              'The committer')
+
+
 def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
     # deprecated; for compatibility
     lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
@@ -1930,7 +1993,7 @@
         lf.log_revision(lr)
 
 
-def _get_info_for_log_files(revisionspec_list, file_list):
+def _get_info_for_log_files(revisionspec_list, file_list, add_cleanup):
     """Find file-ids and kinds given a list of files and a revision range.
 
     We search for files at the end of the range. If not found there,
@@ -1940,6 +2003,8 @@
     :param file_list: the list of paths given on the command line;
       the first of these can be a branch location or a file path,
       the remainder must be file paths
+    :param add_cleanup: When the branch returned is read locked,
+      an unlock call will be queued to the cleanup.
     :return: (branch, info_list, start_rev_info, end_rev_info) where
       info_list is a list of (relative_path, file_id, kind) tuples where
       kind is one of values 'directory', 'file', 'symlink', 'tree-reference'.
@@ -1947,7 +2012,7 @@
     """
     from builtins import _get_revision_range, safe_relpath_files
     tree, b, path = bzrdir.BzrDir.open_containing_tree_or_branch(file_list[0])
-    b.lock_read()
+    add_cleanup(b.lock_read().unlock)
     # XXX: It's damn messy converting a list of paths to relative paths when
     # those paths might be deleted ones, they might be on a case-insensitive
     # filesystem and/or they might be in silly locations (like another branch).

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2010-04-22 14:18:17 +0000
+++ bzrlib/mutabletree.py	2010-05-10 11:34:20 +0000
@@ -182,39 +182,14 @@
                **kwargs):
         # avoid circular imports
         from bzrlib import commit
-        if revprops is None:
-            revprops = {}
         possible_master_transports=[]
-        if not 'branch-nick' in revprops:
-            revprops['branch-nick'] = self.branch._get_nick(
+        revprops = commit.Commit.update_revprops(
+                revprops,
+                self.branch,
+                kwargs.pop('authors', None),
+                kwargs.pop('author', None),
                 kwargs.get('local', False),
                 possible_master_transports)
-        authors = kwargs.pop('authors', None)
-        author = kwargs.pop('author', None)
-        if authors is not None:
-            if author is not None:
-                raise AssertionError('Specifying both author and authors '
-                        'is not allowed. Specify just authors instead')
-            if 'author' in revprops or 'authors' in revprops:
-                # XXX: maybe we should just accept one of them?
-                raise AssertionError('author property given twice')
-            if authors:
-                for individual in authors:
-                    if '\n' in individual:
-                        raise AssertionError('\\n is not a valid character '
-                                'in an author identity')
-                revprops['authors'] = '\n'.join(authors)
-        if author is not None:
-            symbol_versioning.warn('The parameter author was deprecated'
-                   ' in version 1.13. Use authors instead',
-                   DeprecationWarning)
-            if 'author' in revprops or 'authors' in revprops:
-                # XXX: maybe we should just accept one of them?
-                raise AssertionError('author property given twice')
-            if '\n' in author:
-                raise AssertionError('\\n is not a valid character '
-                        'in an author identity')
-            revprops['authors'] = author
         # args for wt.commit start at message from the Commit.commit method,
         args = (message, ) + args
         for hook in MutableTree.hooks['start_commit']:

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2010-04-20 08:47:38 +0000
+++ bzrlib/option.py	2010-05-05 04:08:11 +0000
@@ -506,6 +506,9 @@
 # Declare the standard options
 _standard_option('help', short_name='h',
                  help='Show help message.')
+_standard_option('null', short_name='0',
+                 help='Use an ASCII NUL (\\0) separator rather than '
+                      'a newline.')
 _standard_option('usage',
                  help='Show usage message and options.')
 _standard_option('verbose', short_name='v',
@@ -570,6 +573,8 @@
 _global_option('dry-run',
                help="Show what would be done, but don't actually do anything.")
 _global_option('name-from-revision', help='The path name in the old tree.')
+_global_option('directory', short_name='d', type=unicode,
+               help='Branch to operate on, instead of working directory')
 
 diff_writer_registry = _mod_registry.Registry()
 diff_writer_registry.register('plain', lambda x: x, 'Plaintext diff output.')

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-05-03 06:25:46 +0000
+++ bzrlib/osutils.py	2010-05-13 17:32:55 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
 import os
 import re
 import stat
@@ -25,7 +26,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from datetime import datetime
-import errno
+import getpass
 from ntpath import (abspath as _nt_abspath,
                     join as _nt_join,
                     normpath as _nt_normpath,
@@ -360,14 +361,6 @@
     return _win32_fixdrive(tempfile.mkdtemp(*args, **kwargs).replace('\\', '/'))
 
 
-def _add_rename_error_details(e, old, new):
-    new_e = OSError(e.errno, "failed to rename %s to %s: %s"
-        % (old, new, e.strerror))
-    new_e.filename = old
-    new_e.to_filename = new
-    return new_e
-
-
 def _win32_rename(old, new):
     """We expect to be able to atomically replace 'new' with old.
 
@@ -375,7 +368,7 @@
     and then deleted.
     """
     try:
-        fancy_rename(old, new, rename_func=_wrapped_rename, unlink_func=os.unlink)
+        fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
     except OSError, e:
         if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY, errno.EINVAL):
             # If we try to rename a non-existant file onto cwd, we get
@@ -386,16 +379,6 @@
         raise
 
 
-def _wrapped_rename(old, new):
-    """Rename a file or directory"""
-    try:
-        os.rename(old, new)
-    except (IOError, OSError), e:
-        # this is eventually called by all rename-like functions, so should 
-        # catch all of them
-        raise _add_rename_error_details(e, old, new)
-
-
 def _mac_getcwd():
     return unicodedata.normalize('NFC', os.getcwdu())
 
@@ -406,8 +389,8 @@
 realpath = _posix_realpath
 pathjoin = os.path.join
 normpath = os.path.normpath
-rename = _wrapped_rename # overridden below on win32
 getcwd = os.getcwdu
+rename = os.rename
 dirname = os.path.dirname
 basename = os.path.basename
 split = os.path.split
@@ -2302,3 +2285,15 @@
         return os.fdopen(os.open(filename, flags), mode, bufsize)
 else:
     open_file = open
+
+
+def getuser_unicode():
+    """Return the username as unicode.
+    """
+    try:
+        user_encoding = get_user_encoding()
+        username = getpass.getuser().decode(user_encoding)
+    except UnicodeDecodeError:
+        raise errors.BzrError("Can't decode username as %s." % \
+                user_encoding)
+    return username

=== added directory 'bzrlib/plugins/bash_completion'
=== added file 'bzrlib/plugins/bash_completion/README.txt'
--- bzrlib/plugins/bash_completion/README.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/README.txt	2010-04-22 06:43:53 +0000
@@ -0,0 +1,201 @@
+.. comment
+
+  Copyright (C) 2010 Canonical Ltd
+
+  This file is part of bzr-bash-completion
+
+  bzr-bash-completion free software: you can redistribute it and/or
+  modify it under the terms of the GNU General Public License as
+  published by the Free Software Foundation, either version 2 of the
+  License, or (at your option) any later version.
+
+  bzr-bash-completion is distributed in the hope that it will be
+  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+=====================================
+bzr bash-completion script and plugin
+=====================================
+
+This script generates a shell function which can be used by bash to
+automatically complete the currently typed command when the user
+presses the completion key (usually tab).
+
+It is intended as a bzr plugin, but can be used to some extend as a
+standalone python script as well.
+
+| Copyright (C) 2009, 2010 Canonical Ltd
+
+.. contents::
+
+----------
+Installing
+----------
+
+You only need to do this if you want to use the script as a bzr
+plugin.  Otherwise simply grab the bashcomp.py and place it wherever
+you want.
+
+Installing from bzr repository
+------------------------------
+
+To check out the current code from launchpad, use the following commands::
+
+  mkdir -p ~/.bazaar/plugins
+  cd ~/.bazaar/plugins
+  bzr checkout lp:bzr-bash-completion bash_completion
+
+To update such an installation, execute this command::
+
+  bzr update ~/.bazaar/plugins/bash_completion
+
+Installing using easy_install
+-----------------------------
+
+The following command should install the latest release of the plugin
+on your system::
+
+  easy_install bzr-bash-completion
+
+To use this method, you need to have `Easy Install`_ installed and
+also have write access to the required directories. So maybe you
+should execute this command as root or through sudo_. Or you want to
+`install to a different location`_.
+
+.. _Easy Install: http://peak.telecommunity.com/DevCenter/EasyInstall
+.. _sudo: http://linux.die.net/man/8/sudo
+.. _install to a different location:
+   http://peak.telecommunity.com/DevCenter/EasyInstall#non-root-installation
+
+Installing from tarball
+-----------------------
+
+If you have grabbed a source code tarball, or want to install from a
+bzr checkout in a different place than your bazaar plugins directory,
+then you should use the ``setup.py`` script shipped with the code::
+
+  ./setup.py install
+
+If you want to install the plugin only for your own user account, you
+might wish to pass the option ``--user`` or ``--home=$HOME`` to that
+command. For further information please read the manuals of distutils_
+as well as setuptools_ or distribute_, whatever is available on your
+system, or have a look at the command line help::
+
+  ./setup.py install --help
+
+.. _distutils: http://docs.python.org/install/index.html
+.. _setuptools: http://peak.telecommunity.com/DevCenter/setuptools#what-your-users-should-know
+.. _distribute: http://packages.python.org/distribute/setuptools.html#what-your-users-should-know
+
+-----
+Using
+-----
+
+Using as a plugin
+-----------------
+
+This is the preferred method of generating the completion function, as
+it will ensure proper bzr initialization.
+
+::
+
+  eval "`bzr bash-completion`"
+
+Lazy initialization
+-------------------
+
+Running the above command automatically from your ``~/.bashrc`` file
+or similar can cause annoying delays in the startup of your shell.
+To avoid this problem, you can delay the generation of the completion
+function until you actually need it.
+
+To do so, source the file ``lazy.sh`` shipped with this package from
+your ``~/.bashrc`` file or add it to your ``~/.bash_completion`` if
+your setup uses such a file. On a system-wide installation, the
+directory ``/usr/share/bash-completion/`` might contain such bash
+completion scripts.
+
+If you installed bzr-bash-completion from the repository or a source
+tarball, you find the ``lazy.sh`` script in the root of the source
+tree. If you installed the plugin using easy_install, you should grab
+the script manually from the bzr repository, e.g. through the bazaar
+web interface on launchpad.
+
+Note that the full completion function is generated only once per
+shell session. If you update your bzr installation or change the set
+of installed plugins, then you might wish to regenerate the completion
+function manually as described above in order for completion to take
+these changes into account.
+
+Using as a script
+-----------------
+
+As an alternative, if bzrlib is available to python scripts, the
+following invocation should yield the same results without requiring
+you to add a plugin::
+
+  eval "`./bashcomp.py`"
+
+This approach might have some issues, though, and provides less
+options than the bzr plugin. Therefore if you have the choice, go for
+the plugin setup.
+
+--------------
+Design concept
+--------------
+
+The plugin (or script) is designed to generate a completion function
+containing all the required information about the possible
+completions. This is usually only done once when bash
+initializes. After that, no more invocations of bzr are required. This
+makes the function much faster than a possible implementation talking
+to bzr for each and every completion. On the other hand, this has the
+effect that updates to bzr or its plugins won't show up in the
+completions immediately, but only after the completion function has
+been regenerated.
+
+-------
+License
+-------
+
+As this is built upon a bash completion script originally included in
+the bzr source tree, and as the bzr sources are covered by the GPL 2,
+this script here is licensed under these same terms.
+
+If you require a more liberal license, you'll have to contact all
+those who contributed code to this plugin, be it for bash or for
+python.
+
+.. cut long_description here
+
+-------
+History
+-------
+
+The plugin was created by Martin von Gagern in 2009, building on a
+static completion function of very limited scope distributed together
+with bzr.
+
+----------
+References
+----------
+
+Plugin homepages
+  | https://launchpad.net/bzr-bash-completion
+  | http://pypi.python.org/pypi/bzr-bash-completion
+Bazaar homepage
+  | http://bazaar.canonical.com/
+
+
+
+.. vim: ft=rst
+
+.. emacs
+   Local Variables:
+   mode: rst
+   End:

=== added file 'bzrlib/plugins/bash_completion/__init__.py'
--- bzrlib/plugins/bash_completion/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/__init__.py	2010-05-05 08:08:48 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2009, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+__doc__ = """Generate a shell function for bash command line completion.
+
+This plugin provides a command called bash-completion that generates a
+bash completion function for bzr. See its documentation for details.
+"""
+
+from bzrlib import commands, version_info
+
+
+bzr_plugin_name = 'bash_completion'
+bzr_commands = [ 'bash-completion' ]
+
+commands.plugin_cmds.register_lazy('cmd_bash_completion', [],
+                                   'bzrlib.plugins.bash_completion.bashcomp')
+
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'tests',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/bash_completion/bashcomp.py'
--- bzrlib/plugins/bash_completion/bashcomp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/bashcomp.py	2010-05-05 08:08:48 +0000
@@ -0,0 +1,463 @@
+#!/usr/bin/env python
+
+# Copyright (C) 2009, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib import (
+    commands,
+    config,
+    help_topics,
+    option,
+    plugin,
+)
+import bzrlib
+import re
+
+
+class BashCodeGen(object):
+    """Generate a bash script for given completion data."""
+
+    def __init__(self, data, function_name='_bzr', debug=False):
+        self.data = data
+        self.function_name = function_name
+        self.debug = debug
+
+    def script(self):
+        return ("""\
+# Programmable completion for the Bazaar-NG bzr command under bash.
+# Known to work with bash 2.05a as well as bash 4.1.2, and probably
+# all versions in between as well.
+
+# Based originally on the svn bash completition script.
+# Customized by Sven Wilhelm/Icecrash.com
+# Adjusted for automatic generation by Martin von Gagern
+
+# Generated using the bash_completion plugin.
+# See https://launchpad.net/bzr-bash-completion for details.
+
+# Commands and options of bzr %(bzr_version)s
+
+shopt -s progcomp
+%(function)s
+complete -F %(function_name)s -o default bzr
+"""     % {
+            "function_name": self.function_name,
+            "function": self.function(),
+            "bzr_version": self.bzr_version(),
+        })
+
+    def function(self):
+        return ("""\
+%(function_name)s ()
+{
+	local cur cmds cmdIdx cmd cmdOpts fixedWords i globalOpts
+	local curOpt optEnums
+
+	COMPREPLY=()
+	cur=${COMP_WORDS[COMP_CWORD]}
+
+	cmds='%(cmds)s'
+	globalOpts='%(global_options)s'
+
+	# do ordinary expansion if we are anywhere after a -- argument
+	for ((i = 1; i < COMP_CWORD; ++i)); do
+		[[ ${COMP_WORDS[i]} == "--" ]] && return 0
+	done
+
+	# find the command; it's the first word not starting in -
+	cmd=
+	for ((cmdIdx = 1; cmdIdx < ${#COMP_WORDS[@]}; ++cmdIdx)); do
+		if [[ ${COMP_WORDS[cmdIdx]} != -* ]]; then
+			cmd=${COMP_WORDS[cmdIdx]}
+			break
+		fi
+	done
+
+	# complete command name if we are not already past the command
+	if [[ $COMP_CWORD -le cmdIdx ]]; then
+		COMPREPLY=( $( compgen -W "$cmds $globalOpts" -- $cur ) )
+		return 0
+	fi
+
+	# find the option for which we want to complete a value
+	curOpt=
+	if [[ $cur != -* ]] && [[ $COMP_CWORD -gt 1 ]]; then
+		curOpt=${COMP_WORDS[COMP_CWORD - 1]}
+		if [[ $curOpt == = ]]; then
+			curOpt=${COMP_WORDS[COMP_CWORD - 2]}
+		elif [[ $cur == : ]]; then
+			cur=
+			curOpt="$curOpt:"
+		elif [[ $curOpt == : ]]; then
+			curOpt=${COMP_WORDS[COMP_CWORD - 2]}:
+		fi
+	fi
+%(debug)s
+	cmdOpts=
+	optEnums=
+	fixedWords=
+	case $cmd in
+%(cases)s\
+	*)
+		cmdOpts='--help -h'
+		;;
+	esac
+
+	if [[ -z $fixedWords ]] && [[ -z $optEnums ]] && [[ $cur != -* ]]; then
+		case $curOpt in
+			tag:*)
+				fixedWords="$(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//')"
+				;;
+		esac
+	elif [[ $cur == = ]] && [[ -n $optEnums ]]; then
+		# complete directly after "--option=", list all enum values
+		COMPREPLY=( $optEnums )
+		return 0
+	else
+		fixedWords="$cmdOpts $globalOpts $optEnums $fixedWords"
+	fi
+
+	if [[ -n $fixedWords ]]; then
+		COMPREPLY=( $( compgen -W "$fixedWords" -- $cur ) )
+	fi
+
+	return 0
+}
+"""     % {
+            "cmds": self.command_names(),
+            "function_name": self.function_name,
+            "cases": self.command_cases(),
+            "global_options": self.global_options(),
+            "debug": self.debug_output(),
+        })
+
+    def command_names(self):
+        return " ".join(self.data.all_command_aliases())
+
+    def debug_output(self):
+        if not self.debug:
+            return ''
+        else:
+            return (r"""
+	# Debugging code enabled using the --debug command line switch.
+	# Will dump some variables to the top portion of the terminal.
+	echo -ne '\e[s\e[H'
+	for (( i=0; i < ${#COMP_WORDS[@]}; ++i)); do
+		echo "\$COMP_WORDS[$i]='${COMP_WORDS[i]}'"$'\e[K'
+	done
+	for i in COMP_CWORD COMP_LINE COMP_POINT COMP_TYPE COMP_KEY cur curOpt; do
+		echo "\$${i}=\"${!i}\""$'\e[K'
+	done
+	echo -ne '---\e[K\e[u'
+""")
+
+    def bzr_version(self):
+        bzr_version = bzrlib.version_string
+        if not self.data.plugins:
+            bzr_version += "."
+        else:
+            bzr_version += " and the following plugins:"
+            for name, plugin in sorted(self.data.plugins.iteritems()):
+                bzr_version += "\n# %s" % plugin
+        return bzr_version
+
+    def global_options(self):
+        return " ".join(sorted(self.data.global_options))
+
+    def command_cases(self):
+        cases = ""
+        for command in self.data.commands:
+            cases += self.command_case(command)
+        return cases
+
+    def command_case(self, command):
+        case = "\t%s)\n" % "|".join(command.aliases)
+        if command.plugin:
+            case += "\t\t# plugin \"%s\"\n" % command.plugin
+        options = []
+        enums = []
+        for option in command.options:
+            for message in option.error_messages:
+                case += "\t\t# %s\n" % message
+            if option.registry_keys:
+                for key in option.registry_keys:
+                    options.append("%s=%s" % (option, key))
+                enums.append("%s) optEnums='%s' ;;" %
+                             (option, ' '.join(option.registry_keys)))
+            else:
+                options.append(str(option))
+        case += "\t\tcmdOpts='%s'\n" % " ".join(options)
+        if command.fixed_words:
+            fixed_words = command.fixed_words
+            if isinstance(fixed_words, list):
+                fixed_words = "'%s'" + ' '.join(fixed_words)
+            case += "\t\tfixedWords=%s\n" % fixed_words
+        if enums:
+            case += "\t\tcase $curOpt in\n\t\t\t"
+            case += "\n\t\t\t".join(enums)
+            case += "\n\t\tesac\n"
+        case += "\t\t;;\n"
+        return case
+
+
+class CompletionData(object):
+
+    def __init__(self):
+        self.plugins = {}
+        self.global_options = set()
+        self.commands = []
+
+    def all_command_aliases(self):
+        for c in self.commands:
+            for a in c.aliases:
+                yield a
+
+
+class CommandData(object):
+
+    def __init__(self, name):
+        self.name = name
+        self.aliases = [name]
+        self.plugin = None
+        self.options = []
+        self.fixed_words = None
+
+
+class PluginData(object):
+
+    def __init__(self, name, version=None):
+        if version is None:
+            version = bzrlib.plugin.plugins()[name].__version__
+        self.name = name
+        self.version = version
+
+    def __str__(self):
+        if self.version == 'unknown':
+            return self.name
+        return '%s %s' % (self.name, self.version)
+
+
+class OptionData(object):
+
+    def __init__(self, name):
+        self.name = name
+        self.registry_keys = None
+        self.error_messages = []
+
+    def __str__(self):
+        return self.name
+
+    def __cmp__(self, other):
+        return cmp(self.name, other.name)
+
+
+class DataCollector(object):
+
+    def __init__(self, no_plugins=False, selected_plugins=None):
+        self.data = CompletionData()
+        self.user_aliases = {}
+        if no_plugins:
+            self.selected_plugins = set()
+        elif selected_plugins is None:
+            self.selected_plugins = None
+        else:
+            self.selected_plugins = set([x.replace('-', '_')
+                                         for x in selected_plugins])
+
+    def collect(self):
+        self.global_options()
+        self.aliases()
+        self.commands()
+        return self.data
+
+    def global_options(self):
+        re_switch = re.compile(r'\n(--[A-Za-z0-9-_]+)(?:, (-\S))?\s')
+        help_text = help_topics.topic_registry.get_detail('global-options')
+        for long, short in re_switch.findall(help_text):
+            self.data.global_options.add(long)
+            if short:
+                self.data.global_options.add(short)
+
+    def aliases(self):
+        for alias, expansion in config.GlobalConfig().get_aliases().iteritems():
+            for token in commands.shlex_split_unicode(expansion):
+                if not token.startswith("-"):
+                    self.user_aliases.setdefault(token, set()).add(alias)
+                    break
+
+    def commands(self):
+        for name in sorted(commands.all_command_names()):
+            self.command(name)
+
+    def command(self, name):
+        cmd = commands.get_cmd_object(name)
+        cmd_data = CommandData(name)
+
+        plugin_name = cmd.plugin_name()
+        if plugin_name is not None:
+            if (self.selected_plugins is not None and
+                plugin not in self.selected_plugins):
+                return None
+            plugin_data = self.data.plugins.get(plugin_name)
+            if plugin_data is None:
+                plugin_data = PluginData(plugin_name)
+                self.data.plugins[plugin_name] = plugin_data
+            cmd_data.plugin = plugin_data
+        self.data.commands.append(cmd_data)
+
+        # Find all aliases to the command; both cmd-defined and user-defined.
+        # We assume a user won't override one command with a different one,
+        # but will choose completely new names or add options to existing
+        # ones while maintaining the actual command name unchanged.
+        cmd_data.aliases.extend(cmd.aliases)
+        cmd_data.aliases.extend(sorted([useralias
+            for cmdalias in cmd_data.aliases
+            if cmdalias in self.user_aliases
+            for useralias in self.user_aliases[cmdalias]
+            if useralias not in cmd_data.aliases]))
+
+        opts = cmd.options()
+        for optname, opt in sorted(opts.iteritems()):
+            cmd_data.options.extend(self.option(opt))
+
+        if 'help' == name or 'help' in cmd.aliases:
+            cmd_data.fixed_words = ('"$cmds %s"' %
+                " ".join(sorted(help_topics.topic_registry.keys())))
+
+        return cmd_data
+
+    def option(self, opt):
+        optswitches = {}
+        parser = option.get_optparser({opt.name: opt})
+        parser = self.wrap_parser(optswitches, parser)
+        optswitches.clear()
+        opt.add_option(parser, opt.short_name())
+        if isinstance(opt, option.RegistryOption) and opt.enum_switch:
+            enum_switch = '--%s' % opt.name
+            enum_data = optswitches.get(enum_switch)
+            if enum_data:
+                try:
+                    enum_data.registry_keys = opt.registry.keys()
+                except ImportError, e:
+                    enum_data.error_messages.append(
+                        "ERROR getting registry keys for '--%s': %s"
+                        % (opt.name, str(e).split('\n')[0]))
+        return sorted(optswitches.values())
+
+    def wrap_container(self, optswitches, parser):
+        def tweaked_add_option(*opts, **attrs):
+            for name in opts:
+                optswitches[name] = OptionData(name)
+        parser.add_option = tweaked_add_option
+        return parser
+
+    def wrap_parser(self, optswitches, parser):
+        orig_add_option_group = parser.add_option_group
+        def tweaked_add_option_group(*opts, **attrs):
+            return self.wrap_container(optswitches,
+                orig_add_option_group(*opts, **attrs))
+        parser.add_option_group = tweaked_add_option_group
+        return self.wrap_container(optswitches, parser)
+
+
+def bash_completion_function(out, function_name="_bzr", function_only=False,
+                             debug=False,
+                             no_plugins=False, selected_plugins=None):
+    dc = DataCollector(no_plugins=no_plugins, selected_plugins=selected_plugins)
+    data = dc.collect()
+    cg = BashCodeGen(data, function_name=function_name, debug=debug)
+    if function_only:
+        res = cg.function()
+    else:
+        res = cg.script()
+    out.write(res)
+
+
+class cmd_bash_completion(commands.Command):
+    __doc__ = """Generate a shell function for bash command line completion.
+
+    This command generates a shell function which can be used by bash to
+    automatically complete the currently typed command when the user presses
+    the completion key (usually tab).
+    
+    Commonly used like this:
+        eval "`bzr bash-completion`"
+    """
+
+    takes_options = [
+        option.Option("function-name", short_name="f", type=str, argname="name",
+               help="Name of the generated function (default: _bzr)"),
+        option.Option("function-only", short_name="o", type=None,
+               help="Generate only the shell function, don't enable it"),
+        option.Option("debug", type=None, hidden=True,
+               help="Enable shell code useful for debugging"),
+        option.ListOption("plugin", type=str, argname="name",
+                # param_name="selected_plugins", # doesn't work, bug #387117
+                help="Enable completions for the selected plugin"
+                + " (default: all plugins)"),
+        ]
+
+    def run(self, **kwargs):
+        import sys
+        from bashcomp import bash_completion_function
+        if 'plugin' in kwargs:
+            # work around bug #387117 which prevents us from using param_name
+            if len(kwargs['plugin']) > 0:
+                kwargs['selected_plugins'] = kwargs['plugin']
+            del kwargs['plugin']
+        bash_completion_function(sys.stdout, **kwargs)
+
+
+if __name__ == '__main__':
+
+    import sys
+    import locale
+    import optparse
+
+    def plugin_callback(option, opt, value, parser):
+        values = parser.values.selected_plugins
+        if value == '-':
+            del values[:]
+        else:
+            values.append(value)
+
+    parser = optparse.OptionParser(usage="%prog [-f NAME] [-o]")
+    parser.add_option("--function-name", "-f", metavar="NAME",
+                      help="Name of the generated function (default: _bzr)")
+    parser.add_option("--function-only", "-o", action="store_true",
+                      help="Generate only the shell function, don't enable it")
+    parser.add_option("--debug", action="store_true",
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-plugins", action="store_true",
+                      help="Don't load any bzr plugins")
+    parser.add_option("--plugin", metavar="NAME", type="string",
+                      dest="selected_plugins", default=[],
+                      action="callback", callback=plugin_callback,
+                      help="Enable completions for the selected plugin"
+                      + " (default: all plugins)")
+    (opts, args) = parser.parse_args()
+    if args:
+        parser.error("script does not take positional arguments")
+    kwargs = dict()
+    for name, value in opts.__dict__.iteritems():
+        if value is not None:
+            kwargs[name] = value
+
+    locale.setlocale(locale.LC_ALL, '')
+    if not kwargs.get('no_plugins', False):
+        plugin.load_plugins()
+    commands.install_bzr_command_hooks()
+    bash_completion_function(sys.stdout, **kwargs)

=== added directory 'bzrlib/plugins/bash_completion/tests'
=== added file 'bzrlib/plugins/bash_completion/tests/__init__.py'
--- bzrlib/plugins/bash_completion/tests/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/tests/__init__.py	2010-04-29 20:51:13 +0000
@@ -0,0 +1,23 @@
+# Copyright (C) 2010 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_bashcomp',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/bash_completion/tests/test_bashcomp.py'
--- bzrlib/plugins/bash_completion/tests/test_bashcomp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2010-05-04 19:30:44 +0000
@@ -0,0 +1,318 @@
+# Copyright (C) 2010 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import bzrlib
+from bzrlib import commands, tests
+from bzrlib.plugins.bash_completion.bashcomp import *
+
+import os
+import subprocess
+
+
+class _BashFeature(tests.Feature):
+    """Feature testing whether a bash executable is available."""
+
+    bash_paths = ['/bin/bash', '/usr/bin/bash']
+
+    def __init__(self):
+        super(_BashFeature, self).__init__()
+        self.bash_path = None
+
+    def available(self):
+        if self.bash_path is not None:
+            return self.bash_path is not False
+        for path in self.bash_paths:
+            if os.access(path, os.X_OK):
+                self.bash_path = path
+                return True
+        self.bash_path = False
+        return False
+
+    def feature_name(self):
+        return 'bash'
+
+BashFeature = _BashFeature()
+
+
+class BashCompletionMixin(object):
+    """Component for testing execution of a bash completion script."""
+
+    _test_needs_features = [BashFeature]
+
+    def complete(self, words, cword=-1):
+        """Perform a bash completion.
+
+        :param words: a list of words representing the current command.
+        :param cword: the current word to complete, defaults to the last one.
+        """
+        if self.script is None:
+            self.script = self.get_script()
+        proc = subprocess.Popen([BashFeature.bash_path, '--noprofile'],
+                                stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        if cword < 0:
+            cword = len(words) + cword
+        input = '%s\n' % self.script
+        input += ('COMP_WORDS=( %s )\n' %
+                  ' '.join(["'"+w.replace("'", "'\\''")+"'" for w in words]))
+        input += 'COMP_CWORD=%d\n' % cword
+        input += '%s\n' % getattr(self, 'script_name', '_bzr')
+        input += 'echo ${#COMPREPLY[*]}\n'
+        input += "IFS=$'\\n'\n"
+        input += 'echo "${COMPREPLY[*]}"\n'
+        (out, err) = proc.communicate(input)
+        if '' != err:
+            raise AssertionError('Unexpected error message:\n%s' % err)
+        self.assertEqual('', err, 'No messages to standard error')
+        #import sys
+        #print >>sys.stdout, '---\n%s\n---\n%s\n---\n' % (input, out)
+        lines = out.split('\n')
+        nlines = int(lines[0])
+        del lines[0]
+        self.assertEqual('', lines[-1], 'Newline at end')
+        del lines[-1]
+        if nlines == 0 and len(lines) == 1 and lines[0] == '':
+            del lines[0]
+        self.assertEqual(nlines, len(lines), 'No newlines in generated words')
+        self.completion_result = set(lines)
+        return self.completion_result
+
+    def assertCompletionEquals(self, *words):
+        self.assertEqual(set(words), self.completion_result)
+
+    def assertCompletionContains(self, *words):
+        missing = set(words) - self.completion_result
+        if missing:
+            raise AssertionError('Completion should contain %r but it has %r'
+                                 % (missing, self.completion_result))
+
+    def assertCompletionOmits(self, *words):
+        surplus = set(words) & self.completion_result
+        if surplus:
+            raise AssertionError('Completion should omit %r but it has %r'
+                                 % (surplus, res, self.completion_result))
+
+    def get_script(self):
+        commands.install_bzr_command_hooks()
+        dc = DataCollector()
+        data = dc.collect()
+        cg = BashCodeGen(data)
+        res = cg.function()
+        return res
+
+
+class TestBashCompletion(tests.TestCase, BashCompletionMixin):
+    """Test bash completions that don't execute bzr."""
+
+    def __init__(self, methodName='testMethod'):
+        super(TestBashCompletion, self).__init__(methodName)
+        self.script = None
+
+    def test_simple_scipt(self):
+        """Ensure that the test harness works as expected"""
+        self.script = """
+_bzr() {
+    COMPREPLY=()
+    # add all words in reverse order, with some markup around them
+    for ((i = ${#COMP_WORDS[@]}; i > 0; --i)); do
+        COMPREPLY+=( "-${COMP_WORDS[i-1]}+" )
+    done
+    # and append the current word
+    COMPREPLY+=( "+${COMP_WORDS[COMP_CWORD]}-" )
+}
+"""
+        self.complete(['foo', '"bar', "'baz"], cword=1)
+        self.assertCompletionEquals("-'baz+", '-"bar+', '-foo+', '+"bar-')
+
+    def test_cmd_ini(self):
+        self.complete(['bzr', 'ini'])
+        self.assertCompletionContains('init', 'init-repo', 'init-repository')
+        self.assertCompletionOmits('commit')
+
+    def test_init_opts(self):
+        self.complete(['bzr', 'init', '-'])
+        self.assertCompletionContains('-h', '--2a', '--format=2a')
+
+    def test_global_opts(self):
+        self.complete(['bzr', '-', 'init'], cword=1)
+        self.assertCompletionContains('--no-plugins', '--builtin')
+
+    def test_commit_dashm(self):
+        self.complete(['bzr', 'commit', '-m'])
+        self.assertCompletionEquals('-m')
+
+    def test_status_negated(self):
+        self.complete(['bzr', 'status', '--n'])
+        self.assertCompletionContains('--no-versioned', '--no-verbose')
+
+    def test_init_format_any(self):
+        self.complete(['bzr', 'init', '--format', '=', 'directory'], cword=3)
+        self.assertCompletionContains('1.9', '2a')
+
+    def test_init_format_2(self):
+        self.complete(['bzr', 'init', '--format', '=', '2', 'directory'],
+                      cword=4)
+        self.assertCompletionContains('2a')
+        self.assertCompletionOmits('1.9')
+
+
+class TestBashCompletionInvoking(tests.TestCaseWithTransport,
+                                 BashCompletionMixin):
+    """Test bash completions that might execute bzr.
+
+    Only the syntax ``$(bzr ...`` is supported so far. The bzr command
+    will be replaced by the bzr instance running this selftest.
+    """
+
+    def __init__(self, methodName='testMethod'):
+        super(TestBashCompletionInvoking, self).__init__(methodName)
+        self.script = None
+
+    def get_script(self):
+        s = super(TestBashCompletionInvoking, self).get_script()
+        return s.replace("$(bzr ", "$('%s' " % self.get_bzr_path())
+
+    def test_revspec_tag_all(self):
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        wt.branch.tags.set_tag('3tag', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':'])
+        self.assertCompletionEquals('tag1', 'tag2', '3tag')
+
+    def test_revspec_tag_prefix(self):
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        wt.branch.tags.set_tag('3tag', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':', 't'])
+        self.assertCompletionEquals('tag1', 'tag2')
+
+
+class TestBashCodeGen(tests.TestCase):
+
+    def test_command_names(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqual('bar baz foo', cg.command_names())
+
+    def test_debug_output(self):
+        data = CompletionData()
+        self.assertEqual('', BashCodeGen(data, debug=False).debug_output())
+        self.assertTrue(BashCodeGen(data, debug=True).debug_output())
+
+    def test_bzr_version(self):
+        data = CompletionData()
+        cg = BashCodeGen(data)
+        self.assertEqual('%s.' % bzrlib.version_string, cg.bzr_version())
+        data.plugins['foo'] = PluginData('foo', '1.0')
+        data.plugins['bar'] = PluginData('bar', '2.0')
+        cg = BashCodeGen(data)
+        self.assertEqual('''\
+%s and the following plugins:
+# bar 2.0
+# foo 1.0''' % bzrlib.version_string, cg.bzr_version())
+
+    def test_global_options(self):
+        data = CompletionData()
+        data.global_options.add('--foo')
+        data.global_options.add('--bar')
+        cg = BashCodeGen(data)
+        self.assertEqual('--bar --foo', cg.global_options())
+
+    def test_command_cases(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        bar.options.append(OptionData('--opt'))
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tbar|baz)
+\t\tcmdOpts='--opt'
+\t\t;;
+\tfoo)
+\t\tcmdOpts=''
+\t\t;;
+''', cg.command_cases())
+
+    def test_command_case(self):
+        cmd = CommandData('cmd')
+        cmd.plugin = PluginData('plugger', '1.0')
+        bar = OptionData('--bar')
+        bar.registry_keys = ['that', 'this']
+        bar.error_messages.append('Some error message')
+        cmd.options.append(bar)
+        cmd.options.append(OptionData('--foo'))
+        data = CompletionData()
+        data.commands.append(cmd)
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tcmd)
+\t\t# plugin "plugger 1.0"
+\t\t# Some error message
+\t\tcmdOpts='--bar=that --bar=this --foo'
+\t\tcase $curOpt in
+\t\t\t--bar) optEnums='that this' ;;
+\t\tesac
+\t\t;;
+''', cg.command_case(cmd))
+
+
+class TestDataCollector(tests.TestCase):
+
+    def setUp(self):
+        super(TestDataCollector, self).setUp()
+        commands.install_bzr_command_hooks()
+
+    def test_global_options(self):
+        dc = DataCollector()
+        dc.global_options()
+        self.assertSubset(['--no-plugins', '--builtin'],
+                           dc.data.global_options)
+
+    def test_commands(self):
+        dc = DataCollector()
+        dc.commands()
+        self.assertSubset(['init', 'init-repo', 'init-repository'],
+                           dc.data.all_command_aliases())
+
+    def test_commit_dashm(self):
+        dc = DataCollector()
+        cmd = dc.command('commit')
+        self.assertSubset(['-m'],
+                           [str(o) for o in cmd.options])
+
+    def test_status_negated(self):
+        dc = DataCollector()
+        cmd = dc.command('status')
+        self.assertSubset(['--no-versioned', '--no-verbose'],
+                           [str(o) for o in cmd.options])
+
+    def test_init_format(self):
+        dc = DataCollector()
+        cmd = dc.command('init')
+        for opt in cmd.options:
+            if opt.name == '--format':
+                self.assertSubset(['2a'], opt.registry_keys)
+                return
+        raise AssertionError('Option --format not found')

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2010-05-03 04:08:50 +0000
+++ bzrlib/remote.py	2010-05-13 16:17:54 +0000
@@ -33,7 +33,7 @@
     static_tuple,
     symbol_versioning,
 )
-from bzrlib.branch import BranchReferenceFormat
+from bzrlib.branch import BranchReferenceFormat, BranchWriteLockResult
 from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.errors import (
@@ -43,6 +43,7 @@
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.smart import client, vfs, repository as smart_repo
 from bzrlib.revision import ensure_null, NULL_REVISION
+from bzrlib.repository import RepositoryWriteLockResult
 from bzrlib.trace import mutter, note, warning
 
 
@@ -272,16 +273,19 @@
     def create_workingtree(self, revision_id=None, from_branch=None):
         raise errors.NotLocalUrl(self.transport.base)
 
-    def find_branch_format(self):
+    def find_branch_format(self, name=None):
         """Find the branch 'format' for this bzrdir.
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
-        b = self.open_branch()
+        b = self.open_branch(name=name)
         return b._format
 
-    def get_branch_reference(self):
+    def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
+        if name is not None:
+            # XXX JRV20100304: Support opening colocated branches
+            raise errors.NoColocatedBranchSupport(self)
         response = self._get_branch_reference()
         if response[0] == 'ref':
             return response[1]
@@ -318,9 +322,9 @@
             raise errors.UnexpectedSmartServerResponse(response)
         return response
 
-    def _get_tree_branch(self):
+    def _get_tree_branch(self, name=None):
         """See BzrDir._get_tree_branch()."""
-        return None, self.open_branch()
+        return None, self.open_branch(name=name)
 
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False):
@@ -997,6 +1001,10 @@
         pass
 
     def lock_read(self):
+        """Lock the repository for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         # wrong eventually - want a local lock cache context
         if not self._lock_mode:
             self._note_lock('r')
@@ -1009,6 +1017,7 @@
                 repo.lock_read()
         else:
             self._lock_count += 1
+        return lock.LogicalLockResult(self.unlock)
 
     def _remote_lock_write(self, token):
         path = self.bzrdir._path_for_remote_call(self._client)
@@ -1054,7 +1063,7 @@
             raise errors.ReadOnlyError(self)
         else:
             self._lock_count += 1
-        return self._lock_token or None
+        return RepositoryWriteLockResult(self.unlock, self._lock_token or None)
 
     def leave_lock_in_place(self):
         if not self._lock_token:
@@ -1306,16 +1315,16 @@
         return self._real_repository.make_working_trees()
 
     def refresh_data(self):
-        """Re-read any data needed to to synchronise with disk.
+        """Re-read any data needed to synchronise with disk.
 
         This method is intended to be called after another repository instance
         (such as one used by a smart server) has inserted data into the
-        repository. It may not be called during a write group, but may be
-        called at any other time.
+        repository. On all repositories this will work outside of write groups.
+        Some repository formats (pack and newer for bzrlib native formats)
+        support refresh_data inside write groups. If called inside a write
+        group on a repository that does not support refreshing in a write group
+        IsInWriteGroupError will be raised.
         """
-        if self.is_in_write_group():
-            raise errors.InternalBzrError(
-                "May not refresh_data while in a write group.")
         if self._real_repository is not None:
             self._real_repository.refresh_data()
 
@@ -2387,6 +2396,10 @@
             self._vfs_set_tags_bytes(bytes)
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.repository.lock_read()
         if not self._lock_mode:
             self._note_lock('r')
@@ -2396,13 +2409,14 @@
                 self._real_branch.lock_read()
         else:
             self._lock_count += 1
+        return lock.LogicalLockResult(self.unlock)
 
     def _remote_lock_write(self, token):
         if token is None:
             branch_token = repo_token = ''
         else:
             branch_token = token
-            repo_token = self.repository.lock_write()
+            repo_token = self.repository.lock_write().repository_token
             self.repository.unlock()
         err_context = {'token': token}
         response = self._call(
@@ -2445,7 +2459,7 @@
             self._lock_count += 1
             # Re-lock the repository too.
             self.repository.lock_write(self._repo_lock_token)
-        return self._lock_token or None
+        return BranchWriteLockResult(self.unlock, self._lock_token or None)
 
     def _unlock(self, branch_token, repo_token):
         err_context = {'token': str((branch_token, repo_token))}
@@ -2774,17 +2788,51 @@
         medium = self._branch._client._medium
         if medium._is_remote_before((1, 14)):
             return self._vfs_set_option(value, name, section)
+        if isinstance(value, dict):
+            if medium._is_remote_before((2, 2)):
+                return self._vfs_set_option(value, name, section)
+            return self._set_config_option_dict(value, name, section)
+        else:
+            return self._set_config_option(value, name, section)
+
+    def _set_config_option(self, value, name, section):
         try:
             path = self._branch._remote_path()
             response = self._branch._client.call('Branch.set_config_option',
                 path, self._branch._lock_token, self._branch._repo_lock_token,
                 value.encode('utf8'), name, section or '')
         except errors.UnknownSmartMethod:
+            medium = self._branch._client._medium
             medium._remember_remote_is_before((1, 14))
             return self._vfs_set_option(value, name, section)
         if response != ():
             raise errors.UnexpectedSmartServerResponse(response)
 
+    def _serialize_option_dict(self, option_dict):
+        utf8_dict = {}
+        for key, value in option_dict.items():
+            if isinstance(key, unicode):
+                key = key.encode('utf8')
+            if isinstance(value, unicode):
+                value = value.encode('utf8')
+            utf8_dict[key] = value
+        return bencode.bencode(utf8_dict)
+
+    def _set_config_option_dict(self, value, name, section):
+        try:
+            path = self._branch._remote_path()
+            serialised_dict = self._serialize_option_dict(value)
+            response = self._branch._client.call(
+                'Branch.set_config_option_dict',
+                path, self._branch._lock_token, self._branch._repo_lock_token,
+                serialised_dict, name, section or '')
+        except errors.UnknownSmartMethod:
+            medium = self._branch._client._medium
+            medium._remember_remote_is_before((2, 2))
+            return self._vfs_set_option(value, name, section)
+        if response != ():
+            raise errors.UnexpectedSmartServerResponse(response)
+
     def _real_object(self):
         self._branch._ensure_real()
         return self._branch._real_branch

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2010-02-10 02:17:15 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2010-05-13 18:52:58 +0000
@@ -262,7 +262,6 @@
         remaining_keys = set(keys)
         counter = [0]
         if self._gather_text_refs:
-            bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
             self._text_refs = set()
         def _get_referenced_stream(root_keys, parse_leaf_nodes=False):
             cur_keys = root_keys
@@ -289,8 +288,7 @@
                     # Store is None, because we know we have a LeafNode, and we
                     # just want its entries
                     for file_id, bytes in node.iteritems(None):
-                        name_utf8, file_id, revision_id = bytes_to_info(bytes)
-                        self._text_refs.add((file_id, revision_id))
+                        self._text_refs.add(chk_map._bytes_to_text_key(bytes))
                 def next_stream():
                     stream = source_vf.get_record_stream(cur_keys,
                                                          'as-requested', True)
@@ -647,10 +645,10 @@
         chk_diff = chk_map.iter_interesting_nodes(
             chk_bytes_no_fallbacks, root_key_info.interesting_root_keys,
             root_key_info.uninteresting_root_keys)
-        bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
         text_keys = set()
         try:
-            for record in _filter_text_keys(chk_diff, text_keys, bytes_to_info):
+            for record in _filter_text_keys(chk_diff, text_keys,
+                                            chk_map._bytes_to_text_key):
                 pass
         except errors.NoSuchRevision, e:
             # XXX: It would be nice if we could give a more precise error here.
@@ -864,6 +862,8 @@
         if basis_inv is None:
             if basis_revision_id == _mod_revision.NULL_REVISION:
                 new_inv = self._create_inv_from_null(delta, new_revision_id)
+                if new_inv.root_id is None:
+                    raise errors.RootMissing()
                 inv_lines = new_inv.to_lines()
                 return self._inventory_add_lines(new_revision_id, parents,
                     inv_lines, check_content=False), new_inv
@@ -1087,13 +1087,12 @@
                 uninteresting_root_keys.add(inv.id_to_entry.key())
                 uninteresting_pid_root_keys.add(
                     inv.parent_id_basename_to_file_id.key())
-        bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
         chk_bytes = self.from_repository.chk_bytes
         def _filter_id_to_entry():
             interesting_nodes = chk_map.iter_interesting_nodes(chk_bytes,
                         self._chk_id_roots, uninteresting_root_keys)
             for record in _filter_text_keys(interesting_nodes, self._text_keys,
-                    bytes_to_info):
+                    chk_map._bytes_to_text_key):
                 if record is not None:
                     yield record
             # Consumed
@@ -1187,19 +1186,13 @@
     return result
 
 
-def _filter_text_keys(interesting_nodes_iterable, text_keys, bytes_to_info):
+def _filter_text_keys(interesting_nodes_iterable, text_keys, bytes_to_text_key):
     """Iterate the result of iter_interesting_nodes, yielding the records
     and adding to text_keys.
     """
+    text_keys_update = text_keys.update
     for record, items in interesting_nodes_iterable:
-        for name, bytes in items:
-            # Note: we don't care about name_utf8, because groupcompress repos
-            # are always rich-root, so there are no synthesised root records to
-            # ignore.
-            _, file_id, revision_id = bytes_to_info(bytes)
-            file_id = intern(file_id)
-            revision_id = intern(revision_id)
-            text_keys.add(StaticTuple(file_id, revision_id).intern())
+        text_keys_update([bytes_to_text_key(b) for n,b in items])
         yield record
 
 

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2010-04-26 13:51:08 +0000
+++ bzrlib/repofmt/knitrepo.py	2010-05-04 02:41:17 +0000
@@ -38,6 +38,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     CommitBuilder,
+    IsInWriteGroupError,
     MetaDirRepository,
     MetaDirRepositoryFormat,
     RepositoryFormat,
@@ -210,6 +211,8 @@
     def _refresh_data(self):
         if not self.is_locked():
             return
+        if self.is_in_write_group():
+            raise IsInWriteGroupError(self)
         # Create a new transaction to force all knits to see the scope change.
         # This is safe because we're outside a write group.
         self.control_files._finish_transaction()

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2010-04-22 17:08:27 +0000
+++ bzrlib/repofmt/pack_repo.py	2010-05-11 08:36:16 +0000
@@ -64,11 +64,13 @@
     GraphIndex,
     InMemoryGraphIndex,
     )
+from bzrlib.lock import LogicalLockResult
 from bzrlib.repofmt.knitrepo import KnitRepository
 from bzrlib.repository import (
     CommitBuilder,
     MetaDirRepositoryFormat,
     RepositoryFormat,
+    RepositoryWriteLockResult,
     RootCommitBuilder,
     StreamSource,
     )
@@ -2340,6 +2342,10 @@
         return self._write_lock_count
 
     def lock_write(self, token=None):
+        """Lock the repository for writes.
+
+        :return: A bzrlib.repository.RepositoryWriteLockResult.
+        """
         locked = self.is_locked()
         if not self._write_lock_count and locked:
             raise errors.ReadOnlyError(self)
@@ -2354,8 +2360,13 @@
                 # Writes don't affect fallback repos
                 repo.lock_read()
             self._refresh_data()
+        return RepositoryWriteLockResult(self.unlock, None)
 
     def lock_read(self):
+        """Lock the repository for reads.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         locked = self.is_locked()
         if self._write_lock_count:
             self._write_lock_count += 1
@@ -2368,6 +2379,7 @@
             for repo in self._fallback_repositories:
                 repo.lock_read()
             self._refresh_data()
+        return LogicalLockResult(self.unlock)
 
     def leave_lock_in_place(self):
         # not supported - raise an error

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-04-28 07:03:38 +0000
+++ bzrlib/repository.py	2010-05-13 18:52:58 +0000
@@ -26,7 +26,6 @@
     chk_map,
     config,
     debug,
-    errors,
     fetch as _mod_fetch,
     fifo_cache,
     generate_ids,
@@ -53,6 +52,10 @@
 from bzrlib.testament import Testament
 """)
 
+from bzrlib import (
+    errors,
+    registry,
+    )
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.inter import InterObject
 from bzrlib.inventory import (
@@ -61,8 +64,7 @@
     ROOT_ID,
     entry_factory,
     )
-from bzrlib.lock import _RelockDebugMixin
-from bzrlib import registry
+from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib.trace import (
     log_exception_quietly, note, mutter, mutter_callsite, warning)
 
@@ -71,6 +73,14 @@
 _deprecation_warning_done = False
 
 
+class IsInWriteGroupError(errors.InternalBzrError):
+
+    _fmt = "May not refresh_data of repo %(repo)s while in a write group."
+
+    def __init__(self, repo):
+        errors.InternalBzrError.__init__(self, repo=repo)
+
+
 class CommitBuilder(object):
     """Provides an interface to build up a commit.
 
@@ -278,8 +288,8 @@
 
         :param tree: The tree which is being committed.
         """
-        # NB: if there are no parents then this method is not called, so no
-        # need to guard on parents having length.
+        if len(self.parents) == 0:
+            raise errors.RootMissing()
         entry = entry_factory['directory'](tree.path2id(''), '',
             None)
         entry.revision = self._new_revision_id
@@ -860,6 +870,23 @@
         # versioned roots do not change unless the tree found a change.
 
 
+class RepositoryWriteLockResult(LogicalLockResult):
+    """The result of write locking a repository.
+
+    :ivar repository_token: The token obtained from the underlying lock, or
+        None.
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock, repository_token):
+        LogicalLockResult.__init__(self, unlock)
+        self.repository_token = repository_token
+
+    def __repr__(self):
+        return "RepositoryWriteLockResult(%s, %s)" % (self.repository_token,
+            self.unlock)
+
+
 ######################################################################
 # Repositories
 
@@ -1018,7 +1045,7 @@
                 " id and insertion revid (%r, %r)"
                 % (inv.revision_id, revision_id))
         if inv.root is None:
-            raise AssertionError()
+            raise errors.RootMissing()
         return self._add_inventory_checked(revision_id, inv, parents)
 
     def _add_inventory_checked(self, revision_id, inv, parents):
@@ -1376,6 +1403,12 @@
         data during reads, and allows a 'write_group' to be obtained. Write
         groups must be used for actual data insertion.
 
+        A token should be passed in if you know that you have locked the object
+        some other way, and need to synchronise this object's state with that
+        fact.
+
+        XXX: this docstring is duplicated in many places, e.g. lockable_files.py
+
         :param token: if this is already locked, then lock_write will fail
             unless the token matches the existing lock.
         :returns: a token if this instance supports tokens, otherwise None.
@@ -1384,15 +1417,10 @@
         :raises MismatchedToken: if the specified token doesn't match the token
             of the existing lock.
         :seealso: start_write_group.
-
-        A token should be passed in if you know that you have locked the object
-        some other way, and need to synchronise this object's state with that
-        fact.
-
-        XXX: this docstring is duplicated in many places, e.g. lockable_files.py
+        :return: A RepositoryWriteLockResult.
         """
         locked = self.is_locked()
-        result = self.control_files.lock_write(token=token)
+        token = self.control_files.lock_write(token=token)
         if not locked:
             self._warn_if_deprecated()
             self._note_lock('w')
@@ -1400,9 +1428,14 @@
                 # Writes don't affect fallback repos
                 repo.lock_read()
             self._refresh_data()
-        return result
+        return RepositoryWriteLockResult(self.unlock, token)
 
     def lock_read(self):
+        """Lock the repository for read operations.
+
+        :return: An object with an unlock method which will release the lock
+            obtained.
+        """
         locked = self.is_locked()
         self.control_files.lock_read()
         if not locked:
@@ -1411,6 +1444,7 @@
             for repo in self._fallback_repositories:
                 repo.lock_read()
             self._refresh_data()
+        return LogicalLockResult(self.unlock)
 
     def get_physical_lock_status(self):
         return self.control_files.get_physical_lock_status()
@@ -1634,16 +1668,16 @@
         return missing_keys
 
     def refresh_data(self):
-        """Re-read any data needed to to synchronise with disk.
+        """Re-read any data needed to synchronise with disk.
 
         This method is intended to be called after another repository instance
         (such as one used by a smart server) has inserted data into the
-        repository. It may not be called during a write group, but may be
-        called at any other time.
+        repository. On all repositories this will work outside of write groups.
+        Some repository formats (pack and newer for bzrlib native formats)
+        support refresh_data inside write groups. If called inside a write
+        group on a repository that does not support refreshing in a write group
+        IsInWriteGroupError will be raised.
         """
-        if self.is_in_write_group():
-            raise errors.InternalBzrError(
-                "May not refresh_data while in a write group.")
         self._refresh_data()
 
     def resume_write_group(self, tokens):

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/revisiontree.py	2010-05-06 23:41:35 +0000
@@ -174,8 +174,12 @@
         return set(self._repository.get_ancestry(self._revision_id,
                                                  topo_sorted=False))
 
+    def is_locked(self):
+        return self._repository.is_locked()
+
     def lock_read(self):
         self._repository.lock_read()
+        return self
 
     def __repr__(self):
         return '<%s instance at %x, rev_id=%r>' % (

=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2009-10-21 06:05:49 +0000
+++ bzrlib/smart/branch.py	2010-05-13 16:17:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
 """Server-side branch related request implmentations."""
 
 
-from bzrlib import errors
+from bzrlib import (
+    bencode,
+    errors,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
@@ -194,6 +197,23 @@
         return SuccessfulSmartServerResponse(())
 
 
+class SmartServerBranchRequestSetConfigOptionDict(SmartServerLockedBranchRequest):
+    """Set an option in the branch configuration.
+    
+    New in 2.2.
+    """
+
+    def do_with_locked_branch(self, branch, value_dict, name, section):
+        utf8_dict = bencode.bdecode(value_dict)
+        value_dict = {}
+        for key, value in utf8_dict.items():
+            value_dict[key.decode('utf8')] = value.decode('utf8')
+        if not section:
+            section = None
+        branch._get_config().set_option(value_dict, name, section)
+        return SuccessfulSmartServerResponse(())
+
+
 class SmartServerBranchRequestSetLastRevision(SmartServerSetTipRequest):
 
     def do_tip_change_with_locked_branch(self, branch, new_last_revision_id):
@@ -292,9 +312,11 @@
         if repo_token == '':
             repo_token = None
         try:
-            repo_token = branch.repository.lock_write(token=repo_token)
+            repo_token = branch.repository.lock_write(
+                token=repo_token).repository_token
             try:
-                branch_token = branch.lock_write(token=branch_token)
+                branch_token = branch.lock_write(
+                    token=branch_token).branch_token
             finally:
                 # this leaves the repository with 1 lock
                 branch.repository.unlock()

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2010-04-23 00:44:15 +0000
+++ bzrlib/smart/bzrdir.py	2010-05-06 23:41:35 +0000
@@ -429,7 +429,7 @@
             # It is returned locked, but we need to do the lock to get the lock
             # token.
             repo.unlock()
-            repo_lock_token = repo.lock_write() or ''
+            repo_lock_token = repo.lock_write().repository_token or ''
             if repo_lock_token:
                 repo.leave_lock_in_place()
             repo.unlock()

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-03-25 07:34:15 +0000
+++ bzrlib/smart/repository.py	2010-05-06 23:41:35 +0000
@@ -392,7 +392,7 @@
         if token == '':
             token = None
         try:
-            token = repository.lock_write(token=token)
+            token = repository.lock_write(token=token).repository_token
         except errors.LockContention, e:
             return FailedSmartServerResponse(('LockContention',))
         except errors.UnlockableTransport:

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2010-02-17 17:11:16 +0000
+++ bzrlib/smart/request.py	2010-05-13 16:17:54 +0000
@@ -510,6 +510,8 @@
     'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
 request_handlers.register_lazy( 'Branch.set_config_option',
     'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOption')
+request_handlers.register_lazy( 'Branch.set_config_option_dict',
+    'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOptionDict')
 request_handlers.register_lazy( 'Branch.set_last_revision',
     'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
 request_handlers.register_lazy(

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2010-05-03 04:08:50 +0000
+++ bzrlib/switch.py	2010-05-07 11:50:28 +0000
@@ -78,7 +78,7 @@
     branch_format = control.find_branch_format()
     if branch_format.get_reference(control) is not None:
         # Lightweight checkout: update the branch reference
-        branch_format.set_reference(control, to_branch)
+        branch_format.set_reference(control, None, to_branch)
     else:
         b = control.open_branch()
         bound_branch = b.get_bound_location()

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2010-05-06 07:48:22 +0000
+++ bzrlib/tests/__init__.py	2010-05-14 08:10:35 +0000
@@ -112,13 +112,13 @@
 from bzrlib.tests import (
     test_server,
     TestUtil,
+    treeshape,
     )
 from bzrlib.tests.http_server import HttpServer
 from bzrlib.tests.TestUtil import (
                           TestSuite,
                           TestLoader,
                           )
-from bzrlib.tests.treeshape import build_tree_contents
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
 import bzrlib.version_info_formats.format_custom
@@ -1524,7 +1524,7 @@
             'EDITOR': None,
             'BZR_EMAIL': None,
             'BZREMAIL': None, # may still be present in the environment
-            'EMAIL': None,
+            'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
             'BZR_PROGRESS_BAR': None,
             'BZR_LOG': None,
             'BZR_PLUGIN_PATH': None,
@@ -2578,8 +2578,7 @@
                 content = "contents of %s%s" % (name.encode('utf-8'), end)
                 transport.put_bytes_non_atomic(urlutils.escape(name), content)
 
-    def build_tree_contents(self, shape):
-        build_tree_contents(shape)
+    build_tree_contents = staticmethod(treeshape.build_tree_contents)
 
     def assertInWorkingTree(self, path, root_path='.', tree=None):
         """Assert whether path or paths are in the WorkingTree"""
@@ -4102,8 +4101,11 @@
         if test_id != None:
             ui.ui_factory.clear_term()
             sys.stderr.write('\nWhile running: %s\n' % (test_id,))
+        # Ugly, but the last thing we want here is fail, so bear with it.
+        printable_e = str(e).decode(osutils.get_user_encoding(), 'replace'
+                                    ).encode('ascii', 'replace')
         sys.stderr.write('Unable to remove testing dir %s\n%s'
-                         % (os.path.basename(dirname), e))
+                         % (os.path.basename(dirname), printable_e))
 
 
 class Feature(object):

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/blackbox/__init__.py	2010-05-03 09:19:15 +0000
@@ -52,6 +52,7 @@
                      'bzrlib.tests.blackbox.test_commit',
                      'bzrlib.tests.blackbox.test_conflicts',
                      'bzrlib.tests.blackbox.test_debug',
+                     'bzrlib.tests.blackbox.test_deleted',
                      'bzrlib.tests.blackbox.test_diff',
                      'bzrlib.tests.blackbox.test_dump_btree',
                      'bzrlib.tests.blackbox.test_dpush',
@@ -71,6 +72,7 @@
                      'bzrlib.tests.blackbox.test_locale',
                      'bzrlib.tests.blackbox.test_log',
                      'bzrlib.tests.blackbox.test_logformats',
+                     'bzrlib.tests.blackbox.test_lookup_revision',
                      'bzrlib.tests.blackbox.test_ls',
                      'bzrlib.tests.blackbox.test_lsprof',
                      'bzrlib.tests.blackbox.test_merge',

=== modified file 'bzrlib/tests/blackbox/test_added.py'
--- bzrlib/tests/blackbox/test_added.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_added.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -67,3 +67,11 @@
         # after commit, now no longer listed
         tree.commit(message='add "%s"' % (name))
         check_added('')
+
+    def test_added_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        out, err = self.run_bzr(['added', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_alias.py'
--- bzrlib/tests/blackbox/test_alias.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_alias.py	2010-04-29 03:23:18 +0000
@@ -16,8 +16,11 @@
 #
 
 """Tests of the 'bzr alias' command."""
+import os
+import codecs
 
 from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
 class TestAlias(ExternalBase):
@@ -41,6 +44,26 @@
         out, err = self.run_bzr('alias commit')
         self.assertEquals('bzr alias commit="commit --strict"\n', out)
 
+    def test_unicode_alias(self):
+        """Unicode aliases should work (Bug #529930)"""
+        config_enc = 'utf-8'
+        file_name = u'foo\xb6'
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree([file_name])
+        tree.add(file_name)
+        tree.commit('added')
+
+        ensure_config_dir_exists()
+        CONFIG=(u'[ALIASES]\n'
+                u'ust=st foo\xb6\n')
+
+        codecs.open(config_filename(),'w', config_enc).write(CONFIG)
+
+        out, err = self.run_bzr('ust')
+        self.assertEquals(err, '')
+        self.assertEquals(out, '')
+
     def test_alias_listing_alphabetical(self):
         self.run_bzr('alias commit="commit --strict"')
         self.run_bzr('alias ll="log --short"')

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2010-03-24 14:15:01 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2010-05-02 20:10:25 +0000
@@ -278,3 +278,12 @@
         os.chdir('branch')
         out, err = self.run_bzr('annotate empty')
         self.assertEqual('', out)
+
+    def test_annotate_directory(self):
+        """Test --directory option"""
+        wt = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/hello.txt', 'my helicopter\n')])
+        wt.add(['hello.txt'])
+        wt.commit('commit', committer='test@user')
+        out, err = self.run_bzr('annotate -d a hello.txt')
+        self.assertEqualDiff('1   test@us | my helicopter\n', out)

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2010-02-04 16:06:36 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2010-05-02 20:10:25 +0000
@@ -422,6 +422,21 @@
         self.check_revno(4)
         self.check_revno(4, '../base')
 
+    def test_bind_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/a', 'base/b'])
+        tree.add('a', 'b')
+        tree.commit(message='init')
+        branch = tree.branch
+        tree.bzrdir.sprout('child')
+        self.run_bzr('bind --directory=child base')
+        d = BzrDir.open('child')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind -d child')
+        self.assertEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind --directory child', retcode=3)
+
 
 class TestBind(script.TestCaseWithTransportAndScript):
 

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2010-05-02 18:16:37 +0000
@@ -199,3 +199,22 @@
         self.vfs_transport_factory = memory.MemoryServer
         self.run_bzr_error(['^bzr: ERROR: Not a branch'],
                            ['cat', self.get_url()])
+
+    def test_cat_directory(self):
+        wt = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        wt.add('README')
+        wt.commit('Making sure there is a basis_tree available')
+
+        out, err = self.run_bzr_subprocess(['cat', '--directory=a', 'README'])
+        self.assertEqual('contents of a/README\n', out)
+
+    def test_cat_remote_directory(self):
+        wt = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        wt.add('README')
+        wt.commit('Making sure there is a basis_tree available')
+
+        url = self.get_readonly_url() + '/a'
+        out, err = self.run_bzr_subprocess(['cat', '-d', url, 'README'])
+        self.assertEqual('contents of a/README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_cat_revision.py'
--- bzrlib/tests/blackbox/test_cat_revision.py	2010-02-15 20:23:34 +0000
+++ bzrlib/tests/blackbox/test_cat_revision.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007 Canonical Ltd
+# Copyright (C) 2004-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -64,4 +64,10 @@
         err = self.run_bzr('cat-revision abcd', retcode=3)[1]
         self.assertContainsRe(err, 'The repository .* contains no revision abcd.')
 
-
+    def test_cat_revision_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        tree.commit('This revision', rev_id='abcd')
+        output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
+        self.assertContainsRe(output, 'This revision')
+        self.assertEqual('', errors)

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2010-04-30 09:52:08 +0000
@@ -64,7 +64,8 @@
                          result.open_branch().bzrdir.root_transport.base)
 
     def test_checkout_dash_r(self):
-        self.run_bzr('checkout -r -2 branch checkout')
+        out, err = self.run_bzr(['checkout', '-r', '-2', 'branch', 'checkout'])
+        self.assertContainsRe(out, 'Copying history to "checkout".')
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
@@ -72,7 +73,9 @@
         self.failIfExists('checkout/added_in_2')
 
     def test_checkout_light_dash_r(self):
-        self.run_bzr('checkout --lightweight -r -2 branch checkout')
+        out, err = self.run_bzr(['checkout','--lightweight', '-r', '-2',
+            'branch', 'checkout'])
+        self.assertNotContainsRe(out, 'Copying history')
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')

=== modified file 'bzrlib/tests/blackbox/test_clean_tree.py'
--- bzrlib/tests/blackbox/test_clean_tree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_clean_tree.py	2010-05-05 08:57:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 import os
 
+from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -62,3 +63,28 @@
         self.failIfExists('name')
         self.failIfExists('name~')
         self.failIfExists('name.pyc')
+
+    def test_clean_tree_nested_bzrdir(self):
+        # clean-tree should not blindly delete nested bzrdirs (branches)
+        # bug https://bugs.launchpad.net/bzr/+bug/572098
+        # so it will play well with scmproj/bzr-externals plugins.
+        wt1 = self.make_branch_and_tree('.')
+        wt2 = self.make_branch_and_tree('foo')
+        wt3 = self.make_branch_and_tree('bar')
+        ignores.tree_ignores_add_patterns(wt1, ['./foo'])
+        self.run_bzr(['clean-tree', '--unknown', '--force'])
+        self.failUnlessExists('foo')
+        self.failUnlessExists('bar')
+        self.run_bzr(['clean-tree', '--ignored', '--force'])
+        self.failUnlessExists('foo')
+        self.failUnlessExists('bar')
+
+    def test_clean_tree_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/added', 'a/unknown', 'a/ignored'])
+        tree.add('added')
+        self.run_bzr('clean-tree -d a --unknown --ignored --force')
+        self.failIfExists('a/unknown')
+        self.failIfExists('a/ignored')
+        self.failUnlessExists('a/added')

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2010-04-07 22:22:27 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2010-05-05 18:51:16 +0000
@@ -18,6 +18,7 @@
 """Tests for the commit CLI of bzr."""
 
 import os
+import re
 import sys
 
 from bzrlib import (
@@ -107,6 +108,40 @@
                               'modified hello\.txt\n'
                               'Committed revision 2\.\n$')
 
+    def test_unicode_commit_message_is_filename(self):
+        """Unicode commit message same as a filename (Bug #563646).
+        """
+        file_name = u'\N{euro sign}'
+        self.run_bzr(['init'])
+        open(file_name, 'w').write('hello world')
+        self.run_bzr(['add'])
+        out, err = self.run_bzr(['commit', '-m', file_name])
+        reflags = re.MULTILINE|re.DOTALL|re.UNICODE
+        te = osutils.get_terminal_encoding()
+        self.assertContainsRe(err.decode(te),
+            u'The commit message is a file name:',
+            flags=reflags)
+
+        # Run same test with a filename that causes encode
+        # error for the terminal encoding. We do this
+        # by forcing terminal encoding of ascii for
+        # osutils.get_terminal_encoding which is used
+        # by ui.text.show_warning
+        default_get_terminal_enc = osutils.get_terminal_encoding
+        try:
+            osutils.get_terminal_encoding = lambda: 'ascii'
+            file_name = u'foo\u1234'
+            open(file_name, 'w').write('hello world')
+            self.run_bzr(['add'])
+            out, err = self.run_bzr(['commit', '-m', file_name])
+            reflags = re.MULTILINE|re.DOTALL|re.UNICODE
+            te = osutils.get_terminal_encoding()
+            self.assertContainsRe(err.decode(te, 'replace'),
+                u'The commit message is a file name:',
+                flags=reflags)
+        finally:
+            osutils.get_terminal_encoding = default_get_terminal_enc
+
     def test_warn_about_forgotten_commit_message(self):
         """Test that the lack of -m parameter is caught"""
         wt = self.make_branch_and_tree('.')
@@ -710,3 +745,15 @@
         out, err = self.run_bzr_error(["empty commit message"],
             "commit tree/hello.txt", stdin="n\n")
         self.assertEqual(expected, tree.last_revision())
+
+    def test_commit_without_username(self):
+        """Ensure commit error if username is not set.
+        """
+        self.run_bzr(['init', 'foo'])
+        os.chdir('foo')
+        open('foo.txt', 'w').write('hello')
+        self.run_bzr(['add'])
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['commit', '-m', 'initial'], 3)
+        self.assertContainsRe(err, 'Unable to determine your name')

=== added file 'bzrlib/tests/blackbox/test_deleted.py'
--- bzrlib/tests/blackbox/test_deleted.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_deleted.py	2010-05-02 20:10:25 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2010 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for 'bzr deleted', which shows newly deleted files."""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestDeleted(ExternalBase):
+
+    def test_deleted_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        tree.commit('r1')
+        tree.remove('README')
+        out, err = self.run_bzr(['deleted', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2010-04-19 04:10:53 +0000
+++ bzrlib/tests/blackbox/test_export.py	2010-05-02 20:10:25 +0000
@@ -313,3 +313,9 @@
         har_st = os.stat('t/har')
         self.assertEquals(315532800, har_st.st_mtime)
 
+    def test_export_directory(self):
+        """Test --directory option"""
+        self.example_branch()
+        self.run_bzr(['export', '--directory=branch', 'latest'])
+        self.assertEqual(['goodbye', 'hello'], sorted(os.listdir('latest')))
+        self.check_file_contents('latest/goodbye', 'baz')

=== modified file 'bzrlib/tests/blackbox/test_ignore.py'
--- bzrlib/tests/blackbox/test_ignore.py	2010-04-22 12:45:03 +0000
+++ bzrlib/tests/blackbox/test_ignore.py	2010-05-03 09:19:15 +0000
@@ -156,3 +156,9 @@
                          " and match your ignore pattern:\nb\n"\
                          "These files will continue to be version controlled"\
                          " unless you 'bzr remove' them.\n")
+
+    def test_ignore_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.run_bzr(['ignore', '--directory=a', 'README'])
+        self.check_file_contents('a/.bzrignore', 'README\n')

=== modified file 'bzrlib/tests/blackbox/test_ignored.py'
--- bzrlib/tests/blackbox/test_ignored.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_ignored.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,3 +38,11 @@
         out, err = self.run_bzr('ignored')
         self.assertEqual('', out)
         self.assertEqual('', err)
+
+    def test_ignored_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/README', 'contents'),
+                                  ('a/.bzrignore', 'README')])
+        out, err = self.run_bzr(['ignored', '--directory=a'])
+        self.assertStartsWith(out, 'README')

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2010-04-23 07:15:23 +0000
+++ bzrlib/tests/blackbox/test_init.py	2010-05-05 14:02:53 +0000
@@ -202,3 +202,17 @@
         self.assertEqual(True, branch._get_append_revisions_only())
         self.run_bzr_error(['cannot be set to append-revisions-only'],
                            'init --append-revisions-only --knit knit')
+
+    def test_init_without_username(self):
+        """Ensure init works if username is not set.
+        """
+        # bzr makes user specified whoami mandatory for operations
+        # like commit as whoami is recorded. init however is not so final
+        # and uses whoami only in a lock file. Without whoami the login name
+        # is used. This test is to ensure that init passes even when whoami
+        # is not available.
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['init', 'foo'])
+        self.assertEqual(err, '')
+        self.assertTrue(os.path.exists('foo'))

=== added file 'bzrlib/tests/blackbox/test_lookup_revision.py'
--- bzrlib/tests/blackbox/test_lookup_revision.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_lookup_revision.py	2010-05-02 20:10:25 +0000
@@ -0,0 +1,33 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for bzr lookup-revision.
+"""
+
+import os
+
+from bzrlib import tests
+
+class TestLookupRevision(tests.TestCaseWithTransport):
+
+    def test_lookup_revison_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        tree.commit('This revision', rev_id='abcd')
+        out, err = self.run_bzr(['lookup-revision', '-d', 'a', '1'])
+        self.assertEqual('abcd\n', out)
+        self.assertEqual('', err)

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2010-05-02 20:10:25 +0000
@@ -18,7 +18,7 @@
 
 import os
 
-from bzrlib import ignores
+from bzrlib import ignores, osutils
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -235,3 +235,12 @@
                        '%s/a\n'
                        % (self.test_dir, self.test_dir),
                        self.test_dir, recursive=False)
+
+    def test_ls_directory(self):
+        """Test --directory option"""
+        self.wt = self.make_branch_and_tree('dir')
+        self.build_tree(['dir/sub/', 'dir/sub/file'])
+        self.wt.add(['sub', 'sub/file'])
+        self.wt.commit('commit')
+        self.ls_equals('sub/\nsub/file\n', '--directory=dir')
+        self.ls_equals('sub/file\n', '-d dir sub')

=== modified file 'bzrlib/tests/blackbox/test_modified.py'
--- bzrlib/tests/blackbox/test_modified.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_modified.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -70,3 +70,12 @@
         tree.commit(message='modified %s' %(name))
         check_modified('')
 
+    def test_modified_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        tree.commit('r1')
+        self.build_tree_contents([('a/README', 'changed\n')])
+        out, err = self.run_bzr(['modified', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_nick.py'
--- bzrlib/tests/blackbox/test_nick.py	2010-03-01 16:17:57 +0000
+++ bzrlib/tests/blackbox/test_nick.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006  Canonical Ltd
+# Copyright (C) 2005-2010  Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,3 +78,12 @@
                          child.branch.get_config().has_explicit_nickname())
         osutils.rmtree('../base')
         self.assertEqual('child', self.run_bzr('nick')[0][:-1])
+
+    def test_nick_directory(self):
+        """Test --directory option"""
+        self.make_branch_and_tree('me.dev')
+        nick = self.run_bzr(['nick', '--directory=me.dev'])[0]
+        self.assertEqual('me.dev\n', nick)
+        self.run_bzr(['nick', '-d', 'me.dev', 'moo'])
+        nick = self.run_bzr(['nick', '--directory', 'me.dev'])[0]
+        self.assertEqual('moo\n', nick)

=== modified file 'bzrlib/tests/blackbox/test_re_sign.py'
--- bzrlib/tests/blackbox/test_re_sign.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/blackbox/test_re_sign.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -88,3 +88,16 @@
         self.assertEqualSignature(repo, 'A')
         self.assertEqualSignature(repo, 'B')
         self.assertEqualSignature(repo, 'C')
+
+    def test_resign_directory(self):
+        """Test --directory option"""
+        wt = BzrDir.create_standalone_workingtree('a')
+        wt.commit("base A", allow_pointless=True, rev_id='A')
+        wt.commit("base B", allow_pointless=True, rev_id='B')
+        wt.commit("base C", allow_pointless=True, rev_id='C')
+        repo = wt.branch.repository
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign --directory=a -r revid:A')
+        self.assertEqualSignature(repo, 'A')
+        self.run_bzr('re-sign -d a B')
+        self.assertEqualSignature(repo, 'B')

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2010-03-25 18:02:45 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2010-05-05 14:02:53 +0000
@@ -18,6 +18,7 @@
 
 import os
 
+from bzrlib import osutils
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
 import bzrlib.errors as errors
 from bzrlib.tests import TestCaseInTempDir
@@ -145,3 +146,17 @@
         self.assertLength(0, calls)
         self.run_bzr("init-repository a")
         self.assertLength(1, calls)
+
+    def test_init_repo_without_username(self):
+        """Ensure init-repo works if username is not set.
+        """
+        # bzr makes user specified whoami mandatory for operations
+        # like commit as whoami is recorded. init-repo however is not so final
+        # and uses whoami only in a lock file. Without whoami the login name
+        # is used. This test is to ensure that init-repo passes even when whoami
+        # is not available.
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['init-repo', 'foo'])
+        self.assertEqual(err, '')
+        self.assertTrue(os.path.exists('foo'))

=== modified file 'bzrlib/tests/blackbox/test_unknowns.py'
--- bzrlib/tests/blackbox/test_unknowns.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_unknowns.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -45,3 +45,10 @@
         # after all added, none shown
         tree.add(['b', 'c'])
         self.assertEquals(self.run_bzr('unknowns')[0], '')
+
+    def test_unknowns_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        out, err = self.run_bzr(['unknowns', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2010-04-27 17:09:48 +0000
@@ -20,6 +20,7 @@
 import os
 
 import bzrlib
+from bzrlib import osutils
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
 
@@ -111,3 +112,11 @@
         self.assertEquals('"Branch Identity" does not seem to contain an '
                           'email address.  This is allowed, but not '
                           'recommended.\n', display)
+
+    def test_whoami_not_set(self):
+        """Ensure whoami error if username is not set.
+        """
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['whoami'], 3)
+        self.assertContainsRe(err, 'Unable to determine your name')

=== modified file 'bzrlib/tests/commands/test_checkout.py'
--- bzrlib/tests/commands/test_checkout.py	2010-04-09 03:58:14 +0000
+++ bzrlib/tests/commands/test_checkout.py	2010-05-03 08:33:32 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 from bzrlib.builtins import cmd_checkout
 from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
 

=== modified file 'bzrlib/tests/per_branch/__init__.py'
--- bzrlib/tests/per_branch/__init__.py	2010-04-01 12:53:53 +0000
+++ bzrlib/tests/per_branch/__init__.py	2010-05-13 15:14:41 +0000
@@ -165,6 +165,7 @@
         'branch',
         'break_lock',
         'check',
+        'config',
         'create_checkout',
         'create_clone',
         'commit',

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2010-04-21 04:12:25 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2010-04-29 08:59:13 +0000
@@ -668,7 +668,8 @@
         this_branch = self.make_branch('this')
         other_branch = self.make_branch('other')
         try:
-            this_branch._format.set_reference(this_branch.bzrdir, other_branch)
+            this_branch._format.set_reference(this_branch.bzrdir, None,
+                other_branch)
         except NotImplementedError:
             # that's ok
             pass

=== modified file 'bzrlib/tests/per_branch/test_break_lock.py'
--- bzrlib/tests/per_branch/test_break_lock.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_break_lock.py	2010-05-06 23:41:35 +0000
@@ -41,7 +41,7 @@
     def test_unlocked_repo_locked(self):
         # break lock on the branch should try on the repository even
         # if the branch isn't locked
-        token = self.branch.repository.lock_write()
+        token = self.branch.repository.lock_write().repository_token
         if token is None:
             self.branch.repository.unlock()
             raise tests.TestNotApplicable(

=== added file 'bzrlib/tests/per_branch/test_config.py'
--- bzrlib/tests/per_branch/test_config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_branch/test_config.py	2010-05-13 16:31:08 +0000
@@ -0,0 +1,38 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for branch.get_config behaviour."""
+
+from bzrlib import (
+    branch,
+    errors,
+    remote,
+    tests,
+    )
+from bzrlib.tests import per_branch
+
+
+class TestGetConfig(per_branch.TestCaseWithBranch):
+
+    def test_set_user_option_with_dict(self):
+        b = self.make_branch('b')
+        config = b.get_config()
+        value_dict = {
+            'ascii': 'abcd', u'unicode \N{WATCH}': u'foo \N{INTERROBANG}'}
+        config.set_user_option('name', value_dict.copy())
+        self.assertEqual(value_dict, config.get_user_option('name'))
+
+

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2010-05-06 23:41:35 +0000
@@ -25,6 +25,7 @@
     lock_helpers,
     per_branch,
     )
+from bzrlib.tests.matchers import *
 
 
 class TestBranchLocking(per_branch.TestCaseWithBranch):
@@ -291,7 +292,7 @@
 
     def test_lock_write_returns_None_refuses_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is not None:
                 # This test does not apply, because this lockable supports
@@ -304,7 +305,7 @@
 
     def test_reentering_lock_write_raises_on_token_mismatch(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -321,7 +322,7 @@
 
     def test_lock_write_with_nonmatching_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses
@@ -344,7 +345,7 @@
         """Test that a branch can be locked with a token, if it is already
         locked by that token."""
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses tokens.
@@ -368,7 +369,7 @@
         # If lock_write did not physically acquire the lock (because it was
         # passed some tokens), then unlock should not physically release it.
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -390,19 +391,17 @@
         # to lock with a token that is no longer valid (because the original
         # lock was released).
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         branch.unlock()
         if token is None:
             # This test does not apply, because this lockable refuses
             # tokens.
             return
-
-        self.assertRaises(errors.TokenMismatch,
-                          branch.lock_write, token=token)
+        self.assertRaises(errors.TokenMismatch, branch.lock_write, token=token)
 
     def test_lock_write_reenter_with_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -420,11 +419,15 @@
         new_branch.lock_write()
         new_branch.unlock()
 
+    def test_lock_write_returns_unlockable(self):
+        branch = self.make_branch('b')
+        self.assertThat(branch.lock_write, ReturnsUnlockable(branch))
+
     def test_leave_lock_in_place(self):
         branch = self.make_branch('b')
         # Lock the branch, then use leave_lock_in_place so that when we
         # unlock the branch the lock is still held on disk.
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -445,7 +448,7 @@
     def test_dont_leave_lock_in_place(self):
         branch = self.make_branch('b')
         # Create a lock on disk.
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses lock
@@ -497,6 +500,10 @@
         branch.lock_read()
         branch.unlock()
 
+    def test_lock_read_returns_unlockable(self):
+        branch = self.make_branch('b')
+        self.assertThat(branch.lock_read, ReturnsUnlockable(branch))
+
     def test_lock_write_locks_repo_too(self):
         if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
             # Branch format 4 is combined with the repository, so this test
@@ -523,6 +530,10 @@
         finally:
             branch.unlock()
 
+    def test_lock_write_returns_unlockable(self):
+        branch = self.make_branch('b')
+        self.assertThat(branch.lock_write, ReturnsUnlockable(branch))
+
     def test_lock_and_unlock_leaves_repo_unlocked(self):
         branch = self.make_branch('b')
         branch.lock_write()

=== modified file 'bzrlib/tests/per_bzrdir_colo/test_unsupported.py'
--- bzrlib/tests/per_bzrdir_colo/test_unsupported.py	2010-04-11 19:40:23 +0000
+++ bzrlib/tests/per_bzrdir_colo/test_unsupported.py	2010-04-16 22:28:06 +0000
@@ -35,15 +35,7 @@
 
 class TestNoColocatedSupport(TestCaseWithBzrDir):
 
-    def test_destroy_colocated_branch(self):
-        branch = self.make_branch('branch')
-        # Colocated branches should not be supported *or* 
-        # destroy_branch should not be supported at all
-        self.assertRaises(
-            (errors.NoColocatedBranchSupport, errors.UnsupportedOperation),
-            branch.bzrdir.destroy_branch, 'colo')
-
-    def test_create_colo_branch(self):
+    def make_bzrdir_with_repo(self):
         # a bzrdir can construct a branch and repository for itself.
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -53,7 +45,27 @@
         t = get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
+        return made_control
+
+    def test_destroy_colocated_branch(self):
+        branch = self.make_branch('branch')
+        # Colocated branches should not be supported *or* 
+        # destroy_branch should not be supported at all
+        self.assertRaises(
+            (errors.NoColocatedBranchSupport, errors.UnsupportedOperation),
+            branch.bzrdir.destroy_branch, 'colo')
+
+    def test_create_colo_branch(self):
+        made_control = self.make_bzrdir_with_repo()
         self.assertRaises(errors.NoColocatedBranchSupport, 
             made_control.create_branch, "colo")
 
+    def test_branch_transport(self):
+        made_control = self.make_bzrdir_with_repo()
+        self.assertRaises(errors.NoColocatedBranchSupport,
+            made_control.get_branch_transport, None, "colo")
 
+    def test_get_branch_reference(self):
+        made_control = self.make_bzrdir_with_repo()
+        self.assertRaises(errors.NoColocatedBranchSupport,
+            made_control.get_branch_reference, "colo")

=== modified file 'bzrlib/tests/per_pack_repository.py'
--- bzrlib/tests/per_pack_repository.py	2010-03-18 05:24:02 +0000
+++ bzrlib/tests/per_pack_repository.py	2010-05-06 23:41:35 +0000
@@ -720,6 +720,16 @@
         self.assertEqual(self.format_supports_external_lookups,
             repo._format.supports_external_lookups)
 
+    def _lock_write(self, write_lockable):
+        """Lock write_lockable, add a cleanup and return the result.
+        
+        :param write_lockable: An object with a lock_write method.
+        :return: The result of write_lockable.lock_write().
+        """
+        result = write_lockable.lock_write()
+        self.addCleanup(result.unlock)
+        return result
+
     def test_abort_write_group_does_not_raise_when_suppressed(self):
         """Similar to per_repository.test_write_group's test of the same name.
 
@@ -727,8 +737,7 @@
         """
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         # Damage the repository on the filesystem
         self.get_transport('').rename('repo', 'foo')
@@ -744,8 +753,7 @@
     def test_abort_write_group_does_raise_when_not_suppressed(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         # Damage the repository on the filesystem
         self.get_transport('').rename('repo', 'foo')
@@ -757,8 +765,7 @@
     def test_suspend_write_group(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         repo.texts.add_lines(('file-id', 'revid'), (), ['lines'])
         wg_tokens = repo.suspend_write_group()
@@ -779,8 +786,7 @@
         repo = self.make_repository('repo', format=self.get_format())
         if repo.chk_bytes is None:
             raise TestNotApplicable('no chk_bytes for this repository')
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text = 'a bit of text\n'
         key = ('sha1:' + osutils.sha_string(text),)
@@ -801,8 +807,7 @@
         # Create a repo, start a write group, insert some data, suspend.
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
@@ -822,8 +827,7 @@
     def test_commit_resumed_write_group(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
@@ -851,16 +855,14 @@
         self.vfs_transport_factory = memory.MemoryServer
         # Make a repository with a suspended write group
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
         wg_tokens = repo.suspend_write_group()
         # Make a new repository
         new_repo = self.make_repository('new_repo', format=self.get_format())
-        token = new_repo.lock_write()
-        self.addCleanup(new_repo.unlock)
+        token = self._lock_write(new_repo).repository_token
         hacked_wg_token = (
             '../../../../repo/.bzr/repository/upload/' + wg_tokens[0])
         self.assertRaises(

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2010-05-11 13:31:31 +0000
@@ -193,6 +193,23 @@
         self.assertEqual(revision_id,
             tree.branch.repository.get_inventory(revision_id).revision_id)
 
+    def test_commit_without_root_errors(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([])
+            def do_commit():
+                try:
+                    list(builder.record_iter_changes(
+                        tree, tree.last_revision(), []))
+                    builder.finish_inventory()
+                except:
+                    builder.abort()
+                    raise
+            self.assertRaises(errors.RootMissing, do_commit)
+        finally:
+            tree.unlock()
+
     def test_commit_without_root_or_record_iter_changes_errors(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()

=== modified file 'bzrlib/tests/per_repository/test_refresh_data.py'
--- bzrlib/tests/per_repository/test_refresh_data.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_refresh_data.py	2010-05-11 14:14:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 from bzrlib import (
     errors,
-    remote,
+    repository,
     )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_repository import TestCaseWithRepository
@@ -45,22 +45,34 @@
         self.addCleanup(repo.unlock)
         repo.refresh_data()
 
-    def test_refresh_data_in_write_group_errors(self):
+    def test_refresh_data_in_write_group(self):
+        # refresh_data may either succeed or raise IsInWriteGroupError during a
+        # write group.
         repo = self.make_repository('.')
         repo.lock_write()
         self.addCleanup(repo.unlock)
         repo.start_write_group()
         self.addCleanup(repo.abort_write_group)
-        # No flow control anticipated, BzrError is enough
-        self.assertRaises(errors.BzrError, repo.refresh_data)
+        try:
+            repo.refresh_data()
+        except repository.IsInWriteGroupError:
+            # This is ok.
+            pass
+        else:
+            # This is ok too.
+            pass
 
-    def test_refresh_data_after_fetch_new_data_visible(self):
-        source = self.make_branch_and_tree('source')
-        revid = source.commit('foo')
-        repo = self.make_repository('target')
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+    def fetch_new_revision_into_concurrent_instance(self, repo, token):
+        """Create a new revision (revid 'new-rev') and fetch it into a
+        concurrent instance of repo.
+        """
+        source = self.make_branch_and_memory_tree('source')
+        source.lock_write()
+        self.addCleanup(source.unlock)
+        source.add([''], ['root-id'])
+        revid = source.commit('foo', rev_id='new-rev')
         # Force data reading on weaves/knits
+        repo.all_revision_ids()
         repo.revisions.keys()
         repo.inventories.keys()
         # server repo is the instance a smart server might hold for this
@@ -75,5 +87,34 @@
             server_repo.fetch(source.branch.repository, revid)
         finally:
             server_repo.unlock()
+
+    def test_refresh_data_after_fetch_new_data_visible(self):
+        repo = self.make_repository('target')
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
         repo.refresh_data()
-        self.assertNotEqual({}, repo.get_graph().get_parent_map([revid]))
+        self.assertNotEqual({}, repo.get_graph().get_parent_map(['new-rev']))
+
+    def test_refresh_data_after_fetch_new_data_visible_in_write_group(self):
+        tree = self.make_branch_and_memory_tree('target')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        tree.add([''], ['root-id'])
+        tree.commit('foo', rev_id='commit-in-target')
+        repo = tree.branch.repository
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        self.addCleanup(repo.abort_write_group)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
+        # Call refresh_data.  It either fails with IsInWriteGroupError, or it
+        # succeeds and the new revisions are visible.
+        try:
+            repo.refresh_data()
+        except repository.IsInWriteGroupError:
+            pass
+        else:
+            self.assertEqual(
+                ['commit-in-target', 'new-rev'],
+                sorted(repo.all_revision_ids()))

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2010-04-21 04:53:33 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2010-05-06 23:41:35 +0000
@@ -47,6 +47,7 @@
     per_repository,
     test_server,
     )
+from bzrlib.tests.matchers import *
 
 
 class TestRepositoryMakeBranchAndTree(per_repository.TestCaseWithRepository):
@@ -1087,7 +1088,7 @@
         repo = self.make_repository('r')
         # Lock the repository, then use leave_lock_in_place so that when we
         # unlock the repository the lock is still held on disk.
-        token = repo.lock_write()
+        token = repo.lock_write().repository_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -1107,7 +1108,7 @@
     def test_dont_leave_lock_in_place(self):
         repo = self.make_repository('r')
         # Create a lock on disk.
-        token = repo.lock_write()
+        token = repo.lock_write().repository_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -1141,6 +1142,14 @@
         repo.lock_read()
         repo.unlock()
 
+    def test_lock_read_returns_unlockable(self):
+        repo = self.make_repository('r')
+        self.assertThat(repo.lock_read, ReturnsUnlockable(repo))
+
+    def test_lock_write_returns_unlockable(self):
+        repo = self.make_repository('r')
+        self.assertThat(repo.lock_write, ReturnsUnlockable(repo))
+
 
 class TestCaseWithComplexRepository(per_repository.TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/per_tree/__init__.py'
--- bzrlib/tests/per_tree/__init__.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_tree/__init__.py	2010-05-06 23:41:35 +0000
@@ -386,6 +386,7 @@
         'inv',
         'iter_search_rules',
         'list_files',
+        'locking',
         'path_content_summary',
         'revision_tree',
         'test_trees',

=== added file 'bzrlib/tests/per_tree/test_locking.py'
--- bzrlib/tests/per_tree/test_locking.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_tree/test_locking.py	2010-05-06 23:41:35 +0000
@@ -0,0 +1,28 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Test that all trees support Tree.lock_read()"""
+
+from bzrlib.tests.matchers import *
+from bzrlib.tests.per_tree import TestCaseWithTree
+
+
+class TestLocking(TestCaseWithTree):
+
+    def test_lock_read(self):
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self.workingtree_to_test_tree(work_tree)
+        self.assertThat(tree.lock_read, ReturnsUnlockable(tree))

=== modified file 'bzrlib/tests/per_versionedfile.py'
--- bzrlib/tests/per_versionedfile.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_versionedfile.py	2010-05-14 14:29:28 +0000
@@ -999,13 +999,20 @@
         # we should be able to read from http with a versioned file.
         vf = self.get_file()
         # try an empty file access
-        readonly_vf = self.get_factory()('foo', get_transport(self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo', get_transport(
+                self.get_readonly_url('.')))
         self.assertEqual([], readonly_vf.versions())
+
+    def test_readonly_http_works_with_feeling(self):
+        # we should be able to read from http with a versioned file.
+        vf = self.get_file()
         # now with feeling.
         vf.add_lines('1', [], ['a\n'])
         vf.add_lines('2', ['1'], ['b\n', 'a\n'])
-        readonly_vf = self.get_factory()('foo', get_transport(self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo', get_transport(
+                self.get_readonly_url('.')))
         self.assertEqual(['1', '2'], vf.versions())
+        self.assertEqual(['1', '2'], readonly_vf.versions())
         for version in readonly_vf.versions():
             readonly_vf.get_lines(version)
 

=== modified file 'bzrlib/tests/per_workingtree/test_locking.py'
--- bzrlib/tests/per_workingtree/test_locking.py	2009-08-25 23:38:10 +0000
+++ bzrlib/tests/per_workingtree/test_locking.py	2010-05-11 14:14:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
     lockdir,
     )
 from bzrlib.tests import TestSkipped
+from bzrlib.tests.matchers import *
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -44,6 +45,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_read_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_read, ReturnsUnlockable(wt))
+
     def test_trivial_lock_write_unlock(self):
         """Locking for write and unlocking should work trivially."""
         wt = self.make_branch_and_tree('.')
@@ -59,6 +64,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_write_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_write, ReturnsUnlockable(wt))
+
     def test_trivial_lock_tree_write_unlock(self):
         """Locking for tree write is ok when the branch is not locked."""
         wt = self.make_branch_and_tree('.')
@@ -74,6 +83,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_tree_write_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_tree_write, ReturnsUnlockable(wt))
+
     def test_trivial_lock_tree_write_branch_read_locked(self):
         """It is ok to lock_tree_write when the branch is read locked."""
         wt = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/per_workingtree/test_remove.py'
--- bzrlib/tests/per_workingtree/test_remove.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_remove.py	2010-04-30 10:28:36 +0000
@@ -278,6 +278,20 @@
         self.assertRemovedAndDeleted(files)
         tree._validate()
 
+    def test_remove_directory_with_changed_emigrated_file(self):
+        # As per bug #129880
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('somedir/',), ('somedir/file', 'contents')])
+        tree.add(['somedir', 'somedir/file'])
+        tree.commit(message="first")
+        self.build_tree_contents([('somedir/file', 'changed')])
+        tree.rename_one('somedir/file', 'moved-file')
+        tree.remove('somedir', keep_files=False)
+        self.assertNotInWorkingTree('somedir')
+        self.failIfExists('somedir')
+        self.assertInWorkingTree('moved-file')
+        self.failUnlessExists('moved-file')
+
     def test_remove_directory_with_renames(self):
         """Delete directory with renames in or out."""
 

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-03-11 04:33:41 +0000
+++ bzrlib/tests/stub_sftp.py	2010-05-14 09:34:16 +0000
@@ -283,15 +283,19 @@
         self._socket.close()
 
     def run(self):
+        trace.mutter('SocketListener %r has started', self)
         while True:
             readable, writable_unused, exception_unused = \
                 select.select([self._socket], [], [], 0.1)
             if self._stop_event.isSet():
+                trace.mutter('SocketListener %r has stopped', self)
                 return
             if len(readable) == 0:
                 continue
             try:
                 s, addr_unused = self._socket.accept()
+                trace.mutter('SocketListener %r has accepted connection %r',
+                    self, s)
                 # because the loopback socket is inline, and transports are
                 # never explicitly closed, best to launch a new thread.
                 threading.Thread(target=self._callback, args=(s,)).start()
@@ -442,11 +446,11 @@
                 'the local current working directory.' % (backing_server,))
         self._original_vendor = ssh._ssh_vendor_manager._cached_ssh_vendor
         ssh._ssh_vendor_manager._cached_ssh_vendor = self._vendor
-        # FIXME: the following block should certainly just be self._homedir =
-        # osutils.getcwd() but that fails badly on Unix -- vila 20100224
         if sys.platform == 'win32':
             # Win32 needs to use the UNICODE api
             self._homedir = os.getcwdu()
+            # Normalize the path or it will be wrongly escaped
+            self._homedir = osutils.normpath(self._homedir)
         else:
             # But Linux SFTP servers should just deal in bytestreams
             self._homedir = os.getcwd()

=== modified file 'bzrlib/tests/test__chk_map.py'
--- bzrlib/tests/test__chk_map.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__chk_map.py	2010-05-11 10:45:26 +0000
@@ -18,6 +18,7 @@
 
 from bzrlib import (
     chk_map,
+    inventory,
     tests,
     )
 from bzrlib.static_tuple import StaticTuple
@@ -236,3 +237,44 @@
         self.assertEqual(("sha1:1234",), node.key())
         self.assertEqual('pref\x00fo', node._search_prefix)
         self.assertEqual({'pref\x00fo\x00': ('sha1:abcd',)}, node._items)
+
+
+class Test_BytesToTextKey(tests.TestCase):
+
+    def assertBytesToTextKey(self, key, bytes):
+        self.assertEqual(key,
+                         self.module._bytes_to_text_key(bytes))
+
+    def assertBytesToTextKeyRaises(self, bytes):
+        # These are invalid bytes, and we want to make sure the code under test
+        # raises an exception rather than segfaults, etc. We don't particularly
+        # care what exception.
+        self.assertRaises(Exception, self.module._bytes_to_text_key, bytes)
+
+    def test_file(self):
+        self.assertBytesToTextKey(('file-id', 'revision-id'),
+                 'file: file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_no_kind(self):
+        self.assertBytesToTextKeyRaises(
+                 'file  file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_no_space(self):
+        self.assertBytesToTextKeyRaises(
+                 'file:file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_too_short_file_id(self):
+        self.assertBytesToTextKeyRaises('file:file-id')
+
+    def test_invalid_too_short_parent_id(self):
+        self.assertBytesToTextKeyRaises('file:file-id\nparent-id')
+
+    def test_invalid_too_short_name(self):
+        self.assertBytesToTextKeyRaises('file:file-id\nparent-id\nname')
+
+    def test_dir(self):
+        self.assertBytesToTextKey(('dir-id', 'revision-id'),
+                 'dir: dir-id\nparent-id\nname\nrevision-id')

=== modified file 'bzrlib/tests/test__walkdirs_win32.py'
--- bzrlib/tests/test__walkdirs_win32.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__walkdirs_win32.py	2010-05-16 12:21:31 +0000
@@ -74,7 +74,7 @@
         self.assertReadDir([('foo', 'foo', 'file', u'./foo')],
             '', u'.')
 
-    def test_dir(self):
+    def test_directory(self):
         self.build_tree(['bar/'])
         self.assertReadDir([('bar', 'bar', 'directory', u'./bar')],
             '', u'.')

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_commands.py	2010-05-11 11:05:49 +0000
@@ -79,6 +79,11 @@
         c = self.get_command([option.Option('foo', hidden=False)])
         self.assertContainsRe(c.get_help_text(), '--foo')
 
+    def test_no_help_init_failure(self):
+        class cmd_foo(commands.Command):
+            pass
+        self.assertRaises(ValueError, cmd_foo)
+
 
 class TestGetAlias(tests.TestCase):
 

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_errors.py	2010-04-30 09:05:29 +0000
@@ -713,3 +713,9 @@
         e = errors.FileTimestampUnavailable("/path/foo")
         self.assertEquals("The filestamp for /path/foo is not available.",
             str(e))
+            
+    def test_transform_rename_failed(self):
+        e = errors.TransformRenameFailed(u"from", u"to", "readonly file", 2)
+        self.assertEquals(
+            u"Failed to rename from to to: readonly file",
+            str(e))

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_help.py	2010-05-07 04:25:37 +0000
@@ -122,6 +122,7 @@
             '\n'
             'Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
@@ -142,6 +143,7 @@
             '\n'
             ':Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
@@ -177,6 +179,7 @@
             '\n'
             'Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
@@ -196,6 +199,7 @@
             '\n'
             'Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
@@ -230,6 +234,7 @@
             '\n'
             'Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
@@ -273,6 +278,7 @@
             '\n'
             'Options:\n'
             '  --usage        Show usage message and options.\n'
+            '  -0, --null     Use an ASCII NUL (\\0) separator rather than a newline.\n'
             '  -v, --verbose  Display more information.\n'
             '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'

=== modified file 'bzrlib/tests/test_ignores.py'
--- bzrlib/tests/test_ignores.py	2009-12-26 21:31:56 +0000
+++ bzrlib/tests/test_ignores.py	2010-05-03 04:51:58 +0000
@@ -165,27 +165,55 @@
 
 
 class TestTreeIgnores(TestCaseWithTransport):
+    
+    def assertPatternsEquals(self, patterns):
+        contents = open(".bzrignore", 'rU').read().strip().split('\n')
+        self.assertEquals(sorted(patterns), sorted(contents))
 
     def test_new_file(self):
         tree = self.make_branch_and_tree(".")
         ignores.tree_ignores_add_patterns(tree, ["myentry"])
         self.assertTrue(tree.has_filename(".bzrignore"))
-        self.assertEquals("myentry\n",
-                          open(".bzrignore", 'r').read())
+        self.assertPatternsEquals(["myentry"])
 
     def test_add_to_existing(self):
         tree = self.make_branch_and_tree(".")
         self.build_tree_contents([('.bzrignore', "myentry1\n")])
         tree.add([".bzrignore"])
         ignores.tree_ignores_add_patterns(tree, ["myentry2", "foo"])
-        self.assertEquals("myentry1\nmyentry2\nfoo\n",
-                          open(".bzrignore", 'r').read())
+        self.assertPatternsEquals(["myentry1", "myentry2", "foo"])
 
     def test_adds_ending_newline(self):
         tree = self.make_branch_and_tree(".")
         self.build_tree_contents([('.bzrignore', "myentry1")])
         tree.add([".bzrignore"])
         ignores.tree_ignores_add_patterns(tree, ["myentry2"])
-        self.assertEquals("myentry1\nmyentry2\n",
-                          open(".bzrignore", 'r').read())
-
+        self.assertPatternsEquals(["myentry1", "myentry2"])
+        text = open(".bzrignore", 'r').read()
+        self.assertTrue(text.endswith('\r\n') or
+                        text.endswith('\n') or
+                        text.endswith('\r'))
+
+    def test_does_not_add_dupe(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore', "myentry\n")])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, ["myentry"])
+        self.assertPatternsEquals(["myentry"])
+
+    def test_non_ascii(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore',
+                                   u"myentry\u1234\n".encode('utf-8'))])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, [u"myentry\u5678"])
+        self.assertPatternsEquals([u"myentry\u1234".encode('utf-8'),
+                                   u"myentry\u5678".encode('utf-8')])
+
+    def test_crlf(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore', "myentry1\r\n")])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, ["myentry2", "foo"])
+        self.assertEquals(open('.bzrignore', 'rb').read(), 'myentry1\r\nmyentry2\r\nfoo\r\n')
+        self.assertPatternsEquals(["myentry1", "myentry2", "foo"])

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2010-04-28 07:10:36 +0000
+++ bzrlib/tests/test_log.py	2010-05-05 08:18:32 +0000
@@ -1543,6 +1543,151 @@
     def test_bugs_handler_present(self):
         self.properties_handler_registry.get('bugs_properties_handler')
 
+
+class TestLogForAuthors(TestCaseForLogFormatter):
+
+    def setUp(self):
+        TestCaseForLogFormatter.setUp(self)
+        self.wt = self.make_standard_commit('nicky',
+            authors=['John Doe <jdoe@example.com>',
+                     'Jane Rey <jrey@example.com>'])
+
+    def assertFormatterResult(self, formatter, who, result):
+        formatter_kwargs = dict()
+        if who is not None:
+            author_list_handler = log.author_list_registry.get(who)
+            formatter_kwargs['author_list_handler'] = author_list_handler
+        TestCaseForLogFormatter.assertFormatterResult(self, result,
+            self.wt.branch, formatter, formatter_kwargs=formatter_kwargs)
+
+    def test_line_default(self):
+        self.assertFormatterResult(log.LineLogFormatter, None, """\
+1: John Doe 2005-11-22 add a
+""")
+
+    def test_line_committer(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'committer', """\
+1: Lorem Ipsum 2005-11-22 add a
+""")
+
+    def test_line_first(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'first', """\
+1: John Doe 2005-11-22 add a
+""")
+
+    def test_line_all(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'all', """\
+1: John Doe, Jane Rey 2005-11-22 add a
+""")
+
+
+    def test_short_default(self):
+        self.assertFormatterResult(log.ShortLogFormatter, None, """\
+    1 John Doe\t2005-11-22
+      add a
+
+""")
+
+    def test_short_committer(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'committer', """\
+    1 Lorem Ipsum\t2005-11-22
+      add a
+
+""")
+
+    def test_short_first(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'first', """\
+    1 John Doe\t2005-11-22
+      add a
+
+""")
+
+    def test_short_all(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'all', """\
+    1 John Doe, Jane Rey\t2005-11-22
+      add a
+
+""")
+
+    def test_long_default(self):
+        self.assertFormatterResult(log.LongLogFormatter, None, """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>, Jane Rey <jrey@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_committer(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'committer', """\
+------------------------------------------------------------
+revno: 1
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_first(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'first', """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_all(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'all', """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>, Jane Rey <jrey@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_gnu_changelog_default(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, None, """\
+2005-11-22  John Doe  <jdoe@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_committer(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'committer', """\
+2005-11-22  Lorem Ipsum  <test@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_first(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'first', """\
+2005-11-22  John Doe  <jdoe@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_all(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'all', """\
+2005-11-22  John Doe  <jdoe@example.com>, Jane Rey  <jrey@example.com>
+
+\tadd a
+
+""")
+
 class TestLogExcludeAncestry(tests.TestCaseWithTransport):
 
     def make_branch_with_alternate_ancestries(self, relpath='.'):

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2010-04-27 07:52:08 +0000
+++ bzrlib/tests/test_osutils.py	2010-05-13 17:32:55 +0000
@@ -184,13 +184,6 @@
         shape = sorted(os.listdir('.'))
         self.assertEquals(['A', 'B'], shape)
 
-    def test_rename_error(self):
-        # We wrap os.rename to make it give an error including the filenames
-        # https://bugs.launchpad.net/bzr/+bug/491763
-        err = self.assertRaises(OSError, osutils.rename,
-            'nonexistent', 'target')
-        self.assertContainsString(str(err), 'nonexistent')
-
 
 class TestRandChars(tests.TestCase):
 
@@ -2018,3 +2011,14 @@
         self.assertEquals(self.path, 'test_file')
         self.assertEquals(self.uid, s.st_uid)
         self.assertEquals(self.gid, s.st_gid)
+
+class TestGetuserUnicode(tests.TestCase):
+
+    def test_ascii_user(self):
+        osutils.set_or_unset_env('LOGNAME', 'jrandom')
+        self.assertEqual(u'jrandom', osutils.getuser_unicode())
+
+    def test_unicode_user(self):
+        ue = osutils.get_user_encoding()
+        osutils.set_or_unset_env('LOGNAME', u'jrandom\xb6'.encode(ue))
+        self.assertEqual(u'jrandom\xb6', osutils.getuser_unicode())

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2010-04-20 22:38:03 +0000
+++ bzrlib/tests/test_plugins.py	2010-05-16 13:49:19 +0000
@@ -842,8 +842,8 @@
         osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
-        self.assertEqual('non-standard-dir/__init__.py',
-                         bzrlib.plugins.test_foo.__file__)
+        self.assertIsSameRealPath('non-standard-dir/__init__.py',
+                                  bzrlib.plugins.test_foo.__file__)
 
         # Try importing again now that the source has been compiled
         self._unregister_plugin('test_foo')
@@ -854,8 +854,8 @@
             suffix = 'pyc'
         else:
             suffix = 'pyo'
-        self.assertEqual('non-standard-dir/__init__.%s' % suffix,
-                         bzrlib.plugins.test_foo.__file__)
+        self.assertIsSameRealPath('non-standard-dir/__init__.%s' % suffix,
+                                  bzrlib.plugins.test_foo.__file__)
 
     def test_submodule_loading(self):
         # We create an additional directory under the one for test_foo
@@ -866,8 +866,8 @@
         self.assertEqual('bzrlib.plugins.test_foo',
                          bzrlib.plugins.test_foo.__package__)
         import bzrlib.plugins.test_foo.test_bar
-        self.assertEqual('non-standard-dir/test_bar/__init__.py',
-                         bzrlib.plugins.test_foo.test_bar.__file__)
+        self.assertIsSameRealPath('non-standard-dir/test_bar/__init__.py',
+                                  bzrlib.plugins.test_foo.test_bar.__file__)
 
     def test_loading_from___init__only(self):
         # We rename the existing __init__.py file to ensure that we don't load

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2010-05-03 04:08:50 +0000
+++ bzrlib/tests/test_remote.py	2010-05-14 09:02:35 +0000
@@ -609,7 +609,7 @@
         # _get_tree_branch is a form of open_branch, but it should only ask for
         # branch opening, not any other network requests.
         calls = []
-        def open_branch():
+        def open_branch(name=None):
             calls.append("Called")
             return "a-branch"
         transport = MemoryTransport()
@@ -1652,6 +1652,32 @@
         branch.unlock()
         self.assertFinished(client)
 
+    def test_set_option_with_dict(self):
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.lock_write', ('memory:///', '', ''),
+            'success', ('ok', 'branch token', 'repo token'))
+        encoded_dict_value = 'd5:ascii1:a11:unicode \xe2\x8c\x9a3:\xe2\x80\xbde'
+        client.add_expected_call(
+            'Branch.set_config_option_dict', ('memory:///', 'branch token',
+            'repo token', encoded_dict_value, 'foo', ''),
+            'success', ())
+        client.add_expected_call(
+            'Branch.unlock', ('memory:///', 'branch token', 'repo token'),
+            'success', ('ok',))
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        branch.lock_write()
+        config = branch._get_config()
+        config.set_option(
+            {'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'},
+            'foo')
+        branch.unlock()
+        self.assertFinished(client)
+
     def test_backwards_compat_set_option(self):
         self.setup_smart_server_with_call_log()
         branch = self.make_branch('.')
@@ -1664,6 +1690,20 @@
         self.assertLength(10, self.hpss_calls)
         self.assertEqual('value', branch._get_config().get_option('name'))
 
+    def test_backwards_compat_set_option_with_dict(self):
+        self.setup_smart_server_with_call_log()
+        branch = self.make_branch('.')
+        verb = 'Branch.set_config_option_dict'
+        self.disable_verb(verb)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        self.reset_smart_call_log()
+        config = branch._get_config()
+        value_dict = {'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'}
+        config.set_option(value_dict, 'name')
+        self.assertLength(10, self.hpss_calls)
+        self.assertEqual(value_dict, branch._get_config().get_option('name'))
+
 
 class TestBranchLockWrite(RemoteBranchTestCase):
 
@@ -2257,6 +2297,7 @@
         self.setup_smart_server_with_call_log()
         tree = self.make_branch_and_memory_tree('.')
         tree.lock_write()
+        tree.add('')
         rev1 = tree.commit('First')
         rev2 = tree.commit('Second')
         tree.unlock()
@@ -2301,11 +2342,11 @@
         transport_path = 'quack'
         repo, client = self.setup_fake_client_and_repository(transport_path)
         client.add_success_response('ok', 'a token')
-        result = repo.lock_write()
+        token = repo.lock_write().repository_token
         self.assertEqual(
             [('call', 'Repository.lock_write', ('quack/', ''))],
             client._calls)
-        self.assertEqual('a token', result)
+        self.assertEqual('a token', token)
 
     def test_lock_write_already_locked(self):
         transport_path = 'quack'

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_selftest.py	2010-05-14 09:48:51 +0000
@@ -609,12 +609,13 @@
                 l.attempt_lock()
         test = TestDanglingLock('test_function')
         result = test.run()
+        total_failures = result.errors + result.failures
         if self._lock_check_thorough:
-            self.assertEqual(1, len(result.errors))
+            self.assertLength(1, total_failures)
         else:
             # When _lock_check_thorough is disabled, then we don't trigger a
             # failure
-            self.assertEqual(0, len(result.errors))
+            self.assertLength(0, total_failures)
 
 
 class TestTestCaseWithTransport(tests.TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2010-03-02 22:25:58 +0000
+++ bzrlib/tests/test_smart.py	2010-05-13 16:17:54 +0000
@@ -767,8 +767,8 @@
 class TestLockedBranch(tests.TestCaseWithMemoryTransport):
 
     def get_lock_tokens(self, branch):
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token = branch.lock_write().branch_token
+        repo_token = branch.repository.lock_write().repository_token
         branch.repository.unlock()
         return branch_token, repo_token
 
@@ -802,6 +802,44 @@
         branch.unlock()
 
 
+class TestSmartServerBranchRequestSetConfigOptionDict(TestLockedBranch):
+
+    def setUp(self):
+        TestLockedBranch.setUp(self)
+        # A dict with non-ascii keys and values to exercise unicode
+        # roundtripping.
+        self.encoded_value_dict = (
+            'd5:ascii1:a11:unicode \xe2\x8c\x9a3:\xe2\x80\xbde')
+        self.value_dict = {
+            'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'}
+
+    def test_value_name(self):
+        branch = self.make_branch('.')
+        request = smart_branch.SmartServerBranchRequestSetConfigOptionDict(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token,
+            self.encoded_value_dict, 'foo', '')
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), result)
+        self.assertEqual(self.value_dict, config.get_option('foo'))
+        # Cleanup
+        branch.unlock()
+
+    def test_value_name_section(self):
+        branch = self.make_branch('.')
+        request = smart_branch.SmartServerBranchRequestSetConfigOptionDict(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token,
+            self.encoded_value_dict, 'foo', 'gam')
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), result)
+        self.assertEqual(self.value_dict, config.get_option('foo', 'gam'))
+        # Cleanup
+        branch.unlock()
+
+
 class TestSmartServerBranchRequestSetTagsBytes(TestLockedBranch):
     # Only called when the branch format and tags match [yay factory
     # methods] so only need to test straight forward cases.
@@ -1073,7 +1111,7 @@
             response)
 
 
-class TestSmartServerBranchRequestSetParent(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestSetParent(TestLockedBranch):
 
     def test_set_parent_none(self):
         branch = self.make_branch('base', format="1.9")
@@ -1082,12 +1120,10 @@
         branch.unlock()
         request = smart_branch.SmartServerBranchRequestSetParentLocation(
             self.get_transport())
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         try:
             response = request.execute('base', branch_token, repo_token, '')
         finally:
-            branch.repository.unlock()
             branch.unlock()
         self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), response)
         self.assertEqual(None, branch.get_parent())
@@ -1096,13 +1132,11 @@
         branch = self.make_branch('base', format="1.9")
         request = smart_branch.SmartServerBranchRequestSetParentLocation(
             self.get_transport())
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         try:
             response = request.execute('base', branch_token, repo_token,
             'http://bar/')
         finally:
-            branch.repository.unlock()
             branch.unlock()
         self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), response)
         self.assertEqual('http://bar/', branch.get_parent())
@@ -1137,7 +1171,7 @@
             response)
 
 
-class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestLockWrite(TestLockedBranch):
 
     def setUp(self):
         tests.TestCaseWithMemoryTransport.setUp(self)
@@ -1165,7 +1199,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.')
-        branch_token = branch.lock_write()
+        branch_token = branch.lock_write().branch_token
         branch.leave_lock_in_place()
         branch.unlock()
         response = request.execute('')
@@ -1180,9 +1214,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
@@ -1203,9 +1235,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
@@ -1226,7 +1256,7 @@
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
         repo = branch.repository
-        repo_token = repo.lock_write()
+        repo_token = repo.lock_write().repository_token
         repo.leave_lock_in_place()
         repo.unlock()
         response = request.execute('')
@@ -1249,7 +1279,7 @@
         self.assertEqual('LockFailed', error_name)
 
 
-class TestSmartServerBranchRequestUnlock(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestUnlock(TestLockedBranch):
 
     def setUp(self):
         tests.TestCaseWithMemoryTransport.setUp(self)
@@ -1259,9 +1289,7 @@
         request = smart_branch.SmartServerBranchRequestUnlock(backing)
         branch = self.make_branch('.', format='knit')
         # Lock the branch
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         # Unlock the branch (and repo) object, leaving the physical locks
         # in place.
         branch.leave_lock_in_place()
@@ -1291,13 +1319,12 @@
         request = smart_branch.SmartServerBranchRequestUnlock(backing)
         branch = self.make_branch('.', format='knit')
         # Lock the repository.
-        repo_token = branch.repository.lock_write()
+        repo_token = branch.repository.lock_write().repository_token
         branch.repository.leave_lock_in_place()
         branch.repository.unlock()
         # Issue branch lock_write request on the unlocked branch (with locked
         # repo).
-        response = request.execute(
-            '', 'branch token', repo_token)
+        response = request.execute('', 'branch token', repo_token)
         self.assertEqual(
             smart_req.SmartServerResponse(('TokenMismatch',)), response)
         # Cleanup
@@ -1610,7 +1637,7 @@
         backing = self.get_transport()
         request = smart_repo.SmartServerRepositoryLockWrite(backing)
         repository = self.make_repository('.', format='knit')
-        repo_token = repository.lock_write()
+        repo_token = repository.lock_write().repository_token
         repository.leave_lock_in_place()
         repository.unlock()
         response = request.execute('')
@@ -1658,7 +1685,7 @@
         request = smart_repo.SmartServerRepositoryInsertStreamLocked(
             backing)
         repository = self.make_repository('.', format='knit')
-        lock_token = repository.lock_write()
+        lock_token = repository.lock_write().repository_token
         response = request.execute('', '', lock_token)
         self.assertEqual(None, response)
         response = request.do_chunk(self.make_empty_byte_stream(repository))
@@ -1672,7 +1699,7 @@
         request = smart_repo.SmartServerRepositoryInsertStreamLocked(
             backing)
         repository = self.make_repository('.', format='knit')
-        lock_token = repository.lock_write()
+        lock_token = repository.lock_write().repository_token
         self.assertRaises(
             errors.TokenMismatch, request.execute, '', '', 'wrong-token')
         repository.unlock()
@@ -1687,7 +1714,7 @@
         backing = self.get_transport()
         request = smart_repo.SmartServerRepositoryUnlock(backing)
         repository = self.make_repository('.', format='knit')
-        token = repository.lock_write()
+        token = repository.lock_write().repository_token
         repository.leave_lock_in_place()
         repository.unlock()
         response = request.execute('', token)

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2010-03-12 10:52:01 +0000
+++ bzrlib/tests/test_transform.py	2010-05-13 17:32:55 +0000
@@ -25,23 +25,37 @@
     filters,
     generate_ids,
     osutils,
-    progress,
     revision as _mod_revision,
     rules,
     tests,
     urlutils,
     )
 from bzrlib.bzrdir import BzrDir
-from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
-                              UnversionedParent, ParentLoop, DeletingParent,
-                              NonDirectoryParent)
+from bzrlib.conflicts import (
+    DeletingParent,
+    DuplicateEntry,
+    DuplicateID,
+    MissingParent,
+    NonDirectoryParent,
+    ParentLoop,
+    UnversionedParent,
+)
 from bzrlib.diff import show_diff_trees
-from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
-                           ReusingTransform, CantMoveRoot,
-                           PathsNotVersionedError, ExistingLimbo,
-                           ExistingPendingDeletion, ImmortalLimbo,
-                           ImmortalPendingDeletion, LockError)
-from bzrlib.osutils import file_kind, pathjoin
+from bzrlib.errors import (
+    DuplicateKey,
+    ExistingLimbo,
+    ExistingPendingDeletion,
+    ImmortalLimbo,
+    ImmortalPendingDeletion,
+    LockError,
+    MalformedTransform,
+    NoSuchFile,
+    ReusingTransform,
+)
+from bzrlib.osutils import (
+    file_kind,
+    pathjoin,
+)
 from bzrlib.merge import Merge3Merger, Merger
 from bzrlib.tests import (
     HardlinkFeature,
@@ -49,12 +63,20 @@
     TestCase,
     TestCaseInTempDir,
     TestSkipped,
-    )
-from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths,
-                              resolve_conflicts, cook_conflicts,
-                              build_tree, get_backup_name,
-                              _FileMover, resolve_checkout,
-                              TransformPreview, create_from_tree)
+)
+from bzrlib.transform import (
+    build_tree,
+    create_from_tree,
+    cook_conflicts,
+    _FileMover,
+    FinalPaths,
+    get_backup_name,
+    resolve_conflicts,
+    resolve_checkout,
+    ROOT_PARENT,
+    TransformPreview,
+    TreeTransform,
+)
 
 
 class TestTreeTransform(tests.TestCaseWithTransport):
@@ -830,6 +852,32 @@
         rename.set_executability(True, myfile)
         rename.apply()
 
+    def test_rename_fails(self):
+        # see https://bugs.launchpad.net/bzr/+bug/491763
+        create, root_id = self.get_transform()
+        first_dir = create.new_directory('first-dir', root_id, 'first-id')
+        myfile = create.new_file('myfile', root_id, 'myfile-text',
+                                 'myfile-id')
+        create.apply()
+        # make the file and directory readonly in the hope this will prevent
+        # renames
+        osutils.make_readonly(self.wt.abspath('first-dir'))
+        osutils.make_readonly(self.wt.abspath('myfile'))
+        # now transform to rename
+        rename_transform, root_id = self.get_transform()
+        file_trans_id = rename_transform.trans_id_file_id('myfile-id')
+        dir_id = rename_transform.trans_id_file_id('first-id')
+        rename_transform.adjust_path('newname', dir_id, file_trans_id)
+        e = self.assertRaises(errors.TransformRenameFailed,
+            rename_transform.apply)
+        # Looks like: 
+        # "Failed to rename .../work/.bzr/checkout/limbo/new-1
+        # to .../first-dir/newname: [Errno 13] Permission denied"
+        # so the first filename is not visible in it; we expect a strerror but
+        # it may vary per OS and language so it's not checked here
+        self.assertContainsRe(str(e),
+            "Failed to rename .*first-dir.newname:")
+
     def test_set_executability_order(self):
         """Ensure that executability behaves the same, no matter what order.
 
@@ -2091,6 +2139,38 @@
         self.assertRaises(errors.MalformedTransform, tt.commit, branch,
                           'message')
 
+    def test_commit_rich_revision_data(self):
+        branch, tt = self.get_branch_and_transform()
+        rev_id = tt.commit(branch, 'message', timestamp=1, timezone=43201,
+                           committer='me <me@example.com>',
+                           revprops={'foo': 'bar'}, revision_id='revid-1',
+                           authors=['Author1 <author1@example.com>',
+                              'Author2 <author2@example.com>',
+                               ])
+        self.assertEqual('revid-1', rev_id)
+        revision = branch.repository.get_revision(rev_id)
+        self.assertEqual(1, revision.timestamp)
+        self.assertEqual(43201, revision.timezone)
+        self.assertEqual('me <me@example.com>', revision.committer)
+        self.assertEqual(['Author1 <author1@example.com>',
+                          'Author2 <author2@example.com>'],
+                         revision.get_apparent_authors())
+        del revision.properties['authors']
+        self.assertEqual({'foo': 'bar',
+                          'branch-nick': 'tree'},
+                         revision.properties)
+
+    def test_no_explicit_revprops(self):
+        branch, tt = self.get_branch_and_transform()
+        rev_id = tt.commit(branch, 'message', authors=[
+            'Author1 <author1@example.com>',
+            'Author2 <author2@example.com>', ])
+        revision = branch.repository.get_revision(rev_id)
+        self.assertEqual(['Author1 <author1@example.com>',
+                          'Author2 <author2@example.com>'],
+                         revision.get_apparent_authors())
+        self.assertEqual('tree', revision.properties['branch-nick'])
+
 
 class MockTransform(object):
 

=== modified file 'bzrlib/tests/treeshape.py'
--- bzrlib/tests/treeshape.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/treeshape.py	2010-05-05 00:05:29 +0000
@@ -41,6 +41,10 @@
 
     The template is built relative to the Python process's current
     working directory.
+
+    ('foo/',) will build a directory.
+    ('foo', 'bar') will write 'bar' to 'foo'
+    ('foo@', 'linktarget') will raise an error
     """
     for tt in template:
         name = tt[0]

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2010-03-30 14:44:30 +0000
+++ bzrlib/trace.py	2010-05-05 02:35:02 +0000
@@ -111,6 +111,12 @@
 
 
 def note(*args, **kwargs):
+    """Output a note to the user.
+
+    Takes the same parameters as logging.info.
+
+    :return: None
+    """
     # FIXME note always emits utf-8, regardless of the terminal encoding
     #
     # FIXME: clearing the ui and then going through the abstract logging

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-04-30 11:35:43 +0000
+++ bzrlib/transform.py	2010-05-13 17:32:55 +0000
@@ -25,6 +25,7 @@
     annotate,
     bencode,
     bzrdir,
+    commit,
     delta,
     errors,
     inventory,
@@ -927,12 +928,27 @@
         """
         return _PreviewTree(self)
 
-    def commit(self, branch, message, merge_parents=None, strict=False):
+    def commit(self, branch, message, merge_parents=None, strict=False,
+               timestamp=None, timezone=None, committer=None, authors=None,
+               revprops=None, revision_id=None):
         """Commit the result of this TreeTransform to a branch.
 
         :param branch: The branch to commit to.
         :param message: The message to attach to the commit.
-        :param merge_parents: Additional parents specified by pending merges.
+        :param merge_parents: Additional parent revision-ids specified by
+            pending merges.
+        :param strict: If True, abort the commit if there are unversioned
+            files.
+        :param timestamp: if not None, seconds-since-epoch for the time and
+            date.  (May be a float.)
+        :param timezone: Optional timezone for timestamp, as an offset in
+            seconds.
+        :param committer: Optional committer in email-id format.
+            (e.g. "J Random Hacker <jrandom@example.com>")
+        :param authors: Optional list of authors in email-id format.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.  (Specifying a revision-id
+            may reduce performance for some non-native formats.)
         :return: The revision_id of the revision committed.
         """
         self._check_malformed()
@@ -955,7 +971,13 @@
         if self._tree.get_revision_id() != last_rev_id:
             raise ValueError('TreeTransform not based on branch basis: %s' %
                              self._tree.get_revision_id())
-        builder = branch.get_commit_builder(parent_ids)
+        revprops = commit.Commit.update_revprops(revprops, branch, authors)
+        builder = branch.get_commit_builder(parent_ids,
+                                            timestamp=timestamp,
+                                            timezone=timezone,
+                                            committer=committer,
+                                            revprops=revprops,
+                                            revision_id=revision_id)
         preview = self.get_preview_tree()
         list(builder.record_iter_changes(preview, last_rev_id,
                                          self.iter_changes()))
@@ -1635,7 +1657,7 @@
                       or trans_id in self._new_parent):
                     try:
                         mover.rename(full_path, self._limbo_name(trans_id))
-                    except OSError, e:
+                    except errors.TransformRenameFailed, e:
                         if e.errno != errno.ENOENT:
                             raise
                     else:
@@ -1666,7 +1688,7 @@
                 if trans_id in self._needs_rename:
                     try:
                         mover.rename(self._limbo_name(trans_id), full_path)
-                    except OSError, e:
+                    except errors.TransformRenameFailed, e:
                         # We may be renaming a dangling inventory id
                         if e.errno != errno.ENOENT:
                             raise
@@ -1798,9 +1820,12 @@
             executable = self.is_executable(file_id, path)
         return kind, executable, None
 
+    def is_locked(self):
+        return False
+
     def lock_read(self):
         # Perhaps in theory, this should lock the TreeTransform?
-        pass
+        return self
 
     def unlock(self):
         pass
@@ -2901,10 +2926,12 @@
         """Rename a file from one path to another."""
         try:
             osutils.rename(from_, to)
-        except OSError, e:
+        except (IOError, OSError), e:
             if e.errno in (errno.EEXIST, errno.ENOTEMPTY):
                 raise errors.FileExists(to, str(e))
-            raise
+            # normal OSError doesn't include filenames so it's hard to see where
+            # the problem is, see https://bugs.launchpad.net/bzr/+bug/491763
+            raise errors.TransformRenameFailed(from_, to, str(e), e.errno)
         self.past_renames.append((from_, to))
 
     def pre_delete(self, from_, to):
@@ -2920,7 +2947,10 @@
     def rollback(self):
         """Reverse all renames that have been performed"""
         for from_, to in reversed(self.past_renames):
-            osutils.rename(to, from_)
+            try:
+                osutils.rename(to, from_)
+            except (OSError, IOError), e:
+                raise errors.TransformRenameFailed(to, from_, str(e), e.errno)                
         # after rollback, don't reuse _FileMover
         past_renames = None
         pending_deletions = None

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2010-04-16 07:56:51 +0000
+++ bzrlib/transport/__init__.py	2010-05-04 06:22:51 +0000
@@ -539,6 +539,9 @@
 
         This function will only be defined for Transports which have a
         physical local filesystem representation.
+
+        :raises errors.NotLocalUrl: When no local path representation is
+            available.
         """
         raise errors.NotLocalUrl(self.abspath(relpath))
 

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2010-05-08 05:56:28 +0000
@@ -378,6 +378,11 @@
             port = conn_class.default_port
         self.proxied_host = '%s:%s' % (host, port)
         urllib2.Request.set_proxy(self, proxy, type)
+        # When urllib2 makes a https request with our wrapper code and a proxy,
+        # it sets Host to the https proxy, not the host we want to talk to.
+        # I'm fairly sure this is our fault, but what is the cause is an open
+        # question. -- Robert Collins May 8 2010.
+        self.add_unredirected_header('Host', self.proxied_host)
 
 
 class _ConnectRequest(Request):
@@ -711,7 +716,7 @@
             connect = _ConnectRequest(request)
             response = self.parent.open(connect)
             if response.code != 200:
-                raise ConnectionError("Can't connect to %s via proxy %s" % (
+                raise errors.ConnectionError("Can't connect to %s via proxy %s" % (
                         connect.proxied_host, self.host))
             # Housekeeping
             connection.cleanup_pipe()
@@ -868,21 +873,19 @@
                 print 'Will unbind %s_open for %r' % (type, proxy)
             delattr(self, '%s_open' % type)
 
+        def bind_scheme_request(proxy, scheme):
+            if proxy is None:
+                return
+            scheme_request = scheme + '_request'
+            if self._debuglevel >= 3:
+                print 'Will bind %s for %r' % (scheme_request, proxy)
+            setattr(self, scheme_request,
+                lambda request: self.set_proxy(request, scheme))
         # We are interested only by the http[s] proxies
         http_proxy = self.get_proxy_env_var('http')
+        bind_scheme_request(http_proxy, 'http')
         https_proxy = self.get_proxy_env_var('https')
-
-        if http_proxy is not None:
-            if self._debuglevel >= 3:
-                print 'Will bind http_request for %r' % http_proxy
-            setattr(self, 'http_request',
-                    lambda request: self.set_proxy(request, 'http'))
-
-        if https_proxy is not None:
-            if self._debuglevel >= 3:
-                print 'Will bind http_request for %r' % https_proxy
-            setattr(self, 'https_request',
-                    lambda request: self.set_proxy(request, 'https'))
+        bind_scheme_request(https_proxy, 'https')
 
     def get_proxy_env_var(self, name, default_to='all'):
         """Get a proxy env var.

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2010-04-28 07:53:34 +0000
+++ bzrlib/transport/local.py	2010-04-30 09:10:00 +0000
@@ -403,14 +403,11 @@
         try:
             # *don't* call bzrlib.osutils.rename, because we want to
             # detect conflicting names on rename, and osutils.rename tries to
-            # mask cross-platform differences there; however we do update the
-            # exception to include the filenames
+            # mask cross-platform differences there
             os.rename(path_from, path_to)
         except (IOError, OSError),e:
             # TODO: What about path_to?
-            self._translate_error(
-                osutils._add_rename_error_details(e, path_from, path_to),
-                path_from)
+            self._translate_error(e, path_from)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/tree.py	2010-05-11 08:36:16 +0000
@@ -582,6 +582,10 @@
             yield child.file_id
 
     def lock_read(self):
+        """Lock this tree for multiple read only operations.
+        
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         pass
 
     def revision_tree(self, revision_id):

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2010-03-25 07:34:15 +0000
+++ bzrlib/ui/text.py	2010-04-22 16:04:16 +0000
@@ -233,6 +233,9 @@
 
     def show_warning(self, msg):
         self.clear_term()
+        if isinstance(msg, unicode):
+            te = osutils.get_terminal_encoding()
+            msg = msg.encode(te, 'replace')
         self.stderr.write("bzr: warning: %s\n" % msg)
 
     def _progress_updated(self, task):

=== modified file 'bzrlib/util/_bencode_py.py'
--- bzrlib/util/_bencode_py.py	2009-12-01 19:44:12 +0000
+++ bzrlib/util/_bencode_py.py	2010-05-03 20:57:39 +0000
@@ -90,8 +90,9 @@
             raise TypeError
         try:
             r, l = self.decode_func[x[0]](x, 0)
-        except (IndexError, KeyError):
-            raise ValueError
+        except (IndexError, KeyError, OverflowError), e:
+            import sys
+            raise ValueError, ValueError(str(e)), sys.exc_info()[2]
         if l != len(x):
             raise ValueError
         return r

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2010-04-30 11:35:43 +0000
+++ bzrlib/workingtree.py	2010-05-17 10:04:05 +0000
@@ -77,6 +77,7 @@
 
 from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.lock import LogicalLockResult
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.lockdir import LockDir
 import bzrlib.mutabletree
@@ -462,7 +463,11 @@
         return (file_obj, stat_value)
 
     def get_file_text(self, file_id, path=None, filtered=True):
-        return self.get_file(file_id, path=path, filtered=filtered).read()
+        my_file = self.get_file(file_id, path=path, filtered=filtered)
+        try:
+            return my_file.read()
+        finally:
+            my_file.close()
 
     def get_file_byname(self, filename, filtered=True):
         path = self.abspath(filename)
@@ -1798,34 +1803,48 @@
             raise errors.ObjectNotLocked(self)
 
     def lock_read(self):
-        """See Branch.lock_read, and WorkingTree.unlock."""
+        """Lock the tree for reading.
+
+        This also locks the branch, and can be unlocked via self.unlock().
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_read()
         try:
-            return self._control_files.lock_read()
+            self._control_files.lock_read()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
 
     def lock_tree_write(self):
-        """See MutableTree.lock_tree_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_read()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
 
     def lock_write(self):
-        """See MutableTree.lock_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_write()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
@@ -1956,8 +1975,7 @@
         def recurse_directory_to_add_files(directory):
             # Recurse directory and add all files
             # so we can check if they have changed.
-            for parent_info, file_infos in\
-                self.walkdirs(directory):
+            for parent_info, file_infos in self.walkdirs(directory):
                 for relpath, basename, kind, lstat, fileid, kind in file_infos:
                     # Is it versioned or ignored?
                     if self.path2id(relpath) or self.is_ignored(relpath):
@@ -1998,8 +2016,10 @@
                             # ... but not ignored
                             has_changed_files = True
                             break
-                    elif content_change and (kind[1] is not None):
-                        # Versioned and changed, but not deleted
+                    elif (content_change and (kind[1] is not None) and
+                            osutils.is_inside_any(files, path[1])):
+                        # Versioned and changed, but not deleted, and still
+                        # in one of the dirs to be deleted.
                         has_changed_files = True
                         break
 
@@ -2636,10 +2656,14 @@
 
         In Format2 WorkingTrees we have a single lock for the branch and tree
         so lock_tree_write() degrades to lock_write().
+
+        :return: An object with an unlock method which will release the lock
+            obtained.
         """
         self.branch.lock_write()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return self
         except:
             self.branch.unlock()
             raise

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2010-04-30 11:03:59 +0000
+++ bzrlib/workingtree_4.py	2010-05-11 08:36:16 +0000
@@ -53,6 +53,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.filters import filtered_input_file, internal_size_sha_file_byname
 from bzrlib.inventory import Inventory, ROOT_ID, entry_factory
+from bzrlib.lock import LogicalLockResult
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import (
     file_kind,
@@ -567,7 +568,10 @@
             return _mod_revision.NULL_REVISION
 
     def lock_read(self):
-        """See Branch.lock_read, and WorkingTree.unlock."""
+        """See Branch.lock_read, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_read()
         try:
             self._control_files.lock_read()
@@ -586,6 +590,7 @@
         except:
             self.branch.unlock()
             raise
+        return LogicalLockResult(self.unlock)
 
     def _lock_self_write(self):
         """This should be called after the branch is locked."""
@@ -606,16 +611,23 @@
         except:
             self.branch.unlock()
             raise
+        return LogicalLockResult(self.unlock)
 
     def lock_tree_write(self):
-        """See MutableTree.lock_tree_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_read()
-        self._lock_self_write()
+        return self._lock_self_write()
 
     def lock_write(self):
-        """See MutableTree.lock_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_write()
-        self._lock_self_write()
+        return self._lock_self_write()
 
     @needs_tree_write_lock
     def move(self, from_paths, to_dir, after=False):
@@ -1859,6 +1871,9 @@
             return None
         return ie.executable
 
+    def is_locked(self):
+        return self._locked
+
     def list_files(self, include_root=False, from_dir=None, recursive=True):
         # We use a standard implementation, because DirStateRevisionTree is
         # dealing with one of the parents of the current state
@@ -1877,13 +1892,17 @@
             yield path, 'V', entry.kind, entry.file_id, entry
 
     def lock_read(self):
-        """Lock the tree for a set of operations."""
+        """Lock the tree for a set of operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self._locked:
             self._repository.lock_read()
             if self._dirstate._lock_token is None:
                 self._dirstate.lock_read()
                 self._dirstate_locked = True
         self._locked += 1
+        return LogicalLockResult(self.unlock)
 
     def _must_be_locked(self):
         if not self._locked:

=== added file 'contrib/bash/bzr'
--- contrib/bash/bzr	1970-01-01 00:00:00 +0000
+++ contrib/bash/bzr	2010-04-22 07:01:22 +0000
@@ -0,0 +1,40 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+# Programmable completion for the Bazaar-NG bzr command under bash.
+# Source this file (or add it to your ~/.bash_completion or ~/.bashrc
+# file, depending on your system configuration, and start a new shell)
+# and bash's completion mechanism will know all about bzr's options!
+#
+# This completion function assumes you have the bzr-bash-completion
+# plugin installed as a bzr plugin. It will generate the full
+# completion function at first invocation, thus avoiding long delays
+# for every shell you start.
+
+shopt -s progcomp
+_bzr_lazy ()
+{
+	unset _bzr
+	eval "$(bzr bash-completion)"
+	if [[ $(type -t _bzr) == function ]]; then
+		unset _bzr_lazy
+		_bzr
+		return $?
+	else
+		return 1
+	fi
+}
+complete -F _bzr_lazy -o default bzr

=== removed file 'contrib/bash/bzr'
--- contrib/bash/bzr	2005-09-19 06:05:19 +0000
+++ contrib/bash/bzr	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-# Programmable completion for the Bazaar-NG bzr command under bash. Source
-# this file (or on some systems add it to ~/.bash_completion and start a new
-# shell) and bash's completion mechanism will know all about bzr's options!
-
-# Known to work with bash 2.05a with programmable completion and extended
-# pattern matching enabled (use 'shopt -s extglob progcomp' to enable
-# these if they are not already enabled).
-
-# Based originally on the svn bash completition script.
-# Customized by Sven Wilhelm/Icecrash.com
-
-_bzr ()
-{
-	local cur cmds cmdOpts opt helpCmds optBase i
-
-	COMPREPLY=()
-	cur=${COMP_WORDS[COMP_CWORD]}
-
-	cmds='status diff commit ci checkin move remove log info check ignored'
-
-	if [[ $COMP_CWORD -eq 1 ]] ; then
-		COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
-		return 0
-	fi
-
-	# if not typing an option, or if the previous option required a
-	# parameter, then fallback on ordinary filename expansion
-	helpCmds='help|--help|h|\?'
-	if [[ ${COMP_WORDS[1]} != @($helpCmds) ]] && \
-	   [[ "$cur" != -* ]] ; then
-		return 0
-	fi
-
-	cmdOpts=
-	case ${COMP_WORDS[1]} in
-	status)
-		cmdOpts="--all --show-ids"
-		;;
-	diff)
-		cmdOpts="-r --revision --diff-options"
-		;;
-	commit|ci|checkin)
-		cmdOpts="-r --message -F --file -v --verbose"
-		;;
-	move)
-		cmdOpts=""
-		;;
-	remove)
-		cmdOpts="-v --verbose"
-		;;
-	log)
-		cmdOpts="--forward --timezone -v --verbose --show-ids -r --revision"
-		;;
-	info)
-		cmdOpts=""
-		;;
-	ignored)
-		cmdOpts=""
-		;;
-	check)
-		cmdOpts=""
-		;;
-	help|h|\?)
-		cmdOpts="$cmds $qOpts"
-		;;
-	*)
-		;;
-	esac
-
-	cmdOpts="$cmdOpts --help -h"
-
-	# take out options already given
-	for (( i=2; i<=$COMP_CWORD-1; ++i )) ; do
-		opt=${COMP_WORDS[$i]}
-
-		case $opt in
-		--*)    optBase=${opt/=*/} ;;
-		-*)     optBase=${opt:0:2} ;;
-		esac
-
-		cmdOpts=" $cmdOpts "
-		cmdOpts=${cmdOpts/ ${optBase} / }
-
-		# take out alternatives
-		case $optBase in
-		-v)              cmdOpts=${cmdOpts/ --verbose / } ;;
-		--verbose)       cmdOpts=${cmdOpts/ -v / } ;;
-		-h)              cmdOpts=${cmdOpts/ --help / } ;;
-		--help)          cmdOpts=${cmdOpts/ -h / } ;;
-		-r)              cmdOpts=${cmdOpts/ --revision / } ;;
-		--revision)      cmdOpts=${cmdOpts/ -r / } ;;
-		esac
-
-		# skip next option if this one requires a parameter
-		if [[ $opt == @($optsParam) ]] ; then
-			((++i))
-		fi
-	done
-
-	COMPREPLY=( $( compgen -W "$cmdOpts" -- $cur ) )
-
-	return 0
-}
-complete -F _bzr -o default bzr

=== removed file 'contrib/bash/bzr.simple'
--- contrib/bash/bzr.simple	2007-06-06 19:44:39 +0000
+++ contrib/bash/bzr.simple	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-# -*- shell-script -*-
-
-# experimental bzr bash completion
-
-# author: Martin Pool
-
-_bzr_commands() 
-{
-     bzr help commands | sed -r 's/^([-[:alnum:]]*).*/\1/' | grep '^[[:alnum:]]' 
-}
-
-_bzr() 
-{
-    cur=${COMP_WORDS[COMP_CWORD]}
-    prev=${COMP_WORDS[COMP_CWORD-1]}
-    if [ $COMP_CWORD -eq 1 ]; then
-        COMPREPLY=( $( compgen -W "$(_bzr_commands)" $cur ) )
-    elif [ $COMP_CWORD -eq 2 ]; then
-        case "$prev" in 
-        help)
-            COMPREPLY=( $( compgen -W "$(_bzr_commands) commands" $cur ) )
-            ;;
-        esac
-    fi 
-}
-
-complete -F _bzr -o default bzr
-

=== added file 'contrib/zsh/README'
--- contrib/zsh/README	1970-01-01 00:00:00 +0000
+++ contrib/zsh/README	2010-05-25 11:34:35 +0000
@@ -0,0 +1,19 @@
+This directory used to contain a shell completion function for zsh.
+That script has been unmaintained since 2005, though.  Zsh itself does
+ship a _bzr completion function, and has done so for quite some time
+now.  So we suggest you use the version installed with your zsh
+instead of the version that used to be located in this directory.
+
+Note that the shell-complete builtin of bzr does provide completion
+information in a style suitable for zsh.  There are some completion
+scripts for zsh in the wild that make use of this feature, although
+none of these scripts is officially supported by the bzr developers.
+
+Also note that recent versions of bzr (starting with the 2.3 series)
+do include a bash_completion plugin. It should be possible to improve
+this in such a way that it generates code for zsh instead of bash.  So
+if you want better zsh completion, this might be the place to start.
+
+References:
+https://bugs.launchpad.net/bzr/+bug/560030
+http://zsh.git.sourceforge.net/git/gitweb.cgi?p=zsh/zsh;a=history;f=Completion/Unix/Command/_bzr

=== removed file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-11-18 08:10:07 +0000
+++ contrib/zsh/_bzr	1970-01-01 00:00:00 +0000
@@ -1,316 +0,0 @@
-#compdef bzr
-
-# Rudimentary zsh completion support for bzr.
-
-# -S means there are no options after a -- and that argument is ignored
-
-# To use this you must arrange for it to be in a directory that is on
-# your $fpath, and also for compinit to be run.  I don't understand
-# how to get zsh to reload this file when it changes, other than by
-# starting a new zsh.
-
-local _bzr_subcommands expl curcontext="$curcontext" state line
-local bzrRoot helpArgs parentFile
-typeset -A opt_args
-
-helpArgs=( '(--help)-h[show help message]' '(-h)--help[show help message]' )
-bzrRoot=$(bzr root 2> /dev/null)
-parentFile=$bzrRoot/.bzr/parent
-
-if [[ $service == "bzr" ]]; then
-    _arguments -C -A "-*" \
-    '*::command:->subcmd' && return 0
-
-    if (( CURRENT == 1 )); then
-      _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
-      _describe -t subcommand 'subcommand' _bzr_subcommands
-      return
-    fi
-
-    service="$words[1]"
-    curcontext="${curcontext%:*}=$service:"
-fi
-
-_unknownFiles() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --unknown)"})
-    compadd -af fileList
-    return 0
-}
-
-_unknownRoot() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --from-root --unknown)"})
-    compadd -af fileList
-    return 0
-}
-
-_versionedFiles() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --versioned)"})
-    compadd -af fileList
-    return 0
-}
-
-case $service in
-    (add)
-       _arguments $helpArgs \
-       '--no-recurse[do not recurse into subdirectories]' \
-       '(-q)--quiet' \
-       '(--quiet)-q' \
-       '*:unknown files:_unknownFiles'
-    ;;
-
-    (annotate|praise|blame)
-        _arguments $helpArgs '*:files:_versionedFiles'
-    ;;
-
-    (clone|branch|get)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs \
-            '(-r)--revision:rev:' \
-            '(--revision)-r:rev:' \
-            '--basis[specify basis branch]:basis:' \
-            '*:FROM_LOCATION:_files -/'
-        elif (( CURRENT == 3 )); then
-            _arguments $helpArgs \
-            '(-r)--revision:rev:' \
-            '(--revision)-r:rev:' \
-            '--basis[specify basis branch]:basis:' \
-            '*:TO_LOCATION:_files -/'
-        fi
-    ;;
-
-    (rename|mv)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs '*:old name:_versionedFiles'
-        else
-            _arguments $helpArgs '*:new name:'
-        fi
-    ;;
-
-    (cat)
-        _arguments $helpArgs \
-        '(-r)--revision:rev:' \
-        '(--revision)-r:rev:' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (root)
-        _arguments $helpArgs '*:file:_files'
-    ;;
-
-    (log)
-        _arguments $helpArgs \
-        '(-r)--revision[revision or range]:rev or rev range:' \
-        '(--revision)-r[revision or range]:rev or rev range:' \
-        '(--verbose)-v[show revision manifest]' \
-        '(-v)--verbose[show revision manifest]' \
-        '(--short --long)-l[use long format (default)]' \
-        '(--short -l)--long[use long format (default)]' \
-        '(-l --long)--short[use short format]' \
-        '(--message)-m[specify regexp]:regexp:' \
-        '(-m)--message[specify regexp]:regexp:' \
-        '--show-ids[show file IDs]' \
-        '--forward[reverse direction of revisions]' \
-        '--timezoe[specify timezone for dates]:timezone:' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (resolve)
-        _arguments $helpArgs '--all' '*:file:_versionedFiles'
-    ;;
-
-    (status|st|stat)
-        _arguments $helpArgs \
-        '--all[include unchanged versioned files]' \
-        '--show-ids[show file IDs]' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (check)
-        _arguments $helpArgs \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:DIR:_files -/'
-    ;;
-
-    (mkdir|upgrade|renames)
-        _arguments $helpArgs '*:DIR:_files -/'
-    ;;
-
-    (remove|rm)
-        _arguments $helpArgs \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (pull)
-        _arguments $helpArgs \
-        '--remember[Remember the specified location as a default]' \
-        '--overwrite[Ignore differences, overwrite unconditionally]' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (missing|miss|mis)
-        _arguments $helpArgs \
-        '(--verbose -q --quiet)-v' \
-        '(-v -q --quiet)--verbose' \
-        '(--verbose -v --quiet)-q' \
-        '(-v -q --verbose)--quiet' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (commit|checkin|ci)
-        _arguments $helpArgs \
-        '(--message)-m[commit message]:message text:' \
-        '(-m)--message[commit message]:message text:' \
-        '(--file)-F[commit message from file]:message file:' \
-        '(-F)--file[commit message from file]:message file:' \
-        '--unchanged[include unchanged files]' \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:modified files:_versionedFiles'
-    ;;
-
-    (conflicts|added|deleted|modified|unknowns|directories|ignored)
-        _arguments $helpArgs
-    ;;
-
-    (revno|init|version)
-        _arguments $helpArgs
-    ;;
-
-    (whoami)
-        _arguments $helpArgs '--email[only show e-mail address]' 
-    ;;
-
-    (inventory)
-        _arguments $helpArgs \
-        '(-r)--revision[show inventory of a revision]:revision:' \
-        '(--revision)-r[show inventory of a revision]:revision:' \
-        '--show-ids[show file IDs]'
-    ;;
-
-    (diff|dif|di)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--diff-options[options to pass to gdiff]:diff options:' \
-        '*:files:_files'
-    ;;
-
-    (export)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--format=[format of exported file]:format:(dir tar tgz tbz2)' \
-        '--root[root directory of patch]:_files -/' \
-        '*:destination:_files'
-    ;;
-
-    (ignore)
-        _arguments $helpArgs '*:NAME_PATTERN:_unknownRoot'
-    ;;
-
-    (info)
-        _arguments $helpArgs '*:branch:_files -/'
-    ;;
-
-    (testament)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '(--long)-l' \
-        '(-l)--long' \
-        '*:branch:_files -/'
-    ;;
-
-    (revert|merge-revert)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--no-backup[skip generation of backup~ files]' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (merge)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--reprocess[Reprocess to reduce spurious conflicts]' \
-        '--show-base[Show base revision text in conflicts]' \
-        '--force[ignore uncommitted changes]' \
-        '--merge-type:merge type:(diff3 merge3 weave)' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (ls)
-        _arguments $helpArgs \
-        '(--verbose)-v[display more information]' \
-        '(-v)--verbose[display more information]' \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--from-root[Print all paths from the root of the branch]' \
-        '--non-recursive[Do not recurse into subdirectories]' \
-        '--null[Null separate the files]' \
-        '--ignored[Print ignored files]' \
-        '--unknown[Print unknown files]' \
-        '--versioned[Print versioned files]'
-    ;;
-
-    (move)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs '*:files:_versionedFiles'
-        else
-            _arguments '*:destination dir:_files -/'
-        fi
-    ;;
-
-    (help)
-        _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
-        _arguments $helpArgs \
-        '(--long)-l' \
-        '(-l)--long' \
-        '*:subcmds:->cmds'
-        _describe -t subcommand 'subcommand' _bzr_subcommands
-    ;;
-
-    # Plugins
-
-    (visualize|visualise|viz|vis)
-        _arguments $helpArgs \
-            '(-r)--revision[starting revision]:rev:' \
-            '(--revision)-r[starting revision]:rev:'
-    ;;
-
-    (gannotate|gblame|gpraise)
-        _arguments $helpArgs \
-            '--all[show annotations on all lines]' \
-            '--plain[do not hightlight annotation lines]' \
-            '*:files:_versionedFiles'
-    ;;
-
-    (push)
-        _arguments $helpArgs \
-        '--overwrite[Ignore differences, overwrite unconditionally]' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (clean-tree)
-        _arguments $helpArgs \
-        '--dry-run[show files to delete instead of deleting them]' \
-        '--ignored[delete all ignored files]' \
-        '--detritus[delete conflict files, merge backups, failed self-tests, *~, *.tmp, etc]'
-    ;;
-
-
-    (*)
-        _message "unknown bzr command completion: $service"
-    ;;
-esac

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2010-04-16 12:59:03 +0000
+++ doc/developers/HACKING.txt	2010-05-12 11:04:21 +0000
@@ -179,11 +179,9 @@
 
 * Bazaar - http://bazaar-vcs.org/
 
-* Bundle Buggy - http://bundlebuggy.aaronbentley.com/
-
 * Patch Queue Manager - https://launchpad.net/pqm/
 
-For further information, see http://bazaar-vcs.org/BzrDevelopment.
+For further information, see <http://wiki.bazaar.canonical.com/BzrDevelopment>.
 
 
 
@@ -272,239 +270,6 @@
 <http://doc.bazaar-vcs.org/developers/overview.html>`_.
 
 
-The Code Review Process
-#######################
-
-All code changes coming in to Bazaar are reviewed by someone else.
-Normally changes by core contributors are reviewed by one other core
-developer, and changes from other people are reviewed by two core
-developers.  Use intelligent discretion if the patch is trivial.
-
-Good reviews do take time. They also regularly require a solid
-understanding of the overall code base. In practice, this means a small
-number of people often have a large review burden - with knowledge comes
-responsibility. No one likes their merge requests sitting in a queue going
-nowhere, so reviewing sooner rather than later is strongly encouraged.
-
-
-
-
-
-Review cover letters
-====================
-
-Please put a "cover letter" on your merge request explaining:
-
-* the reason **why** you're making this change
-
-* **how** this change achieves this purpose
-
-* anything else you may have fixed in passing
-
-* anything significant that you thought of doing, such as a more
-  extensive fix or a different approach, but didn't or couldn't do now
-
-A good cover letter makes reviewers' lives easier because they can decide
-from the letter whether they agree with the purpose and approach, and then
-assess whether the patch actually does what the cover letter says.
-Explaining any "drive-by fixes" or roads not taken may also avoid queries
-from the reviewer.  All in all this should give faster and better reviews.
-Sometimes writing the cover letter helps the submitter realize something
-else they need to do.  The size of the cover letter should be proportional
-to the size and complexity of the patch.
-
-
-Reviewing proposed changes
-==========================
-
-Anyone is welcome to review code, and reply to the thread with their
-opinion or comments.
-
-The simplest way to review a proposed change is to just read the patch on
-the list or in Bundle Buggy.  For more complex changes it may be useful
-to make a new working tree or branch from trunk, and merge the proposed
-change into it, so you can experiment with the code or look at a wider
-context.
-
-There are three main requirements for code to get in:
-
-* Doesn't reduce test coverage: if it adds new methods or commands,
-  there should be tests for them.  There is a good test framework
-  and plenty of examples to crib from, but if you are having trouble
-  working out how to test something feel free to post a draft patch
-  and ask for help.
-
-* Doesn't reduce design clarity, such as by entangling objects
-  we're trying to separate.  This is mostly something the more
-  experienced reviewers need to help check.
-
-* Improves bugs, features, speed, or code simplicity.
-
-Code that goes in should not degrade any of these aspects.  Patches are
-welcome that only cleanup the code without changing the external
-behaviour.  The core developers take care to keep the code quality high
-and understandable while recognising that perfect is sometimes the enemy
-of good.
-
-It is easy for reviews to make people notice other things which should be
-fixed but those things should not hold up the original fix being accepted.
-New things can easily be recorded in the Bug Tracker instead.
-
-It's normally much easier to review several smaller patches than one large
-one.  You might want to use ``bzr-loom`` to maintain threads of related
-work, or submit a preparatory patch that will make your "real" change
-easier.
-
-
-Checklist for reviewers
-=======================
-
-* Do you understand what the code's doing and why?
-
-* Will it perform reasonably for large inputs, both in memory size and
-  run time?  Are there some scenarios where performance should be
-  measured?
-
-* Is it tested, and are the tests at the right level?  Are there both
-  blackbox (command-line level) and API-oriented tests?
-
-* If this change will be visible to end users or API users, is it
-  appropriately documented in NEWS?
-
-* Does it meet the coding standards below?
-
-* If it changes the user-visible behaviour, does it update the help
-  strings and user documentation?
-
-* If it adds a new major concept or standard practice, does it update the
-  developer documentation?
-
-* (your ideas here...)
-
-
-Reviews on Launchpad
-====================
-
-From May 2009 on, we prefer people to propose code reviews through
-Launchpad.
-
- * <https://launchpad.net/+tour/code-review>
-
- * <https://help.launchpad.net/Code/Review>
-
-Anyone can propose or comment on a merge proposal just by creating a
-Launchpad account.
-
-There are two ways to create a new merge proposal: through the web
-interface or by email.
-
-
-Proposing a merge through the web
----------------------------------
-
-To create the proposal through the web, first push your branch to Launchpad.
-For example, a branch dealing with documentation belonging to the Launchpad
-User mbp could be pushed as ::
-
-  bzr push lp:~mbp/bzr/doc
-
-Then go to the branch's web page, which in this case would be
-<https://code.launchpad.net/~mbp/bzr/doc>.  You can simplify this step by just
-running ::
-
-  bzr lp-open
-
-You can then click "Propose for merging into another branch", and enter your
-cover letter (see above) into the web form.  Typically you'll want to merge
-into ``~bzr/bzr/trunk`` which will be the default; you might also want to
-nominate merging into a release branch for a bug fix.  There is the option to
-specify a specific reviewer or type of review, and you shouldn't normally
-change those.
-
-Submitting the form takes you to the new page about the merge proposal
-containing the diff of the changes, comments by interested people, and
-controls to comment or vote on the change.
-
-Proposing a merge by mail
--------------------------
-
-To propose a merge by mail, send a bundle to ``merge@code.launchpad.net``.
-
-You can generate a merge request like this::
-
-  bzr send -o bug-1234.diff
-
-``bzr send`` can also send mail directly if you prefer; see the help.
-
-Reviewing changes
------------------
-
-From <https://code.launchpad.net/bzr/+activereviews> you can see all
-currently active reviews, and choose one to comment on.  This page also
-shows proposals that are now approved and should be merged by someone with
-PQM access.
-
-
-Reviews through Bundle Buggy
-============================
-
-The Bundle Buggy tool used up to May 2009 is still available as a review
-mechanism.
-
-Sending patches for review
---------------------------
-
-If you'd like to propose a change, please post to the
-bazaar@lists.canonical.com list with a bundle, patch, or link to a
-branch. Put ``[PATCH]`` or ``[MERGE]`` in the subject so Bundle Buggy
-can pick it out, and explain the change in the email message text.
-Remember to update the NEWS file as part of your change if it makes any
-changes visible to users or plugin developers. Please include a diff
-against mainline if you're giving a link to a branch.
-
-You can generate a merge request like this::
-
-  bzr send -o bug-1234.patch
-
-A ``.patch`` extension is recommended instead of .bundle as many mail clients
-will send the latter as a binary file.
-
-``bzr send`` can also send mail directly if you prefer; see the help.
-
-Please do **NOT** put [PATCH] or [MERGE] in the subject line if you don't
-want it to be merged. If you want comments from developers rather than
-to be merged, you can put ``[RFC]`` in the subject line.
-
-If this change addresses a bug, please put the bug number in the subject
-line too, in the form ``[#1]`` so that Bundle Buggy can recognize it.
-
-If the change is intended for a particular release mark that in the
-subject too, e.g. ``[1.6]``.
-Anyone can "vote" on the mailing list by expressing an opinion. Core
-developers can also vote using Bundle Buggy. Here are the voting codes and
-their explanations.
-
-:approve:  Reviewer wants this submission merged.
-:tweak:    Reviewer wants this submission merged with small changes. (No
-  re-review required.)
-:abstain:  Reviewer does not intend to vote on this patch.
-:resubmit: Please make changes and resubmit for review.
-:reject:   Reviewer doesn't want this kind of change merged.
-:comment:  Not really a vote. Reviewer just wants to comment, for now.
-
-If a change gets two approvals from core reviewers, and no rejections,
-then it's OK to come in.  Any of the core developers can bring it into the
-bzr.dev trunk and backport it to maintenance branches if required.  The
-Release Manager will merge the change into the branch for a pending
-release, if any. As a guideline, core developers usually merge their own
-changes and volunteer to merge other contributions if they were the second
-reviewer to agree to a change.
-
-To track the progress of proposed changes, use Bundle Buggy. See
-http://bundlebuggy.aaronbentley.com/help for a link to all the
-outstanding merge requests together with an explanation of the columns.
-Bundle Buggy will also mail you a link to track just your change.
-
 Coding Style Guidelines
 #######################
 

=== added file 'doc/developers/code-review.txt'
--- doc/developers/code-review.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/code-review.txt	2010-05-12 11:04:21 +0000
@@ -0,0 +1,170 @@
+The Code Review Process
+#######################
+
+All code changes coming in to Bazaar are reviewed by someone else.
+Normally changes by core contributors are reviewed by one other core
+developer, and changes from other people are reviewed by two core
+developers.  Use intelligent discretion if the patch is trivial.
+
+Good reviews do take time. They also regularly require a solid
+understanding of the overall code base. In practice, this means a small
+number of people often have a large review burden - with knowledge comes
+responsibility. No one likes their merge requests sitting in a queue going
+nowhere, so reviewing sooner rather than later is strongly encouraged.
+
+
+
+Review cover letters
+====================
+
+Please put a "cover letter" on your merge request explaining:
+
+* the reason **why** you're making this change
+
+* **how** this change achieves this purpose
+
+* anything else you may have fixed in passing
+
+* anything significant that you thought of doing, such as a more
+  extensive fix or a different approach, but didn't or couldn't do now
+
+A good cover letter makes reviewers' lives easier because they can decide
+from the letter whether they agree with the purpose and approach, and then
+assess whether the patch actually does what the cover letter says.
+Explaining any "drive-by fixes" or roads not taken may also avoid queries
+from the reviewer.  All in all this should give faster and better reviews.
+Sometimes writing the cover letter helps the submitter realize something
+else they need to do.  The size of the cover letter should be proportional
+to the size and complexity of the patch.
+
+
+Reviewing proposed changes
+==========================
+
+Anyone is welcome to review code, and reply to the thread with their
+opinion or comments.
+
+The simplest way to review a proposed change is to just read the patch on
+the list or in Bundle Buggy.  For more complex changes it may be useful
+to make a new working tree or branch from trunk, and merge the proposed
+change into it, so you can experiment with the code or look at a wider
+context.
+
+There are three main requirements for code to get in:
+
+* Doesn't reduce test coverage: if it adds new methods or commands,
+  there should be tests for them.  There is a good test framework
+  and plenty of examples to crib from, but if you are having trouble
+  working out how to test something feel free to post a draft patch
+  and ask for help.
+
+* Doesn't reduce design clarity, such as by entangling objects
+  we're trying to separate.  This is mostly something the more
+  experienced reviewers need to help check.
+
+* Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should not degrade any of these aspects.  Patches are
+welcome that only cleanup the code without changing the external
+behaviour.  The core developers take care to keep the code quality high
+and understandable while recognising that perfect is sometimes the enemy
+of good.
+
+It is easy for reviews to make people notice other things which should be
+fixed but those things should not hold up the original fix being accepted.
+New things can easily be recorded in the Bug Tracker instead.
+
+It's normally much easier to review several smaller patches than one large
+one.  You might want to use ``bzr-loom`` to maintain threads of related
+work, or submit a preparatory patch that will make your "real" change
+easier.
+
+
+Checklist for reviewers
+=======================
+
+* Do you understand what the code's doing and why?
+
+* Will it perform reasonably for large inputs, both in memory size and
+  run time?  Are there some scenarios where performance should be
+  measured?
+
+* Is it tested, and are the tests at the right level?  Are there both
+  blackbox (command-line level) and API-oriented tests?
+
+* If this change will be visible to end users or API users, is it
+  appropriately documented in NEWS?
+
+* Does it meet the coding standards below?
+
+* If it changes the user-visible behaviour, does it update the help
+  strings and user documentation?
+
+* If it adds a new major concept or standard practice, does it update the
+  developer documentation?
+
+* (your ideas here...)
+
+
+Reviews on Launchpad
+====================
+
+From May 2009 on, we prefer people to propose code reviews through
+Launchpad.
+
+ * <https://launchpad.net/+tour/code-review>
+
+ * <https://help.launchpad.net/Code/Review>
+
+Anyone can propose or comment on a merge proposal just by creating a
+Launchpad account.
+
+There are two ways to create a new merge proposal: through the web
+interface or by email.
+
+
+Proposing a merge through the web
+---------------------------------
+
+To create the proposal through the web, first push your branch to Launchpad.
+For example, a branch dealing with documentation belonging to the Launchpad
+User mbp could be pushed as ::
+
+  bzr push lp:~mbp/bzr/doc
+
+Then go to the branch's web page, which in this case would be
+<https://code.launchpad.net/~mbp/bzr/doc>.  You can simplify this step by just
+running ::
+
+  bzr lp-open
+
+You can then click "Propose for merging into another branch", and enter your
+cover letter (see above) into the web form.  Typically you'll want to merge
+into ``~bzr/bzr/trunk`` which will be the default; you might also want to
+nominate merging into a release branch for a bug fix.  There is the option to
+specify a specific reviewer or type of review, and you shouldn't normally
+change those.
+
+Submitting the form takes you to the new page about the merge proposal
+containing the diff of the changes, comments by interested people, and
+controls to comment or vote on the change.
+
+Proposing a merge by mail
+-------------------------
+
+To propose a merge by mail, send a bundle to ``merge@code.launchpad.net``.
+
+You can generate a merge request like this::
+
+  bzr send -o bug-1234.diff
+
+``bzr send`` can also send mail directly if you prefer; see the help.
+
+Reviewing changes
+-----------------
+
+From <https://code.launchpad.net/bzr/+activereviews> you can see all
+currently active reviews, and choose one to comment on.  This page also
+shows proposals that are now approved and should be merged by someone with
+PQM access.
+

=== modified file 'doc/developers/index-plain.txt'
--- doc/developers/index-plain.txt	2010-02-23 04:12:26 +0000
+++ doc/developers/index-plain.txt	2010-05-12 11:04:21 +0000
@@ -24,6 +24,8 @@
 
 * `Testing <testing.html>`_ |--| Guide to writing tests for Bazaar.
 
+* `Code Review <code-review.html>`_.
+
 * `Writing plugins <http://doc.bazaar.canonical.com/plugins/en/plugin-development.html>`_
   |--| specific advice on writing Bazaar plugins. (web link)
 

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2010-02-23 04:12:26 +0000
+++ doc/developers/index.txt	2010-05-12 11:04:21 +0000
@@ -23,6 +23,7 @@
    bug-handling
    HACKING
    testing
+   code-review
 
 * `Contributing to Bazaar Documentation <http://wiki.bazaar.canonical.com/ContributingToTheDocs>`_ (wiki)
 

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2010-04-16 07:56:51 +0000
+++ doc/developers/testing.txt	2010-05-12 10:51:28 +0000
@@ -6,7 +6,7 @@
 The Importance of Testing
 =========================
 
-Reliability is a critical success factor for any Version Control System.
+Reliability is a critical success factor for any version control system.
 We want Bazaar to be highly reliable across multiple platforms while
 evolving over time to meet the needs of its community.
 
@@ -183,6 +183,16 @@
 
 .. _testrepository: https://launchpad.net/testrepository
 
+
+Babune continuous integration
+-----------------------------
+
+We have a Hudson continuous-integration system that automatically runs 
+tests across various platforms.  In the future we plan to add more 
+combinations including testing plugins.  See 
+<http://babune.ladeuil.net:24842/>.  (Babune = Bazaar Buildbot Network.)
+
+
 Writing Tests
 =============
 
@@ -772,12 +782,15 @@
 
 TestCase
     A base TestCase that extends the Python standard library's
-    TestCase in several ways.  It adds more assertion methods (e.g.
-    ``assertContainsRe``), ``addCleanup``, and other features (see its API
-    docs for details).  It also has a ``setUp`` that makes sure that
-    global state like registered hooks and loggers won't interfere with
-    your test.  All tests should use this base class (whether directly or
-    via a subclass).
+    TestCase in several ways.  TestCase is build on
+    ``testtools.TestCase``, which gives it support for more assertion
+    methods (e.g.  ``assertContainsRe``), ``addCleanup``, and other
+    features (see its API docs for details).  It also has a ``setUp`` that
+    makes sure that global state like registered hooks and loggers won't
+    interfere with your test.  All tests should use this base class
+    (whether directly or via a subclass).  Note that we are trying not to
+    add more assertions at this point, and instead to build up a library
+    of ``bzrlib.tests.matchers``.
 
 TestCaseWithMemoryTransport
     Extends TestCase and adds methods like ``get_transport``,

=== modified file 'setup.py'
--- setup.py	2010-04-08 04:34:03 +0000
+++ setup.py	2010-04-19 21:04:31 +0000
@@ -281,7 +281,6 @@
     add_pyrex_extension('bzrlib._dirstate_helpers_pyx',
                         libraries=['Ws2_32'])
     add_pyrex_extension('bzrlib._walkdirs_win32')
-    z_lib = 'zdll'
 else:
     if have_pyrex and pyrex_version_info[:3] == (0,9,4):
         # Pyrex 0.9.4.1 fails to compile this extension correctly
@@ -298,8 +297,7 @@
     else:
         add_pyrex_extension('bzrlib._dirstate_helpers_pyx')
     add_pyrex_extension('bzrlib._readdir_pyx')
-    z_lib = 'z'
-add_pyrex_extension('bzrlib._chk_map_pyx', libraries=[z_lib])
+add_pyrex_extension('bzrlib._chk_map_pyx')
 ext_modules.append(Extension('bzrlib._patiencediff_c',
                              ['bzrlib/_patiencediff_c.c']))
 if have_pyrex and pyrex_version_info < (0, 9, 6, 3):

